"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const core_1 = require("@ts-ioc/core");
/**
 * annotation build strategy token inject.
 *
 * @export
 * @class InjectAnnoBuildStrategyToken
 * @extends {RefRegistration<IAnnoBuildStrategy<T>>}
 * @template T
 */
class InjectAnnoBuildStrategyToken extends core_1.RefRegistration {
    constructor(type) {
        super(type, 'AnnoBuildStrategy');
    }
    static getClassAnnations() {
        return { "name": "InjectAnnoBuildStrategyToken", "params": { "constructor": ["type"] } };
    }
}
exports.InjectAnnoBuildStrategyToken = InjectAnnoBuildStrategyToken;
/**
 *  base AnnoBuildStrategy Token
 */
exports.AnnoBuildStrategyToken = new InjectAnnoBuildStrategyToken(Object);
/**
 * annotation instance build strategy.
 *
 * @export
 * @class AnnoBuildStrategy
 * @implements {IAnnoBuildStrategy<T>}
 * @template T
 */
let AnnoBuildStrategy = class AnnoBuildStrategy {
    build(instance, config, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (!instance) {
                return;
            }
            yield this.beforeInit(instance, config);
            yield this.process(instance, config, options);
            yield this.afterInit(instance, config);
        });
    }
    /**
     * after annotation instance init.
     *
     * @protected
     * @param {BootHooks<T>} instance
     * @param {AnnotationConfigure<T>} config
     * @returns {Promise<void>}
     * @memberof AnnoBuildStrategy
     */
    beforeInit(instance, config) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (core_1.isFunction(instance.anBeforeInit)) {
                yield instance.anBeforeInit(config);
            }
        });
    }
    /**
     * build annotation instance process.
     *
     * @protected
     * @param {T} instance
     * @param {AnnotationConfigure<T>} config
     * @param {BuildOptions<T>} [options]
     * @returns {Promise<void>}
     * @memberof AnnoBuildStrategy
     */
    process(instance, config, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
        });
    }
    /**
     * after annotation instance init.
     *
     * @protected
     * @param {BootHooks<T>} instance
     * @param {AnnotationConfigure<T>} config
     * @returns {Promise<void>}
     * @memberof AnnoBuildStrategy
     */
    afterInit(instance, config) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (core_1.isFunction(instance.anAfterInit)) {
                yield instance.anAfterInit(config);
            }
        });
    }
    static getClassAnnations() {
        return { "name": "AnnoBuildStrategy", "params": { "build": ["instance", "config", "options"], "beforeInit": ["instance", "config"], "process": ["instance", "config", "options"], "afterInit": ["instance", "config"] } };
    }
};
AnnoBuildStrategy = tslib_1.__decorate([
    core_1.Singleton(exports.AnnoBuildStrategyToken)
], AnnoBuildStrategy);
exports.AnnoBuildStrategy = AnnoBuildStrategy;

//# sourceMappingURL=../sourcemaps/annotations/AnnoBuildStrategy.js.map
