"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const runnable_1 = require("../runnable");
/**
 * boot
 *
 * @export
 * @abstract
 * @class Boot
 * @extends {Service<T>}
 * @implements {IBoot<T>}
 * @template T
 */
class Boot extends runnable_1.Service {
    /**
     * on boot init.
     *
     * @param {RunnableOptions<T>} options
     * @param {RunOptions<T>} bootOptions
     * @returns {Promise<void>}
     * @memberof IBoot
     */
    onInit(options, bootOptions) {
        const _super = Object.create(null, {
            onInit: { get: () => super.onInit }
        });
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield _super.onInit.call(this, options, bootOptions);
        });
    }
    /**
     * run boot.
     *
     * @abstract
     * @param {*} [data]
     * @returns {Promise<any>}
     * @memberof Boot
     */
    run(data) {
        return this.start(data);
    }
    static getClassAnnations() {
        return { "name": "Boot", "params": { "onInit": ["options", "bootOptions"], "run": ["data"] } };
    }
}
exports.Boot = Boot;

//# sourceMappingURL=../sourcemaps/boot/boots.js.map
