"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const core_1 = require("@ts-ioc/core");
const DIModuleValidate_1 = require("./DIModuleValidate");
const utils_1 = require("../utils");
const InjectedModule_1 = require("./InjectedModule");
const ConfigureRegister_1 = require("../boot/ConfigureRegister");
const IConfigureManager_1 = require("../boot/IConfigureManager");
const IRunnableBuilder_1 = require("../boot/IRunnableBuilder");
/**
 * DIModule injector token.
 */
exports.DIModuleInjectorToken = new core_1.InjectModuleInjectorToken('@DIModule');
/**
 * DIModule injector.
 *
 * @export
 * @class DIModuleInjector
 * @extends {ModuleInjector}
 */
let DIModuleInjector = class DIModuleInjector extends core_1.ModuleInjector {
    constructor(validate) {
        super(validate);
    }
    setup(container, type) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield this.importModule(container, type);
        });
    }
    syncSetup(container, type) {
        throw new Error('DIModule can not sync setup.');
    }
    valid(container, type) {
        if (!this.validate) {
            return true;
        }
        return this.validate.valid(type);
    }
    getMetaAccessor(container, token) {
        return container.getService(core_1.MetaAccessorToken, token);
    }
    importModule(container, type) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let pools = container.get(utils_1.ContainerPoolToken);
            let decorator = this.validate.getDecorator();
            let accor = this.getMetaAccessor(container, decorator);
            let metaConfig = accor.getMetadata(type, container, undefined, decorator ? dec => dec === decorator : undefined);
            let newContainer = metaConfig.asRoot === true ? pools.getDefault() : pools.create(type, container);
            newContainer.register(type);
            yield this.registerConfgureDepds(newContainer, metaConfig, type);
            let exps = yield this.getConfigExports(newContainer, metaConfig);
            let injMd = new InjectedModule_1.InjectedModule(metaConfig.token || type, metaConfig, newContainer, type, exps);
            container.bindProvider(new core_1.InjectReference(core_1.ProviderMap, type), exps);
            container.bindProvider(new InjectedModule_1.InjectedModuleToken(type), injMd);
            if (metaConfig.asRoot) {
                return injMd;
            }
            yield this.registerConfigExports(container, newContainer, injMd);
            // init global configure.
            if (!pools.isDefault(newContainer)) {
                yield this.registerConfigrue(newContainer, injMd);
            }
            return injMd;
        });
    }
    registerConfgureDepds(container, config, type) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (core_1.isArray(config.imports) && config.imports.length) {
                yield container.loadModule(...config.imports);
            }
        });
    }
    getConfigExports(container, config) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let parser = container.getProviderParser();
            let map = parser.parse(...config.providers || []);
            // bind module providers
            container.bindProviders(map);
            let builder = container.getBuilder();
            let exptypes = [].concat(...builder.loader.getTypes(config.exports || []));
            exptypes.forEach(ty => {
                let classPd = container.resolveValue(new core_1.InjectClassProvidesToken(ty));
                map.add(ty, (...pds) => container.resolve(ty, core_1.ResoveWay.nodes, ...pds));
                if (classPd && core_1.isArray(classPd.provides) && classPd.provides.length) {
                    classPd.provides.forEach(p => {
                        if (!map.has(p)) {
                            map.add(p, (...pds) => container.resolve(p, core_1.ResoveWay.nodes, ...pds));
                        }
                    });
                }
            });
            return map;
        });
    }
    registerConfigExports(container, newContainer, injMd) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (container === newContainer) {
                return container;
            }
            if (injMd) {
                let chain = container.getResolvers();
                chain.next(injMd);
            }
            return container;
        });
    }
    registerConfigrue(newContainer, injMd) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let registers = [];
            newContainer.iteratorServices((serType, fac, resolver) => {
                registers.push({
                    resolver: resolver,
                    serType: serType
                });
            }, ConfigureRegister_1.ConfigureRegister, injMd.type, true, core_1.ResoveWay.current);
            if (registers.length) {
                let mgr = newContainer.get(IConfigureManager_1.ConfigureMgrToken);
                let appConfig = yield mgr.getConfig();
                yield Promise.all(registers.map(ser => ser.resolver.resolve(ser.serType).register(appConfig, ser.resolver.resolve(IRunnableBuilder_1.CurrentRunnableBuilderToken))));
            }
        });
    }
    static getClassAnnations() {
        return { "name": "DIModuleInjector", "params": { "constructor": ["validate"], "setup": ["container", "type"], "syncSetup": ["container", "type"], "valid": ["container", "type"], "getMetaAccessor": ["container", "token"], "importModule": ["container", "type"], "registerConfgureDepds": ["container", "config", "type"], "getConfigExports": ["container", "config"], "registerConfigExports": ["container", "newContainer", "injMd"], "registerConfigrue": ["newContainer", "injMd"] } };
    }
};
DIModuleInjector = tslib_1.__decorate([
    core_1.Singleton(exports.DIModuleInjectorToken),
    tslib_1.__param(0, core_1.Inject(DIModuleValidate_1.DIModuleValidateToken)),
    tslib_1.__metadata("design:paramtypes", [Object])
], DIModuleInjector);
exports.DIModuleInjector = DIModuleInjector;

//# sourceMappingURL=../sourcemaps/modules/DIModuleInjector.js.map
