"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@ts-ioc/core");
/**
 * injected module.
 *
 * @export
 * @class InjectedModule
 * @template T
 */
class InjectedModule {
    constructor(token, config, container, type, exports) {
        this.token = token;
        this.config = config;
        this.container = container;
        this.type = type;
        this.exports = exports;
    }
    getProviderMap() {
        if (!this.exports) {
            this.exports = this.container;
        }
        return this.exports;
    }
    get size() {
        return this.getProviderMap().size;
    }
    resolve(token, resway, ...providers) {
        let way;
        if (core_1.isNumber(resway)) {
            way = resway;
        }
        else {
            way = core_1.ResoveWay.all;
            if (resway) {
                providers.unshift(resway);
            }
        }
        let pdr = this.getProviderMap();
        if (pdr && (way & core_1.ResoveWay.current) && pdr.has(token)) {
            return pdr.resolve(token, core_1.ResoveWay.current, ...providers);
        }
        if (pdr !== this.container && (way & core_1.ResoveWay.traverse)) {
            return this.container.resolve(token, core_1.ResoveWay.traverse, ...providers);
        }
        return null;
    }
    has(token, aliasOrway) {
        let key = this.container.getTokenKey(token, core_1.isString(aliasOrway) ? aliasOrway : null);
        let pdr = this.getProviderMap();
        let resway = core_1.isNumber(aliasOrway) ? aliasOrway : core_1.ResoveWay.all;
        if (pdr && (resway & core_1.ResoveWay.current) && pdr.has(key)) {
            return true;
        }
        if (pdr !== this.container && (resway & core_1.ResoveWay.traverse)) {
            return this.container.has(key, core_1.ResoveWay.traverse);
        }
        return false;
    }
    getTokenImpl(token, resway) {
        return this.container.getTokenImpl(token, resway || core_1.ResoveWay.nodes);
    }
    unregister(token, resway) {
        let key = this.container.getTokenKey(token);
        let pdr = this.getProviderMap();
        resway = core_1.isNumber(resway) ? resway : core_1.ResoveWay.nodes;
        if (pdr && (resway & core_1.ResoveWay.current)) {
            pdr.unregister(key, core_1.ResoveWay.current);
        }
        if (resway & core_1.ResoveWay.traverse) {
            this.container.unregister(key, core_1.ResoveWay.nodes);
        }
        return this;
    }
    forEach(callbackfn) {
        let pdr = this.getProviderMap();
        if (pdr) {
            if (pdr.forEach((fac, tk) => {
                if (core_1.isToken(tk)) {
                    return callbackfn(tk, fac, pdr);
                }
            }) === false) {
                return false;
            }
        }
        return this.container.forEach((tk, fac) => {
            return callbackfn(tk, fac, this.container);
        });
    }
    static getClassAnnations() {
        return { "name": "InjectedModule", "params": { "constructor": ["token", "config", "container", "type", "exports"], "getProviderMap": [], "resolve": ["token", "resway", "providers"], "has": ["token", "aliasOrway"], "getTokenImpl": ["token", "resway"], "unregister": ["token", "resway"], "forEach": ["callbackfn"] } };
    }
}
exports.InjectedModule = InjectedModule;
/**
 * Injected Module Token.
 *
 * @export
 * @class InjectModuleMetaConfigToken
 * @extends {Registration<Type<T>>}
 * @template T
 */
class InjectedModuleToken extends core_1.Registration {
    constructor(type) {
        super(type, 'InjectedModule');
    }
    static getClassAnnations() {
        return { "name": "InjectedModuleToken", "params": { "constructor": ["type"] } };
    }
}
exports.InjectedModuleToken = InjectedModuleToken;

//# sourceMappingURL=../sourcemaps/modules/InjectedModule.js.map
