"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const core_1 = require("@ts-ioc/core");
/**
 * runnable options token.
 */
exports.RunnableOptionsToken = new core_1.InjectToken('boot_runnable_options');
/**
 * boot.
 *
 * @export
 * @class Boot
 * @implements {IBoot<T>}
 * @template T
 */
class RunnableBase {
    constructor(options) {
        this._options = options;
    }
    get options() {
        return this._options;
    }
    get bootOptions() {
        return this._bootOptions;
    }
    onInit(options, bootOptions) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (options) {
                this._options = options;
            }
            this._bootOptions = bootOptions;
        });
    }
    getTarget() {
        return this.options.instance;
    }
    getModuleToken() {
        return this.options.mdToken;
    }
    getTargetType() {
        return this.options.type || core_1.lang.getClass(this.options.instance);
    }
    static getClassAnnations() {
        return { "name": "RunnableBase", "params": { "constructor": ["options"], "onInit": ["options", "bootOptions"], "getTarget": [], "getModuleToken": [], "getTargetType": [] } };
    }
}
tslib_1.__decorate([
    core_1.Inject(core_1.ContainerToken),
    tslib_1.__metadata("design:type", Object)
], RunnableBase.prototype, "container", void 0);
exports.RunnableBase = RunnableBase;
/**
 * target is Runnable or not.
 *
 * @export
 * @param {*} target
 * @returns {target is RunnableBase<any>}
 */
function isRunnable(target) {
    if (target instanceof RunnableBase) {
        return true;
    }
    return false;
}
exports.isRunnable = isRunnable;

//# sourceMappingURL=../sourcemaps/runnable/Runnable.js.map
