'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var tslib_1 = _interopDefault(require('tslib'));
var core_1 = _interopDefault(require('@ts-ioc/core'));
var reflectMetadata = _interopDefault(require('reflect-metadata'));

function unwrapExports (x) {
	return x && x.__esModule && Object.prototype.hasOwnProperty.call(x, 'default') ? x['default'] : x;
}

function createCommonjsModule(fn, module) {
	return module = { exports: {} }, fn(module, module.exports), module.exports;
}

var IAnnotationBuilder = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });

const annoBuilderDesc = 'DI_AnnotationBuilder';
/**
 * inject Annotation class builder.
 *
 * @export
 * @class InjectBootstrapBuilder
 * @extends {Registration<T>}
 * @template T
 */
class InjectAnnotationBuilder extends core_1.RefRegistration {
    constructor(type) {
        super(type, annoBuilderDesc);
    }
    static getClassAnnations() {
        return { "name": "InjectAnnotationBuilder", "params": { "constructor": ["type"] } };
    }
}
exports.InjectAnnotationBuilder = InjectAnnotationBuilder;
/**
 * Annotation class builder token.
 */
exports.AnnotationBuilderToken = new InjectAnnotationBuilder('');
/**
 * Default Annotation class builder token.
 */
exports.DefaultAnnotationBuilderToken = new InjectAnnotationBuilder('default');


});

unwrapExports(IAnnotationBuilder);
var IAnnotationBuilder_1 = IAnnotationBuilder.InjectAnnotationBuilder;
var IAnnotationBuilder_2 = IAnnotationBuilder.AnnotationBuilderToken;
var IAnnotationBuilder_3 = IAnnotationBuilder.DefaultAnnotationBuilderToken;

var Annotation = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });


/**
 * create type builder decorator
 *
 * @export
 * @template T
 * @param {string} name
 * @param {string} [decorType]
 * @param {(Token<IAnnotationBuilder<any>> | IAnnotationBuilder<any>)} [defaultBuilder]
 * @param {defaultBoot?: Token<any> | ((metadata: T) => Token<any>)} [defaultBoot]
 * @param {MetadataAdapter} [adapter]
 * @param {MetadataExtends<T>} [metadataExtends]
 * @returns {IAnnotationDecorator<T>}
 */
function createAnnotationDecorator(name, defaultBuilder, defaultBoot, adapter, metadataExtends) {
    return core_1.createClassDecorator(name, args => {
        if (adapter) {
            adapter(args);
        }
    }, metadata => {
        if (metadataExtends) {
            metadataExtends(metadata);
        }
        if (defaultBuilder && !metadata.defaultAnnoBuilder) {
            metadata.defaultAnnoBuilder = defaultBuilder;
        }
        if (!metadata.bootstrap && defaultBoot) {
            let defboot = core_1.isToken(defaultBoot) ? defaultBoot : defaultBoot(metadata);
            if (defboot) {
                metadata.bootstrap = defboot;
            }
        }
        return metadata;
    });
}
exports.createAnnotationDecorator = createAnnotationDecorator;
/**
 * Annotation decorator, use to define class build way via config.
 *
 * @Annotation
 */
exports.Annotation = createAnnotationDecorator('Annotation', IAnnotationBuilder.AnnotationBuilderToken);


});

unwrapExports(Annotation);
var Annotation_1 = Annotation.createAnnotationDecorator;
var Annotation_2 = Annotation.Annotation;

var IModuleBuilder = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });

const moduleBuilderDesc = 'DI_ModuleBuilder';
/**
 * inject module builder token.
 *
 * @export
 * @class InjectModuleBuilder
 * @extends {Registration<T>}
 * @template T
 */
class InjectModuleBuilderToken extends core_1.RefRegistration {
    constructor(type) {
        super(type, moduleBuilderDesc);
    }
    static getClassAnnations() {
        return { "name": "InjectModuleBuilderToken", "params": { "constructor": ["type"] } };
    }
}
exports.InjectModuleBuilderToken = InjectModuleBuilderToken;
/**
 * default module builder token.
 */
exports.ModuleBuilderToken = new InjectModuleBuilderToken(Object);


});

unwrapExports(IModuleBuilder);
var IModuleBuilder_1 = IModuleBuilder.InjectModuleBuilderToken;
var IModuleBuilder_2 = IModuleBuilder.ModuleBuilderToken;

var DIModule = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });



/**
 * create bootstrap decorator.
 *
 * @export
 * @template T
 * @param {string} name decorator name.
 * @param {Token<IModuleBuilder>} [defaultBuilder]
 * @param {Token<IAnnotationBuilder<any>>} [defaultAnnoBuilder]
 * @param {defaultBoot?: Token<any> | ((metadata: T) => Token<any>)} [defaultBoot]
 * @param {MetadataAdapter} [adapter]
 * @param {MetadataExtends<T>} [metadataExtends]
 * @returns {IDIModuleDecorator<T>}
 */
function createDIModuleDecorator(name, defaultBuilder, defaultAnnoBuilder, defaultBoot, adapter, metadataExtends) {
    return core_1.createClassDecorator(name, args => {
        if (adapter) {
            adapter(args);
        }
    }, metadata => {
        if (metadataExtends) {
            metadataExtends(metadata);
        }
        if (!metadata.name && core_1.isClass(metadata.token)) {
            let isuglify = /^[a-z]$/.test(metadata.token.name);
            let classAnnations = core_1.lang.getClassAnnations(metadata.token);
            if (isuglify && classAnnations) {
                metadata.name = classAnnations.name;
            }
            else {
                metadata.name = metadata.token.name;
            }
        }
        metadata.decorType = name;
        if (defaultBuilder && !metadata.defaultBuilder) {
            metadata.defaultBuilder = defaultBuilder;
        }
        if (defaultAnnoBuilder && !metadata.defaultAnnoBuilder) {
            metadata.defaultAnnoBuilder = defaultAnnoBuilder;
        }
        if (!metadata.bootstrap && defaultBoot) {
            let defboot = core_1.isToken(defaultBoot) ? defaultBoot : defaultBoot(metadata);
            if (defboot) {
                metadata.bootstrap = defboot;
            }
        }
    });
}
exports.createDIModuleDecorator = createDIModuleDecorator;
/**
 * DIModule Decorator, definde class as DI module.
 *
 * @DIModule
 */
exports.DIModule = createDIModuleDecorator('DIModule', IModuleBuilder.ModuleBuilderToken, IAnnotationBuilder.AnnotationBuilderToken);


});

unwrapExports(DIModule);
var DIModule_1 = DIModule.createDIModuleDecorator;
var DIModule_2 = DIModule.DIModule;

var Runnable = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });


/**
 * runnable options token.
 */
exports.RunnableOptionsToken = new core_1.InjectToken('boot_runnable_options');
/**
 * boot.
 *
 * @export
 * @class Boot
 * @implements {IBoot<T>}
 * @template T
 */
class RunnableBase {
    constructor(options) {
        this._options = options;
    }
    get options() {
        return this._options;
    }
    get bootOptions() {
        return this._bootOptions;
    }
    onInit(options, bootOptions) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (options) {
                this._options = options;
            }
            this._bootOptions = bootOptions;
        });
    }
    getTarget() {
        return this.options.instance;
    }
    getModuleToken() {
        return this.options.mdToken;
    }
    getTargetType() {
        return this.options.type || core_1.lang.getClass(this.options.instance);
    }
    static getClassAnnations() {
        return { "name": "RunnableBase", "params": { "constructor": ["options"], "onInit": ["options", "bootOptions"], "getTarget": [], "getModuleToken": [], "getTargetType": [] } };
    }
}
tslib_1.__decorate([
    core_1.Inject(core_1.ContainerToken),
    tslib_1.__metadata("design:type", Object)
], RunnableBase.prototype, "container", void 0);
exports.RunnableBase = RunnableBase;
/**
 * target is Runnable or not.
 *
 * @export
 * @param {*} target
 * @returns {target is RunnableBase<any>}
 */
function isRunnable(target) {
    if (target instanceof RunnableBase) {
        return true;
    }
    return false;
}
exports.isRunnable = isRunnable;


});

unwrapExports(Runnable);
var Runnable_1 = Runnable.RunnableOptionsToken;
var Runnable_2 = Runnable.RunnableBase;
var Runnable_3 = Runnable.isRunnable;

var Runner_1 = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });


/**
 * boot element.
 *
 * @export
 * @abstract
 * @class Boot
 * @implements {IBoot}
 */
class Runner extends Runnable.RunnableBase {
    constructor(options) {
        super(options);
    }
    static getClassAnnations() {
        return { "name": "Runner", "params": { "constructor": ["options"], "run": ["data"] } };
    }
}
exports.Runner = Runner;
/**
 * target is runner or not.
 *
 * @export
 * @param {*} target
 * @returns {target is IRunner<any>}
 */
function isRunner(target) {
    if (target instanceof Runner) {
        return true;
    }
    if (target && core_1.isFunction(target.run)) {
        return true;
    }
    return false;
}
exports.isRunner = isRunner;


});

unwrapExports(Runner_1);
var Runner_2 = Runner_1.Runner;
var Runner_3 = Runner_1.isRunner;

var Service_1 = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });


/**
 * base service.
 *
 * @export
 * @abstract
 * @class Service
 * @implements {IService}
 */
class Service extends Runnable.RunnableBase {
    constructor(options) {
        super(options);
    }
    static getClassAnnations() {
        return { "name": "Service", "params": { "constructor": ["options"], "start": ["data"], "stop": [] } };
    }
}
exports.Service = Service;
/**
 * target is service or not.
 *
 * @export
 * @param {*} target
 * @returns {target is IService<any>}
 */
function isService(target) {
    if (target instanceof Service) {
        return true;
    }
    if (target && core_1.isFunction(target.start)) {
        return true;
    }
    return false;
}
exports.isService = isService;


});

unwrapExports(Service_1);
var Service_2 = Service_1.Service;
var Service_3 = Service_1.isService;

var types = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });

/**
 * application runner token.
 *
 * @export
 * @class InjectRunnerToken
 * @extends {Registration<IRunner<T>>}
 * @template T
 */
class InjectRunnableToken extends core_1.RefRegistration {
    constructor(type) {
        super(type, 'Runnable');
    }
    static getClassAnnations() {
        return { "name": "InjectRunnableToken", "params": { "constructor": ["type"] } };
    }
}
exports.InjectRunnableToken = InjectRunnableToken;
/**
 * default service token.
 */
exports.ServiceToken = new InjectRunnableToken('service');
/**
 * default runner token.
 */
exports.RunnerToken = new InjectRunnableToken('runner');


});

unwrapExports(types);
var types_1 = types.InjectRunnableToken;
var types_2 = types.ServiceToken;
var types_3 = types.RunnerToken;

var runnable = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });

tslib_1.__exportStar(Runnable, exports);
tslib_1.__exportStar(Runner_1, exports);
tslib_1.__exportStar(Service_1, exports);
tslib_1.__exportStar(types, exports);


});

unwrapExports(runnable);

var boots = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });


/**
 * boot
 *
 * @export
 * @abstract
 * @class Boot
 * @extends {Service<T>}
 * @implements {IBoot<T>}
 * @template T
 */
class Boot extends runnable.Service {
    /**
     * on boot init.
     *
     * @param {RunnableOptions<T>} options
     * @param {RunOptions<T>} bootOptions
     * @returns {Promise<void>}
     * @memberof IBoot
     */
    onInit(options, bootOptions) {
        const _super = Object.create(null, {
            onInit: { get: () => super.onInit }
        });
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield _super.onInit.call(this, options, bootOptions);
        });
    }
    /**
     * run boot.
     *
     * @abstract
     * @param {*} [data]
     * @returns {Promise<any>}
     * @memberof Boot
     */
    run(data) {
        return this.start(data);
    }
    static getClassAnnations() {
        return { "name": "Boot", "params": { "onInit": ["options", "bootOptions"], "run": ["data"] } };
    }
}
exports.Boot = Boot;


});

unwrapExports(boots);
var boots_1 = boots.Boot;

var DIModuleValidate = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });


/**
 * DIModule Validate Token
 */
exports.DIModuleValidateToken = new core_1.InjectModuleValidateToken('@DIModule');
/**
 * DIModuel Validate
 *
 * @export
 * @class DIModuelValidate
 * @extends {ModuelValidate}
 */
let DIModuelValidate = class DIModuelValidate extends core_1.ModuelValidate {
    getDecorator() {
        return '@DIModule';
    }
    static getClassAnnations() {
        return { "name": "DIModuelValidate", "params": { "getDecorator": [] } };
    }
};
DIModuelValidate = tslib_1.__decorate([
    core_1.Singleton(exports.DIModuleValidateToken)
], DIModuelValidate);
exports.DIModuelValidate = DIModuelValidate;


});

unwrapExports(DIModuleValidate);
var DIModuleValidate_1 = DIModuleValidate.DIModuleValidateToken;
var DIModuleValidate_2 = DIModuleValidate.DIModuelValidate;

var ContainerPool_1 = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });

const rootContainer = '__ioc_root_container';
/**
 * container pool
 *
 * @export
 * @class ContainerPool
 */
class ContainerPool {
    constructor(containerBuilder) {
        this.containerBuilder = containerBuilder;
        this.pools = new Map();
    }
    createContainer() {
        return this.containerBuilder.create();
    }
    getTokenKey(token) {
        if (token instanceof core_1.Registration) {
            return token.toString();
        }
        return token;
    }
    isDefault(container) {
        return container === this._default;
    }
    hasDefault() {
        return !!this._default;
    }
    getDefault() {
        if (!this._default) {
            this._default = this.createContainer();
            this.pools.set(rootContainer, [this._default]);
        }
        return this._default;
    }
    set(token, container) {
        let key = this.getTokenKey(token);
        if (this.pools.has(key)) {
            this.pools.get(key).push(container);
        }
        else {
            this.pools.set(key, [container]);
        }
    }
    get(token) {
        let key = this.getTokenKey(token);
        if (!this.has(key)) {
            return null;
        }
        return this.pools.get(token);
    }
    has(token) {
        return this.pools.has(this.getTokenKey(token));
    }
    create(token, parent) {
        parent = parent || this.getDefault();
        let container = parent.getBuilder().create();
        this.setParent(container, parent);
        this.set(token, container);
        return container;
    }
    setParent(container, parent) {
        if (this.isDefault(container)) {
            return;
        }
        parent = parent || this.getDefault();
        if (parent && parent !== container) {
            if (container.parent) {
                container.parent.children.splice(container.parent.children.indexOf(container), 1);
            }
            parent.children.push(container);
            container.parent = parent;
        }
    }
    keys() {
        return Array.from(this.pools.keys());
    }
    values() {
        return Array.from(this.pools.values()).reduce((p, c) => p.concat(c), []);
    }
    iterator(express, root) {
        root = root || this.getDefault();
        express(root);
        this.iteratorChildren(express, root.children);
    }
    iteratorChildren(express, children) {
        if (children && children.length) {
            children.forEach(c => {
                express(c);
                this.iteratorChildren(express, c.children);
            });
        }
    }
    static getClassAnnations() {
        return { "name": "ContainerPool", "params": { "constructor": ["containerBuilder"], "createContainer": [], "getTokenKey": ["token"], "isDefault": ["container"], "hasDefault": [], "getDefault": [], "set": ["token", "container"], "get": ["token"], "has": ["token"], "create": ["token", "parent"], "setParent": ["container", "parent"], "keys": [], "values": [], "iterator": ["express", "root"], "iteratorChildren": ["express", "children"] } };
    }
}
exports.ContainerPool = ContainerPool;
/**
 *  container pool token.
 */
exports.ContainerPoolToken = new core_1.InjectToken('DI_ContainerPool');


});

unwrapExports(ContainerPool_1);
var ContainerPool_2 = ContainerPool_1.ContainerPool;
var ContainerPool_3 = ContainerPool_1.ContainerPoolToken;

var Events_1 = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });

/**
 * custom events.
 *
 * @export
 * @class Events
 */
class Events {
    constructor() {
        this.eventsMap = {};
    }
    on(name, event) {
        this.eventsMap[name] = this.eventsMap[name] || [];
        if (this.eventsMap[name].indexOf(event) < 0) {
            this.eventsMap[name].push(event);
        }
        return this;
    }
    once(name, event) {
        let onceEvent = (...args) => {
            event(...args);
            this.off(name, onceEvent);
        };
        this.on(name, onceEvent);
    }
    off(name, event) {
        if (this.eventsMap[name]) {
            if (event) {
                this.eventsMap[name].splice(this.eventsMap[name].indexOf(event), 1);
            }
            else {
                delete this.eventsMap[name];
            }
        }
        return this;
    }
    emit(name, ...args) {
        let events = this.eventsMap[name];
        if (core_1.isArray(events)) {
            events.forEach(ev => {
                ev(...args);
            });
        }
    }
    static getClassAnnations() {
        return { "name": "Events", "params": { "constructor": [], "on": ["name", "event"], "once": ["name", "event"], "off": ["name", "event"], "emit": ["name", "args"] } };
    }
}
exports.Events = Events;


});

unwrapExports(Events_1);
var Events_2 = Events_1.Events;

var types$2 = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });


/**
 * is target di module class or not.
 *
 * @export
 * @param {*} target
 * @returns
 */
function isDIModuleClass(target) {
    return core_1.isClass(target) && core_1.hasOwnClassMetadata(DIModule.DIModule, target);
}
exports.isDIModuleClass = isDIModuleClass;


});

unwrapExports(types$2);
var types_1$1 = types$2.isDIModuleClass;

var utils = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });

tslib_1.__exportStar(ContainerPool_1, exports);
tslib_1.__exportStar(Events_1, exports);
tslib_1.__exportStar(types$2, exports);


});

unwrapExports(utils);

var InjectedModule_1 = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });

/**
 * injected module.
 *
 * @export
 * @class InjectedModule
 * @template T
 */
class InjectedModule {
    constructor(token, config, container, type, exports) {
        this.token = token;
        this.config = config;
        this.container = container;
        this.type = type;
        this.exports = exports;
    }
    getProviderMap() {
        if (!this.exports) {
            this.exports = this.container;
        }
        return this.exports;
    }
    get size() {
        return this.getProviderMap().size;
    }
    resolve(token, resway, ...providers) {
        let way;
        if (core_1.isNumber(resway)) {
            way = resway;
        }
        else {
            way = core_1.ResoveWay.all;
            if (resway) {
                providers.unshift(resway);
            }
        }
        let pdr = this.getProviderMap();
        if (pdr && (way & core_1.ResoveWay.current) && pdr.has(token)) {
            return pdr.resolve(token, core_1.ResoveWay.current, ...providers);
        }
        if (pdr !== this.container && (way & core_1.ResoveWay.traverse)) {
            return this.container.resolve(token, core_1.ResoveWay.traverse, ...providers);
        }
        return null;
    }
    has(token, aliasOrway) {
        let key = this.container.getTokenKey(token, core_1.isString(aliasOrway) ? aliasOrway : null);
        let pdr = this.getProviderMap();
        let resway = core_1.isNumber(aliasOrway) ? aliasOrway : core_1.ResoveWay.all;
        if (pdr && (resway & core_1.ResoveWay.current) && pdr.has(key)) {
            return true;
        }
        if (pdr !== this.container && (resway & core_1.ResoveWay.traverse)) {
            return this.container.has(key, core_1.ResoveWay.traverse);
        }
        return false;
    }
    getTokenImpl(token, resway) {
        return this.container.getTokenImpl(token, resway || core_1.ResoveWay.nodes);
    }
    unregister(token, resway) {
        let key = this.container.getTokenKey(token);
        let pdr = this.getProviderMap();
        resway = core_1.isNumber(resway) ? resway : core_1.ResoveWay.nodes;
        if (pdr && (resway & core_1.ResoveWay.current)) {
            pdr.unregister(key, core_1.ResoveWay.current);
        }
        if (resway & core_1.ResoveWay.traverse) {
            this.container.unregister(key, core_1.ResoveWay.nodes);
        }
        return this;
    }
    forEach(callbackfn) {
        let pdr = this.getProviderMap();
        if (pdr) {
            if (pdr.forEach((fac, tk) => {
                if (core_1.isToken(tk)) {
                    return callbackfn(tk, fac, pdr);
                }
            }) === false) {
                return false;
            }
        }
        return this.container.forEach((tk, fac) => {
            return callbackfn(tk, fac, this.container);
        });
    }
    static getClassAnnations() {
        return { "name": "InjectedModule", "params": { "constructor": ["token", "config", "container", "type", "exports"], "getProviderMap": [], "resolve": ["token", "resway", "providers"], "has": ["token", "aliasOrway"], "getTokenImpl": ["token", "resway"], "unregister": ["token", "resway"], "forEach": ["callbackfn"] } };
    }
}
exports.InjectedModule = InjectedModule;
/**
 * Injected Module Token.
 *
 * @export
 * @class InjectModuleMetaConfigToken
 * @extends {Registration<Type<T>>}
 * @template T
 */
class InjectedModuleToken extends core_1.Registration {
    constructor(type) {
        super(type, 'InjectedModule');
    }
    static getClassAnnations() {
        return { "name": "InjectedModuleToken", "params": { "constructor": ["type"] } };
    }
}
exports.InjectedModuleToken = InjectedModuleToken;


});

unwrapExports(InjectedModule_1);
var InjectedModule_2 = InjectedModule_1.InjectedModule;
var InjectedModule_3 = InjectedModule_1.InjectedModuleToken;

var ConfigureRegister_1 = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });


/**
 * configure register.
 *
 * @export
 * @abstract
 * @class ConfigureRegister
 * @implements {IConfigureRegister<T>}
 * @template T
 */
let ConfigureRegister = class ConfigureRegister {
    constructor() {
    }
    static getClassAnnations() {
        return { "name": "ConfigureRegister", "params": { "constructor": [], "register": ["config", "runBuilder"] } };
    }
};
tslib_1.__decorate([
    core_1.Inject(core_1.ContainerToken),
    tslib_1.__metadata("design:type", Object)
], ConfigureRegister.prototype, "container", void 0);
ConfigureRegister = tslib_1.__decorate([
    core_1.Abstract(),
    tslib_1.__metadata("design:paramtypes", [])
], ConfigureRegister);
exports.ConfigureRegister = ConfigureRegister;


});

unwrapExports(ConfigureRegister_1);
var ConfigureRegister_2 = ConfigureRegister_1.ConfigureRegister;

var IConfigureManager = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });

/**
 * configure manager token.
 */
exports.ConfigureMgrToken = new core_1.InjectToken('config-mgr');
/**
 * default configuration token.
 */
exports.DefaultConfigureToken = new core_1.InjectToken('DI_Default_Configuration');
/**
 * configure loader token.
 */
exports.ConfigureLoaderToken = new core_1.InjectToken('DI_Configure_Loader');


});

unwrapExports(IConfigureManager);
var IConfigureManager_1 = IConfigureManager.ConfigureMgrToken;
var IConfigureManager_2 = IConfigureManager.DefaultConfigureToken;
var IConfigureManager_3 = IConfigureManager.ConfigureLoaderToken;

var IRunnableBuilder = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });

/**
 *  process run root.
 */
exports.ProcessRunRootToken = new core_1.InjectToken('ioc_processRunRoot');
/**
 * runable builder token.
 */
exports.RunnableBuilderToken = new core_1.InjectToken('DI_RunnableBuilder');
exports.CurrentRunnableBuilderToken = new core_1.InjectToken('Current_DI_RunnableBuilder');


});

unwrapExports(IRunnableBuilder);
var IRunnableBuilder_1 = IRunnableBuilder.ProcessRunRootToken;
var IRunnableBuilder_2 = IRunnableBuilder.RunnableBuilderToken;
var IRunnableBuilder_3 = IRunnableBuilder.CurrentRunnableBuilderToken;

var DIModuleInjector_1 = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });








/**
 * DIModule injector token.
 */
exports.DIModuleInjectorToken = new core_1.InjectModuleInjectorToken('@DIModule');
/**
 * DIModule injector.
 *
 * @export
 * @class DIModuleInjector
 * @extends {ModuleInjector}
 */
let DIModuleInjector = class DIModuleInjector extends core_1.ModuleInjector {
    constructor(validate) {
        super(validate);
    }
    setup(container, type) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield this.importModule(container, type);
        });
    }
    syncSetup(container, type) {
        throw new Error('DIModule can not sync setup.');
    }
    valid(container, type) {
        if (!this.validate) {
            return true;
        }
        return this.validate.valid(type);
    }
    getMetaAccessor(container, token) {
        return container.getService(core_1.MetaAccessorToken, token);
    }
    importModule(container, type) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let pools = container.get(utils.ContainerPoolToken);
            let decorator = this.validate.getDecorator();
            let accor = this.getMetaAccessor(container, decorator);
            let metaConfig = accor.getMetadata(type, container, undefined, decorator ? dec => dec === decorator : undefined);
            let newContainer = metaConfig.asRoot === true ? pools.getDefault() : pools.create(type, container);
            newContainer.register(type);
            yield this.registerConfgureDepds(newContainer, metaConfig, type);
            let exps = yield this.getConfigExports(newContainer, metaConfig);
            let injMd = new InjectedModule_1.InjectedModule(metaConfig.token || type, metaConfig, newContainer, type, exps);
            container.bindProvider(new core_1.InjectReference(core_1.ProviderMap, type), exps);
            container.bindProvider(new InjectedModule_1.InjectedModuleToken(type), injMd);
            if (metaConfig.asRoot) {
                return injMd;
            }
            yield this.registerConfigExports(container, newContainer, injMd);
            // init global configure.
            if (!pools.isDefault(newContainer)) {
                yield this.registerConfigrue(newContainer, injMd);
            }
            return injMd;
        });
    }
    registerConfgureDepds(container, config, type) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (core_1.isArray(config.imports) && config.imports.length) {
                yield container.loadModule(...config.imports);
            }
        });
    }
    getConfigExports(container, config) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let parser = container.getProviderParser();
            let map = parser.parse(...config.providers || []);
            // bind module providers
            container.bindProviders(map);
            let builder = container.getBuilder();
            let exptypes = [].concat(...builder.loader.getTypes(config.exports || []));
            exptypes.forEach(ty => {
                let classPd = container.resolveValue(new core_1.InjectClassProvidesToken(ty));
                map.add(ty, (...pds) => container.resolve(ty, core_1.ResoveWay.nodes, ...pds));
                if (classPd && core_1.isArray(classPd.provides) && classPd.provides.length) {
                    classPd.provides.forEach(p => {
                        if (!map.has(p)) {
                            map.add(p, (...pds) => container.resolve(p, core_1.ResoveWay.nodes, ...pds));
                        }
                    });
                }
            });
            return map;
        });
    }
    registerConfigExports(container, newContainer, injMd) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (container === newContainer) {
                return container;
            }
            if (injMd) {
                let chain = container.getResolvers();
                chain.next(injMd);
            }
            return container;
        });
    }
    registerConfigrue(newContainer, injMd) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let registers = [];
            newContainer.iteratorServices((serType, fac, resolver) => {
                registers.push({
                    resolver: resolver,
                    serType: serType
                });
            }, ConfigureRegister_1.ConfigureRegister, injMd.type, true, core_1.ResoveWay.current);
            if (registers.length) {
                let mgr = newContainer.get(IConfigureManager.ConfigureMgrToken);
                let appConfig = yield mgr.getConfig();
                yield Promise.all(registers.map(ser => ser.resolver.resolve(ser.serType).register(appConfig, ser.resolver.resolve(IRunnableBuilder.CurrentRunnableBuilderToken))));
            }
        });
    }
    static getClassAnnations() {
        return { "name": "DIModuleInjector", "params": { "constructor": ["validate"], "setup": ["container", "type"], "syncSetup": ["container", "type"], "valid": ["container", "type"], "getMetaAccessor": ["container", "token"], "importModule": ["container", "type"], "registerConfgureDepds": ["container", "config", "type"], "getConfigExports": ["container", "config"], "registerConfigExports": ["container", "newContainer", "injMd"], "registerConfigrue": ["newContainer", "injMd"] } };
    }
};
DIModuleInjector = tslib_1.__decorate([
    core_1.Singleton(exports.DIModuleInjectorToken),
    tslib_1.__param(0, core_1.Inject(DIModuleValidate.DIModuleValidateToken)),
    tslib_1.__metadata("design:paramtypes", [Object])
], DIModuleInjector);
exports.DIModuleInjector = DIModuleInjector;


});

unwrapExports(DIModuleInjector_1);
var DIModuleInjector_2 = DIModuleInjector_1.DIModuleInjectorToken;
var DIModuleInjector_3 = DIModuleInjector_1.DIModuleInjector;

var AnnoBuildStrategy_1 = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });


/**
 * annotation build strategy token inject.
 *
 * @export
 * @class InjectAnnoBuildStrategyToken
 * @extends {RefRegistration<IAnnoBuildStrategy<T>>}
 * @template T
 */
class InjectAnnoBuildStrategyToken extends core_1.RefRegistration {
    constructor(type) {
        super(type, 'AnnoBuildStrategy');
    }
    static getClassAnnations() {
        return { "name": "InjectAnnoBuildStrategyToken", "params": { "constructor": ["type"] } };
    }
}
exports.InjectAnnoBuildStrategyToken = InjectAnnoBuildStrategyToken;
/**
 *  base AnnoBuildStrategy Token
 */
exports.AnnoBuildStrategyToken = new InjectAnnoBuildStrategyToken(Object);
/**
 * annotation instance build strategy.
 *
 * @export
 * @class AnnoBuildStrategy
 * @implements {IAnnoBuildStrategy<T>}
 * @template T
 */
let AnnoBuildStrategy = class AnnoBuildStrategy {
    build(instance, config, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (!instance) {
                return;
            }
            yield this.beforeInit(instance, config);
            yield this.process(instance, config, options);
            yield this.afterInit(instance, config);
        });
    }
    /**
     * after annotation instance init.
     *
     * @protected
     * @param {BootHooks<T>} instance
     * @param {AnnotationConfigure<T>} config
     * @returns {Promise<void>}
     * @memberof AnnoBuildStrategy
     */
    beforeInit(instance, config) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (core_1.isFunction(instance.anBeforeInit)) {
                yield instance.anBeforeInit(config);
            }
        });
    }
    /**
     * build annotation instance process.
     *
     * @protected
     * @param {T} instance
     * @param {AnnotationConfigure<T>} config
     * @param {BuildOptions<T>} [options]
     * @returns {Promise<void>}
     * @memberof AnnoBuildStrategy
     */
    process(instance, config, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
        });
    }
    /**
     * after annotation instance init.
     *
     * @protected
     * @param {BootHooks<T>} instance
     * @param {AnnotationConfigure<T>} config
     * @returns {Promise<void>}
     * @memberof AnnoBuildStrategy
     */
    afterInit(instance, config) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (core_1.isFunction(instance.anAfterInit)) {
                yield instance.anAfterInit(config);
            }
        });
    }
    static getClassAnnations() {
        return { "name": "AnnoBuildStrategy", "params": { "build": ["instance", "config", "options"], "beforeInit": ["instance", "config"], "process": ["instance", "config", "options"], "afterInit": ["instance", "config"] } };
    }
};
AnnoBuildStrategy = tslib_1.__decorate([
    core_1.Singleton(exports.AnnoBuildStrategyToken)
], AnnoBuildStrategy);
exports.AnnoBuildStrategy = AnnoBuildStrategy;


});

unwrapExports(AnnoBuildStrategy_1);
var AnnoBuildStrategy_2 = AnnoBuildStrategy_1.InjectAnnoBuildStrategyToken;
var AnnoBuildStrategy_3 = AnnoBuildStrategy_1.AnnoBuildStrategyToken;
var AnnoBuildStrategy_4 = AnnoBuildStrategy_1.AnnoBuildStrategy;

var AnnotationBuilder_2 = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });

var AnnotationBuilder_1;




/**
 * Annotation class builder. build class with metadata and config.
 *
 * @export
 * @class AnnotationBuilder
 * @implements {implements IAnnotationBuilder<T>}
 * @template T
 */
let AnnotationBuilder = AnnotationBuilder_1 = class AnnotationBuilder {
    constructor() {
    }
    /**
     * get metadata accessor.
     *
     * @param {(Token<any>|| AnnotationConfigure<T>)} token
     * @param {AnnotationConfigure<T>} [config]
     * @returns {IMetaAccessor<any>}
     * @memberof AnnotationBuilder
     */
    getMetaAccessor(token, config) {
        let mtk;
        if (core_1.isToken(token)) {
            mtk = token;
        }
        else {
            config = token;
        }
        return this.container.getService(core_1.MetaAccessorToken, mtk ? [mtk, core_1.lang.getClass(this)] : core_1.lang.getClass(this), tk => new core_1.InjectMetaAccessorToken(tk), config ? (config.defaultMetaAccessor || core_1.MetaAccessorToken) : core_1.MetaAccessorToken);
    }
    /**
     * build token type with config.
     *
     * @param {(Token<T> | AnnotationConfigure<T>)} token
     * @param {(AnnotationConfigure<T> | BuildOptions<T>)} [config]
     * @param {BuildOptions<T>} [options]
     * @returns {Promise<T>}
     * @memberof AnnotationBuilder
     */
    build(token, config, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let tk;
            let cfg;
            if (core_1.isToken(token)) {
                tk = token;
                if (core_1.isUndefined(options)) {
                    options = config;
                    cfg = null;
                }
                else {
                    cfg = config;
                }
                cfg = this.getMetaAccessor(tk, cfg).getMetadata(tk, this.container, cfg);
            }
            else {
                options = config;
                cfg = token;
                let accessor = this.getMetaAccessor(cfg);
                tk = accessor.getToken(cfg, this.container);
                if (core_1.isToken(tk)) {
                    cfg = accessor.getMetadata(tk, this.container, cfg);
                }
            }
            if (core_1.isClass(tk) && !this.container.hasRegister(tk)) {
                this.container.register(tk);
            }
            if (options && core_1.isFunction(options.vaild) && !options.vaild(tk)) {
                return null;
            }
            let builder = (options && options.builder) ? options.builder : this.getBuilder(tk, cfg);
            if (!this.isEqual(builder)) {
                return yield builder.build(tk, cfg, core_1.lang.assign(options || {}, { builder: builder }));
            }
            else {
                yield this.registerExts(cfg);
                let instance = yield this.createInstance(tk, cfg, options);
                if (!instance) {
                    return null;
                }
                let strategy = this.container.getService(AnnoBuildStrategy_1.AnnoBuildStrategyToken, [
                    tk || core_1.lang.getClass(instance),
                    ...((options && options.target) ? [{ target: core_1.lang.getClass(options.target), level: core_1.RefTagLevel.self }] : []),
                    { target: core_1.lang.getClass(this), level: core_1.RefTagLevel.self }
                ], tk => new AnnoBuildStrategy_1.InjectAnnoBuildStrategyToken(tk));
                if (strategy) {
                    yield strategy.build(instance, cfg, options);
                }
                if (options && core_1.isFunction(options.onCompleted)) {
                    options.onCompleted(tk, cfg, instance, this);
                }
                return instance;
            }
        });
    }
    /**
     * run runable.
     *
     * @param {(Token<T> | AnnotationConfigure<T>)} runable
     * @param {*} [data] bootstrap data, build data, Runnable data.
     * @returns {Promise<Runnable<T>>}
     * @memberof IGModuleBuilder
     */
    boot(runable, config, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let instance;
            let builder;
            let tk;
            let cfg;
            let onCompleted = (token, config, hook, builder) => {
                tk = token;
                cfg = config;
                instance = hook;
                builder = builder;
            };
            if (core_1.isToken(runable)) {
                if (core_1.isUndefined(options)) {
                    options = config;
                    cfg = null;
                }
                else {
                    cfg = config;
                }
                yield this.build(runable, cfg, core_1.lang.assign({
                    onCompleted: onCompleted
                }, options));
            }
            else {
                yield this.build(runable, core_1.lang.assign({
                    onCompleted: onCompleted
                }, config));
            }
            builder = builder || this;
            let data = options ? options.data : undefined;
            let runOptions = { instance: instance, type: core_1.lang.getClass(instance), mdToken: tk, config: cfg, data: data };
            let runner = builder.resolveRunable(instance, runOptions, options);
            if (runner && core_1.isFunction(runner.onInit)) {
                yield runner.onInit(runOptions, options);
            }
            if (runnable.isRunner(runner)) {
                yield runner.run(data);
            }
            else if (runnable.isService(runner)) {
                yield runner.start(data);
            }
            return runner;
        });
    }
    /**
     * run annotation instance.
     *
     * @param {T} instance
     * @param {AnnotationConfigure<T>} [config]
     * @param {Token<T>} [token]
     * @returns {Promise<Runnable<T>>}
     * @memberof AnnotationBuilder
     */
    resolveRunable(instance, runableOptions, options) {
        if (!instance) {
            return null;
        }
        let tk = runableOptions.mdToken || runableOptions.type;
        if (!runableOptions.config) {
            runableOptions.config = this.getMetaAccessor(tk).getMetadata(tk, this.container);
        }
        if (instance instanceof runnable.Runner || instance instanceof runnable.Service) {
            return instance;
        }
        else {
            let provider = { provide: runnable.RunnableOptionsToken, useValue: runableOptions.config };
            return this.container.getService([runnable.RunnerToken, runnable.ServiceToken], [
                tk,
                ...((options && options.target) ? [{ target: core_1.lang.getClass(options.target), level: core_1.RefTagLevel.self }] : []),
                { target: core_1.lang.getClass(this), level: core_1.RefTagLevel.self }
            ], tk => new runnable.InjectRunnableToken(tk), runableOptions.config.defaultRunnable || true, provider);
        }
    }
    createInstance(token, config, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (!core_1.isToken(token)) {
                return null;
            }
            let instance = this.resolveToken(token, options ? options.target : undefined, ...(options.providers || []));
            if (core_1.isNullOrUndefined(instance)) {
                console.log(`can not find token ${token ? token.toString() : null} in container.`);
                return null;
            }
            return instance;
        });
    }
    getBuilder(token, config, options) {
        let builder;
        if (config && config.annoBuilder) {
            if (core_1.isClass(config.annoBuilder)) {
                if (!this.container.has(config.annoBuilder)) {
                    this.container.register(config.annoBuilder);
                }
            }
            if (core_1.isToken(config.annoBuilder)) {
                builder = this.container.resolve(config.annoBuilder);
            }
            else if (config.annoBuilder instanceof AnnotationBuilder_1) {
                builder = config.annoBuilder;
            }
        }
        if (options && options.builder) {
            builder = options.builder;
        }
        if (!builder && token) {
            builder = this.container.getService(IAnnotationBuilder.AnnotationBuilderToken, [
                token,
                ...(options && options.target) ? [{ target: core_1.lang.getClass(options.target), level: core_1.RefTagLevel.self }] : []
            ], (tk) => new IAnnotationBuilder.InjectAnnotationBuilder(tk), config.defaultAnnoBuilder || IAnnotationBuilder.AnnotationBuilderToken);
        }
        if (builder && !builder.container) {
            builder.container = this.container;
        }
        return builder || this;
    }
    /**
     * register extension before create instance.
     *
     * @protected
     * @param {AnnotationConfigure<T>} [config]
     * @memberof AnnotationBuilder
     */
    registerExts(config) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
        });
    }
    isEqual(build) {
        if (!build) {
            return false;
        }
        if (build === this) {
            return true;
        }
        if (core_1.lang.getClass(build) === core_1.lang.getClass(this)) {
            return true;
        }
        return false;
    }
    resolveToken(token, target, ...providers) {
        if (target) {
            return this.container.getService(token, target, ...providers);
        }
        return this.container.resolve(token, ...providers);
    }
    static getClassAnnations() {
        return { "name": "AnnotationBuilder", "params": { "constructor": [], "getMetaAccessor": ["token", "config"], "build": ["token", "config", "options"], "boot": ["runable", "config", "options"], "resolveRunable": ["instance", "runableOptions", "options"], "createInstance": ["token", "config", "options"], "getBuilder": ["token", "config", "options"], "registerExts": ["config"], "isEqual": ["build"], "resolveToken": ["token", "target", "providers"] } };
    }
};
tslib_1.__decorate([
    core_1.Inject(core_1.ContainerToken),
    tslib_1.__metadata("design:type", Object)
], AnnotationBuilder.prototype, "container", void 0);
AnnotationBuilder = AnnotationBuilder_1 = tslib_1.__decorate([
    core_1.Singleton(IAnnotationBuilder.AnnotationBuilderToken),
    tslib_1.__metadata("design:paramtypes", [])
], AnnotationBuilder);
exports.AnnotationBuilder = AnnotationBuilder;


});

unwrapExports(AnnotationBuilder_2);
var AnnotationBuilder_3 = AnnotationBuilder_2.AnnotationBuilder;

var annotations = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });

tslib_1.__exportStar(IAnnotationBuilder, exports);
tslib_1.__exportStar(AnnoBuildStrategy_1, exports);
tslib_1.__exportStar(AnnotationBuilder_2, exports);


});

unwrapExports(annotations);

var ModuleBuilder_1 = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });







/**
 * inject module load token.
 *
 * @export
 * @class InjectModuleLoadToken
 * @extends {Registration<T>}
 * @template T
 */
class InjectModuleLoadToken extends core_1.Registration {
    constructor(token) {
        super(token, 'module_loader');
    }
    static getClassAnnations() {
        return { "name": "InjectModuleLoadToken", "params": { "constructor": ["token"] } };
    }
}
exports.InjectModuleLoadToken = InjectModuleLoadToken;
/**
 * module builder
 *
 * @export
 * @class ModuleBuilder
 * @implements {IModuleBuilder}
 * @template T
 */
let ModuleBuilder = class ModuleBuilder {
    constructor() {
    }
    getPools() {
        return this.pools;
    }
    getInjectedModule(type) {
        return this.getPools().getDefault().get(new InjectedModule_1.InjectedModuleToken(type));
    }
    /**
     * get metadata accessor.
     *
     * @param {IContainer} container
     * @param {Token<any>} token
     * @param {ModuleConfigure} [config]
     * @returns {IMetaAccessor<any>}
     * @memberof ModuleBuilder
     */
    getMetaAccessor(container, token, config) {
        let mtk;
        if (core_1.isToken(token)) {
            mtk = token;
        }
        else {
            config = token;
        }
        return container.getService(core_1.MetaAccessorToken, mtk ? [mtk, core_1.lang.getClass(this)] : core_1.lang.getClass(this), tk => new core_1.InjectMetaAccessorToken(tk), config ? (config.defaultMetaAccessor || core_1.MetaAccessorToken) : core_1.MetaAccessorToken);
    }
    /**
     * load module.
     *
     * @param {(Token<T> | ModuleConfigure)} token
     * @param {(ModuleConfig<T> | BootOptions<T>)} [config]
     * @param {BootOptions<T>} [options]
     * @returns {Promise<InjectedModule<T>>}
     * @memberof ModuleBuilder
     */
    load(token, config, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let params = this.vaildParams(token, config, options);
            let env = params.options ? params.options.env : null;
            if (env instanceof InjectedModule_1.InjectedModule) {
                return env;
            }
            let parent = yield this.getParentContainer(env);
            if (params.token) {
                let injmd = yield this.loadViaToken(params.token, parent);
                if (params.config) {
                    yield this.loadViaConfig(params.config, injmd.container || parent);
                }
                return injmd;
            }
            else {
                return yield this.loadViaConfig(params.config, parent);
            }
        });
    }
    /**
     * bootstrap with module and config.
     *
     * @param {(Token<T> | ModuleConfig<T>)} token
     * @param {(ModuleConfig<T> | BootOptions<T>)} [config]
     * @param {BootOptions<T>} [options]
     * @returns {Promise<Runnable<T>>}
     * @memberof ModuleBuilder
     */
    bootstrap(token, config, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let params = this.vaildParams(token, config, options);
            options = params.options || {};
            let injmdl;
            if (params.token) {
                injmdl = yield this.load(params.token, params.config, options);
            }
            else {
                injmdl = yield this.load(params.config, options);
            }
            options.env = injmdl;
            let cfg = injmdl.config;
            let tk = injmdl.token || injmdl.type;
            let container = injmdl.container;
            let accessor = this.getMetaAccessor(container, tk);
            let bootToken = accessor.getBootToken(cfg, container);
            if (bootToken) {
                let anBuilder = this.getAnnoBuilder(container, bootToken, cfg);
                return yield anBuilder.boot(bootToken, cfg, options);
            }
            else {
                let mdBuilder = this.getAnnoBuilder(container, tk, cfg);
                return yield mdBuilder.boot(tk, cfg, options);
            }
        });
    }
    vaildParams(token, config, options) {
        let params = {};
        if (core_1.isToken(token)) {
            params.token = token;
            if (core_1.isUndefined(options)) {
                params.options = config;
            }
            else {
                params.config = config;
                params.options = options;
            }
        }
        else {
            params.config = token;
            params.options = config;
        }
        return params;
    }
    loadViaToken(token, parent) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let injmdl = yield this.import(token, parent);
            if (!injmdl) {
                let cfg = this.getMetaAccessor(parent, token).getMetadata(token, parent);
                injmdl = new InjectedModule_1.InjectedModule(token, cfg, parent);
            }
            return injmdl;
        });
    }
    loadViaConfig(config, parent) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let injmd = null;
            let token = this.getMetaAccessor(parent, config).getToken(config, parent);
            if (token) {
                injmd = yield this.import(token, parent);
                if (!injmd) {
                    let cfg = this.getMetaAccessor(parent, token).getMetadata(token, parent, config);
                    injmd = new InjectedModule_1.InjectedModule(token, cfg, parent);
                }
            }
            else {
                token = null;
            }
            if (core_1.isArray(config.imports) && config.imports.length) {
                yield parent.loadModule(...config.imports);
            }
            if (core_1.isArray(config.providers) && config.providers.length) {
                let parser = parent.getProviderParser();
                let pdrmap = parser.parse(...config.providers);
                pdrmap.keys().forEach(key => {
                    core_1.isToken(key) && parent.bindProvider(key, (...providers) => pdrmap.resolve(key, ...providers));
                });
            }
            if (injmd) {
                injmd.config = core_1.lang.assign({}, injmd.config, config);
            }
            else {
                injmd = new InjectedModule_1.InjectedModule(token, config, parent);
            }
            return injmd;
        });
    }
    import(token, parent) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let type = core_1.isClass(token) ? token : parent.getTokenImpl(token);
            if (core_1.isClass(type)) {
                let key = new InjectedModule_1.InjectedModuleToken(type);
                if (parent.has(key)) {
                    return parent.get(key);
                }
                else {
                    yield parent.loadModule(type);
                    if (parent.has(key)) {
                        return parent.get(key);
                    }
                }
            }
            return null;
        });
    }
    getParentContainer(env) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let parent;
            if (env) {
                if (env instanceof core_1.Container) {
                    parent = env;
                }
                else if (env instanceof InjectedModule_1.InjectedModule) {
                    parent = env.container.parent;
                }
            }
            if (!parent) {
                parent = this.getPools().getDefault();
            }
            return parent;
        });
    }
    getAnnoBuilder(container, token, config) {
        let builder;
        if (core_1.isClass(config.annoBuilder)) {
            if (!container.has(config.annoBuilder)) {
                container.register(config.annoBuilder);
            }
        }
        if (core_1.isToken(config.annoBuilder)) {
            builder = container.resolve(config.annoBuilder);
        }
        else if (config.annoBuilder instanceof annotations.AnnotationBuilder) {
            builder = config.annoBuilder;
        }
        if (!builder && token) {
            builder = container.getService(annotations.AnnotationBuilderToken, token, tk => new annotations.InjectAnnotationBuilder(tk), config.defaultAnnoBuilder || annotations.AnnotationBuilderToken);
        }
        if (builder) {
            builder.container = container;
        }
        return builder;
    }
    static getClassAnnations() {
        return { "name": "ModuleBuilder", "params": { "constructor": [], "getPools": [], "getInjectedModule": ["type"], "getMetaAccessor": ["container", "token", "config"], "load": ["token", "config", "options"], "bootstrap": ["token", "config", "options"], "vaildParams": ["token", "config", "options"], "loadViaToken": ["token", "parent"], "loadViaConfig": ["config", "parent"], "import": ["token", "parent"], "getParentContainer": ["env"], "getAnnoBuilder": ["container", "token", "config"] } };
    }
};
tslib_1.__decorate([
    core_1.Inject(utils.ContainerPoolToken),
    tslib_1.__metadata("design:type", utils.ContainerPool)
], ModuleBuilder.prototype, "pools", void 0);
ModuleBuilder = tslib_1.__decorate([
    core_1.Singleton(IModuleBuilder.ModuleBuilderToken),
    tslib_1.__metadata("design:paramtypes", [])
], ModuleBuilder);
exports.ModuleBuilder = ModuleBuilder;


});

unwrapExports(ModuleBuilder_1);
var ModuleBuilder_2 = ModuleBuilder_1.InjectModuleLoadToken;
var ModuleBuilder_3 = ModuleBuilder_1.ModuleBuilder;

var modules = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });

tslib_1.__exportStar(DIModuleInjector_1, exports);
tslib_1.__exportStar(DIModuleValidate, exports);
tslib_1.__exportStar(InjectedModule_1, exports);
tslib_1.__exportStar(IModuleBuilder, exports);
tslib_1.__exportStar(ModuleBuilder_1, exports);


});

unwrapExports(modules);

var BootModuleInjector = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });



/**
 * Bootstrap module Validate Token
 */
exports.BootstrapValidateToken = new core_1.InjectModuleValidateToken('@Bootstrap');
/**
 * Bootstrap module Validate
 *
 * @export
 * @class DIModuelValidate
 * @extends {ModuelValidate}
 */
let BootstrapValidate = class BootstrapValidate extends core_1.ModuelValidate {
    getDecorator() {
        return '@Bootstrap';
    }
    static getClassAnnations() {
        return { "name": "BootstrapValidate", "params": { "getDecorator": [] } };
    }
};
BootstrapValidate = tslib_1.__decorate([
    core_1.Singleton(exports.BootstrapValidateToken)
], BootstrapValidate);
exports.BootstrapValidate = BootstrapValidate;
exports.BootstrapInjectorToken = new core_1.InjectModuleInjectorToken('@Bootstrap');
let BootstrapInjector = class BootstrapInjector extends modules.DIModuleInjector {
    constructor(validate) {
        super(validate);
    }
    static getClassAnnations() {
        return { "name": "BootstrapInjector", "params": { "constructor": ["validate"] } };
    }
};
BootstrapInjector = tslib_1.__decorate([
    core_1.Singleton(exports.BootstrapInjectorToken),
    tslib_1.__param(0, core_1.Inject(exports.BootstrapValidateToken)),
    tslib_1.__metadata("design:paramtypes", [Object])
], BootstrapInjector);
exports.BootstrapInjector = BootstrapInjector;


});

unwrapExports(BootModuleInjector);
var BootModuleInjector_1 = BootModuleInjector.BootstrapValidateToken;
var BootModuleInjector_2 = BootModuleInjector.BootstrapValidate;
var BootModuleInjector_3 = BootModuleInjector.BootstrapInjectorToken;
var BootModuleInjector_4 = BootModuleInjector.BootstrapInjector;

var ConfigureManager_1 = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });



/**
 * configure manager.
 *
 * @export
 * @class ConfigureManager
 */
let ConfigureManager = class ConfigureManager {
    /**
     * Creates an instance of ConfigureManager.
     * @param {string} [baseURL]
     * @memberof ConfigureManager
     */
    constructor(baseURL) {
        this.baseURL = baseURL;
        this.configs = [];
    }
    /**
     * use configuration.
     *
     * @param {(string | AppConfigure)} [config]
     * @returns {this} this configure manager.
     * @memberof Bootstrap
     */
    useConfiguration(config) {
        if (core_1.isUndefined(config)) {
            config = '';
        }
        // clean cached config.
        this.config = null;
        let idx = this.configs.indexOf(config);
        if (idx >= 0) {
            this.configs.splice(idx, 1);
        }
        this.configs.push(config);
        return this;
    }
    /**
     * get config.
     *
     * @returns {Promise<T>}
     * @memberof ConfigureManager
     */
    getConfig() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (!this.config) {
                this.config = yield this.initConfig();
            }
            return this.config;
        });
    }
    /**
     * init config.
     *
     * @protected
     * @returns
     * @memberof ConfigureManager
     */
    initConfig() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let config = yield this.getDefaultConfig();
            if (this.configs.length < 1) {
                this.configs.push(''); // load default loader config.
            }
            let exts = yield Promise.all(this.configs.map(cfg => {
                if (core_1.isString(cfg)) {
                    return this.loadConfig(cfg);
                }
                else {
                    return cfg;
                }
            }));
            exts.forEach(exCfg => {
                if (exCfg) {
                    core_1.lang.assign(config, exCfg);
                }
            });
            return config;
        });
    }
    /**
     * load config.
     *
     * @protected
     * @param {string} src
     * @returns {Promise<T>}
     * @memberof ConfigureManager
     */
    loadConfig(src) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (this.container.has(IConfigureManager.ConfigureLoaderToken)) {
                let loader = this.container.resolve(IConfigureManager.ConfigureLoaderToken, { baseURL: this.baseURL, container: this.container });
                return yield loader.load(src);
            }
            else if (src) {
                let builder = this.container.getBuilder();
                let cfg = yield builder.loader.load([src]);
                return cfg.length ? cfg[0] : null;
            }
            else {
                return null;
            }
        });
    }
    /**
     * get default config.
     *
     * @protected
     * @returns {Promise<T>}
     * @memberof ConfigureManager
     */
    getDefaultConfig() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (this.container.has(IConfigureManager.DefaultConfigureToken)) {
                return this.container.resolve(IConfigureManager.DefaultConfigureToken);
            }
            else {
                return {};
            }
        });
    }
    static getClassAnnations() {
        return { "name": "ConfigureManager", "params": { "constructor": ["baseURL"], "useConfiguration": ["config"], "getConfig": [], "initConfig": [], "loadConfig": ["src"], "getDefaultConfig": [] } };
    }
};
tslib_1.__decorate([
    core_1.Inject(core_1.ContainerToken),
    tslib_1.__metadata("design:type", Object)
], ConfigureManager.prototype, "container", void 0);
ConfigureManager = tslib_1.__decorate([
    core_1.Singleton(IConfigureManager.ConfigureMgrToken),
    tslib_1.__metadata("design:paramtypes", [String])
], ConfigureManager);
exports.ConfigureManager = ConfigureManager;


});

unwrapExports(ConfigureManager_1);
var ConfigureManager_2 = ConfigureManager_1.ConfigureManager;

var BootModule_1 = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });








/**
 * Bootstrap ext for ioc. auto run setup after registered.
 * with @IocExt('setup') decorator.
 * @export
 * @class BootModule
 */
let BootModule = class BootModule {
    constructor(container) {
        this.container = container;
    }
    /**
     * register aop for container.
     *
     * @memberof AopModule
     */
    setup() {
        let container = this.container;
        let lifeScope = container.get(core_1.LifeScopeToken);
        lifeScope.registerDecorator(Annotation.Annotation, core_1.CoreActions.bindProvider, core_1.CoreActions.cache, core_1.CoreActions.componentBeforeInit, core_1.CoreActions.componentInit, core_1.CoreActions.componentAfterInit);
        lifeScope.registerDecorator(DIModule.DIModule, core_1.CoreActions.bindProvider, core_1.CoreActions.cache, core_1.CoreActions.componentBeforeInit, core_1.CoreActions.componentInit, core_1.CoreActions.componentAfterInit);
        lifeScope.registerDecorator(Bootstrap.Bootstrap, core_1.CoreActions.bindProvider, core_1.CoreActions.cache, core_1.CoreActions.componentBeforeInit, core_1.CoreActions.componentInit, core_1.CoreActions.componentAfterInit);
        container.use(annotations, modules, boot);
    }
    static getClassAnnations() {
        return { "name": "BootModule", "params": { "constructor": ["container"], "setup": [] } };
    }
};
BootModule = tslib_1.__decorate([
    core_1.IocExt('setup'),
    tslib_1.__param(0, core_1.Inject(core_1.ContainerToken)),
    tslib_1.__metadata("design:paramtypes", [Object])
], BootModule);
exports.BootModule = BootModule;


});

unwrapExports(BootModule_1);
var BootModule_2 = BootModule_1.BootModule;

var RunnableBuilder_1 = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });









/**
 * runnable events
 *
 * @export
 * @enum {number}
 */
var RunnableEvents;
(function (RunnableEvents) {
    /**
     * on root container created.
     */
    RunnableEvents["onRootContainerCreated"] = "onRootContainerCreated";
    /**
     * on root container inited.
     */
    RunnableEvents["onRootContainerInited"] = "onRootContainerInited";
    /**
     * on module created.
     */
    RunnableEvents["onModuleCreated"] = "onModuleCreated";
    /**
     * on registered runner use module.
     */
    RunnableEvents["registeredExt"] = "registeredExt";
})(RunnableEvents = exports.RunnableEvents || (exports.RunnableEvents = {}));
/**
 * runnable builder.
 *
 * @export
 * @class RunnableBuilder
 * @extends {ModuleBuilder}
 * @template T
 */
let RunnableBuilder = class RunnableBuilder extends modules.ModuleBuilder {
    constructor(baseURL) {
        super();
        this.inited = false;
        this._baseURL = baseURL;
        this.customRegs = [];
        this.depModules = [];
        this.beforeInitPds = new Map();
        this.afterInitPds = new Map();
        this.events = new utils.Events();
        this.initEvents();
    }
    initEvents() {
        this.on(RunnableEvents.onRootContainerInited, (container) => {
            this.afterInitPds.forEach((val, key) => {
                container.bindProvider(key, val);
            });
        });
    }
    on(name, event) {
        this.events.on(name, event);
        return this;
    }
    off(name, event) {
        this.events.off(name, event);
        return this;
    }
    emit(name, ...args) {
        this.events.emit(name, ...args);
    }
    getPools() {
        if (!this.pools) {
            this.pools = new utils.ContainerPool(this.createContainerBuilder());
            this.createDefaultContainer();
        }
        return this.pools;
    }
    getRunRoot(resolver) {
        return this._baseURL || (resolver || this.getPools().getDefault()).resolve(IRunnableBuilder.ProcessRunRootToken) || '';
    }
    /**
     * has register in pools.
     * use must after `initContainerPools`.
     *
     * @template T
     * @param {Token<T>} key
     * @returns {boolean}
     * @memberof RunnableBuilder
     */
    has(key, aliasOrway) {
        return this.getPools().values().some(c => c.has(key, aliasOrway));
    }
    /**
     * resove token in pools.
     * use must after `initContainerPools`.
     * @template T
     * @param {Token<T>} token
     * @param {...ParamProviders[]} providers
     * @returns {T}
     * @memberof RunnableBuilder
     */
    resolve(token, resway, ...providers) {
        let resolved;
        this.getPools().values().some(c => {
            resolved = c.resolve(token, resway, ...providers);
            return !core_1.isNullOrUndefined(resolved);
        });
        return resolved;
    }
    createContainerBuilder() {
        return new core_1.ContainerBuilder();
    }
    /**
     * use module as global Depdences module.
     *
     * @param {...LoadType[]} modules
     * @returns {this}
     * @memberof RunnableBuilder
     */
    use(...modules$$2) {
        this.depModules = this.depModules.concat(modules$$2);
        this.inited = false;
        return this;
    }
    /**
     * bind provider
     *
     * @template T
     * @param {Token<T>} provide
     * @param {Token<T> | Factory<T>} provider
     * @param {boolean} [beforeInit]
     * @returns {this}
     * @memberof RunnableBuilder
     */
    provider(provide, provider, beforeInit) {
        if (beforeInit) {
            this.beforeInitPds.set(provide, provider);
        }
        else {
            this.afterInitPds.set(provide, provider);
        }
        return this;
    }
    getProvider(provide, beforeInit) {
        if (beforeInit) {
            return this.beforeInitPds.has(provide) ? this.beforeInitPds.get(provide) : null;
        }
        else {
            return this.afterInitPds.has(provide) ? this.afterInitPds.get(provide) : null;
        }
    }
    initContainerPools() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (this.inited) {
                return;
            }
            let container = this.getPools().getDefault();
            yield this.registerExts(container);
            yield this.registerByConfigure(container);
            this.inited = true;
            this.events.emit(RunnableEvents.onRootContainerInited, container);
        });
    }
    load(token, config, options) {
        const _super = Object.create(null, {
            load: { get: () => super.load }
        });
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield this.initContainerPools();
            return yield _super.load.call(this, token, config, options);
        });
    }
    bootstrap(token, config, options) {
        const _super = Object.create(null, {
            bootstrap: { get: () => super.bootstrap }
        });
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let params = this.vaildParams(token, config, options);
            options = params.options || {};
            let injmdl = params.token ? yield this.load(params.token, params.config, options) : yield this.load(params.config, options);
            options.env = injmdl;
            options.bootBuilder = this;
            options.configManager = this.getConfigManager();
            let builder = this.getBuilder(injmdl);
            if (!this.isSame(builder, this)) {
                return params.token ? yield builder.bootstrap(params.token, params.config, options) : yield builder.bootstrap(params.config, options);
            }
            else {
                return params.token ? yield _super.bootstrap.call(this, params.token, params.config, options) : yield _super.bootstrap.call(this, params.config, options);
            }
        });
    }
    isSame(mdb1, mdb2) {
        if (!mdb1 || !mdb2) {
            return false;
        }
        if (mdb1 === mdb2) {
            return true;
        }
        if (core_1.lang.getClass(mdb1) === core_1.lang.getClass(mdb2)) {
            return true;
        }
        return false;
    }
    /**
     * get module builder
     *
     * @param {(Token<T> |RunnableConfigure)} token
     * @param {ModuleEnv} [env]
     * @returns {IModuleBuilder<T>}
     * @memberof RunnableBuilder
     */
    getBuilderByConfig(token, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let injmdl = yield this.load(token, options);
            return this.getBuilder(injmdl);
        });
    }
    getBuilder(injmdl) {
        let cfg = injmdl.config;
        let container = injmdl.container;
        let builder;
        if (cfg) {
            if (core_1.isToken(cfg.builder)) {
                if (core_1.isClass(cfg.builder)) {
                    if (!container.has(cfg.builder)) {
                        container.register(cfg.builder);
                    }
                    if (cfg.builder === core_1.lang.getClass(this)) {
                        builder = this;
                    }
                }
                if (!builder) {
                    builder = container.resolve(cfg.builder);
                }
            }
            else if (cfg.builder instanceof modules.ModuleBuilder) {
                builder = cfg.builder;
            }
        }
        let tko = injmdl.token;
        if (!builder && tko) {
            builder = container.getService(modules.ModuleBuilderToken, tko, (tk) => new modules.InjectModuleBuilderToken(tk), cfg.defaultBuilder || modules.ModuleBuilderToken);
        }
        return builder || this;
    }
    getConfigManager() {
        if (!this.configMgr) {
            this.configMgr = this.createConfigureMgr();
        }
        return this.configMgr;
    }
    createConfigureMgr() {
        let container = this.getPools().getDefault();
        return container.getService(IConfigureManager.ConfigureMgrToken, core_1.lang.getClass(this), { baseURL: this.getRunRoot(container) });
    }
    createDefaultContainer() {
        let container = this.pools.getDefault();
        container.register(BootModule_1.BootModule);
        let chain = container.getBuilder().getInjectorChain(container);
        chain.first(container.resolve(modules.DIModuleInjectorToken));
        chain.first(container.resolve(BootModuleInjector.BootstrapInjectorToken));
        container.bindProvider(utils.ContainerPoolToken, () => this.getPools());
        container.bindProvider(IRunnableBuilder.CurrentRunnableBuilderToken, () => this);
        this.beforeInitPds.forEach((val, key) => {
            container.bindProvider(key, val);
        });
        this.events.emit(RunnableEvents.onRootContainerCreated, container);
        return container;
    }
    /**
     * register ioc exts
     *
     * @protected
     * @param {IContainer} container
     * @param {AppConfigure} config
     * @memberof RunnableBuilder
     */
    registerExts(container) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (this.depModules.length) {
                let types = yield container.loadModule(...this.depModules);
                this.emit(RunnableEvents.registeredExt, types, container);
            }
        });
    }
    /**
     * register by configure.
     *
     * @protected
     * @param {IContainer} container
     * @param {ModuleConfig<T>} config
     * @returns {Promise<void>}
     * @memberof RunnableBuilder
     */
    registerByConfigure(container) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let configManager = this.getConfigManager();
            let config = yield configManager.getConfig();
            if (!config.baseURL) {
                config.baseURL = this.getRunRoot(container);
            }
            yield core_1.PromiseUtil.step(this.customRegs.map(cs => () => tslib_1.__awaiter(this, void 0, void 0, function* () {
                let tokens = yield cs(container, config, this);
                return tokens;
            })));
            let curClass = core_1.lang.getClass(this);
            let registers = [];
            // only run root register.
            container.iteratorServices((serType, fac, resolver) => {
                if (!config.baseURL) {
                    config.baseURL = this.getRunRoot(resolver);
                }
                registers.push({
                    resolver: resolver,
                    serType: serType
                });
            }, ConfigureRegister_1.ConfigureRegister, curClass, true, core_1.ResoveWay.nodes);
            yield Promise.all(registers.map(ser => ser.resolver.resolve(ser.serType).register(config, this)));
        });
    }
    static getClassAnnations() {
        return { "name": "RunnableBuilder", "params": { "constructor": ["baseURL"], "initEvents": [], "on": ["name", "event"], "off": ["name", "event"], "emit": ["name", "args"], "getPools": [], "getRunRoot": ["resolver"], "has": ["key", "aliasOrway"], "resolve": ["token", "resway", "providers"], "createContainerBuilder": [], "use": ["modules"], "provider": ["provide", "provider", "beforeInit"], "getProvider": ["provide", "beforeInit"], "initContainerPools": [], "load": ["token", "config", "options"], "bootstrap": ["token", "config", "options"], "isSame": ["mdb1", "mdb2"], "getBuilderByConfig": ["token", "options"], "getBuilder": ["injmdl"], "getConfigManager": [], "createConfigureMgr": [], "createDefaultContainer": [], "registerExts": ["container"], "registerByConfigure": ["container"] } };
    }
};
RunnableBuilder = tslib_1.__decorate([
    core_1.Injectable(IRunnableBuilder.RunnableBuilderToken),
    tslib_1.__metadata("design:paramtypes", [String])
], RunnableBuilder);
exports.RunnableBuilder = RunnableBuilder;


});

unwrapExports(RunnableBuilder_1);
var RunnableBuilder_2 = RunnableBuilder_1.RunnableEvents;
var RunnableBuilder_3 = RunnableBuilder_1.RunnableBuilder;

var IApplicationBuilder = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });

/**
 *  application builder token.
 */
exports.ApplicationBuilderToken = new core_1.InjectToken('DI_AppBuilder');


});

unwrapExports(IApplicationBuilder);
var IApplicationBuilder_1 = IApplicationBuilder.ApplicationBuilderToken;

var AppConfigure = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });

/**
 * application configuration token.
 */
exports.AppConfigureToken = new core_1.InjectToken('DI_APP_Configuration');


});

unwrapExports(AppConfigure);
var AppConfigure_1 = AppConfigure.AppConfigureToken;

var ApplicationBuilder_1 = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });




/**
 * application events
 *
 * @export
 * @enum {number}
 */
exports.AppEvents = RunnableBuilder_1.RunnableEvents;
/**
 * application events
 */
exports.ApplicationEvents = RunnableBuilder_1.RunnableEvents;
/**
 * application builder.
 *
 * @export
 * @class Default ApplicationBuilder
 * @extends {ModuleBuilder}
 * @template T
 */
let ApplicationBuilder = class ApplicationBuilder extends RunnableBuilder_1.RunnableBuilder {
    constructor(baseURL) {
        super(baseURL);
        this.configs = [];
    }
    /**
     * use configuration.
     *
     * @param {(string | AppConfigure)} [config]
     * @returns {this} global config for this application.
     * @memberof Bootstrap
     */
    useConfiguration(config) {
        if (this.configMgr) {
            this.configMgr.useConfiguration(config);
        }
        this.configs.push(config);
        return this;
    }
    createConfigureMgr() {
        let cfgMgr = super.createConfigureMgr();
        if (this.configs && this.configs.length) {
            this.configs.forEach(cfg => cfgMgr.useConfiguration(cfg));
        }
        return cfgMgr;
    }
    static getClassAnnations() {
        return { "name": "ApplicationBuilder", "params": { "constructor": ["baseURL"], "useConfiguration": ["config"], "createConfigureMgr": [] } };
    }
};
ApplicationBuilder = tslib_1.__decorate([
    core_1.Injectable(IApplicationBuilder.ApplicationBuilderToken),
    tslib_1.__metadata("design:paramtypes", [String])
], ApplicationBuilder);
exports.ApplicationBuilder = ApplicationBuilder;


});

unwrapExports(ApplicationBuilder_1);
var ApplicationBuilder_2 = ApplicationBuilder_1.AppEvents;
var ApplicationBuilder_3 = ApplicationBuilder_1.ApplicationEvents;
var ApplicationBuilder_4 = ApplicationBuilder_1.ApplicationBuilder;

var boot = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });

tslib_1.__exportStar(boots, exports);
tslib_1.__exportStar(BootModuleInjector, exports);
tslib_1.__exportStar(IConfigureManager, exports);
tslib_1.__exportStar(ConfigureManager_1, exports);
tslib_1.__exportStar(ConfigureRegister_1, exports);
tslib_1.__exportStar(IRunnableBuilder, exports);
tslib_1.__exportStar(RunnableBuilder_1, exports);
tslib_1.__exportStar(IApplicationBuilder, exports);
tslib_1.__exportStar(AppConfigure, exports);
tslib_1.__exportStar(ApplicationBuilder_1, exports);


});

unwrapExports(boot);

var Bootstrap = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });




/**
 * create bootstrap decorator.
 *
 * @export
 * @template T
 * @param {string} name
 * @param {Token<IRunnableBuilder<any>>>} [builder] default builder
 * @param {Token<IAnnotationBuilder<any>>} [defaultAnnoBuilder] default type builder.
 * @param {defaultBoot?: Token<any> | ((metadata: T) => Token<any>)} [defaultBoot]
 * @param {MetadataAdapter} [adapter]
 * @param {MetadataExtends<T>} [metadataExtends]
 * @returns {IBootstrapDecorator<T>}
 */
function createBootstrapDecorator(name, defaultBuilder, defaultAnnoBuilder, defaultBoot, adapter, metadataExtends) {
    return DIModule.createDIModuleDecorator(name, defaultBuilder, defaultAnnoBuilder, defaultBoot, adapter, (metadata) => {
        if (metadataExtends) {
            metadataExtends(metadata);
        }
        // static main.
        if (core_1.isClass(metadata.type) && core_1.isFunction(metadata.type['main'])) {
            setTimeout(() => {
                metadata.type['main'](metadata);
            }, 100);
        }
        else if (metadata.bootstrap) {
            setTimeout(() => {
                let builder;
                if (core_1.isClass(metadata.builder) && core_1.lang.isExtendsClass(metadata.builder, boot.RunnableBuilder)) {
                    builder = new metadata.builder();
                }
                if (!builder) {
                    builder = new boot.ApplicationBuilder();
                }
                if (builder instanceof boot.ApplicationBuilder) {
                    if (metadata.bootConfiguration) {
                        builder.useConfiguration(metadata.bootConfiguration);
                    }
                    builder.useConfiguration(core_1.lang.omit(metadata, 'imports', 'exports', 'providers', 'bootDeps', 'bootConfiguration'));
                }
                builder
                    .use(...(metadata.bootDeps || []))
                    .bootstrap(metadata.type);
            }, 100);
        }
        else {
            throw new Error(`boot config error. has not found static main and bootstrap in [class: ${metadata.type.name}]`);
        }
        return metadata;
    });
}
exports.createBootstrapDecorator = createBootstrapDecorator;
/**
 * Bootstrap Decorator, definde class as mvc bootstrap module.
 *
 * @Bootstrap
 */
exports.Bootstrap = createBootstrapDecorator('Bootstrap', boot.ApplicationBuilderToken, IAnnotationBuilder.AnnotationBuilderToken);


});

unwrapExports(Bootstrap);
var Bootstrap_1 = Bootstrap.createBootstrapDecorator;
var Bootstrap_2 = Bootstrap.Bootstrap;

var decorators = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });

tslib_1.__exportStar(Annotation, exports);
tslib_1.__exportStar(DIModule, exports);
tslib_1.__exportStar(Bootstrap, exports);


});

unwrapExports(decorators);

var D__workspace_github_tsioc_packages_bootstrap_es2015 = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });

tslib_1.__exportStar(decorators, exports);
tslib_1.__exportStar(boot, exports);
tslib_1.__exportStar(annotations, exports);
tslib_1.__exportStar(modules, exports);
tslib_1.__exportStar(runnable, exports);
tslib_1.__exportStar(utils, exports);
tslib_1.__exportStar(BootModule_1, exports);


});

var index$6 = unwrapExports(D__workspace_github_tsioc_packages_bootstrap_es2015);

module.exports = index$6;

//# sourceMappingURL=sourcemaps/bootstrap.js.map
