'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var tslib_1 = _interopDefault(require('tslib'));
var core_1 = _interopDefault(require('@ts-ioc/core'));
var reflectMetadata = _interopDefault(require('reflect-metadata'));

function unwrapExports (x) {
	return x && x.__esModule && Object.prototype.hasOwnProperty.call(x, 'default') ? x['default'] : x;
}

function createCommonjsModule(fn, module) {
	return module = { exports: {} }, fn(module, module.exports), module.exports;
}

var IAnnotationBuilder = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });


var annoBuilderDesc = 'DI_AnnotationBuilder';
/**
 * inject Annotation class builder.
 *
 * @export
 * @class InjectBootstrapBuilder
 * @extends {Registration<T>}
 * @template T
 */
var InjectAnnotationBuilder = /** @class */ (function (_super) {
    tslib_1.__extends(InjectAnnotationBuilder, _super);
    function InjectAnnotationBuilder(type) {
        return _super.call(this, type, annoBuilderDesc) || this;
    }
    InjectAnnotationBuilder.getClassAnnations = function () {
        return { "name": "InjectAnnotationBuilder", "params": { "constructor": ["type"] } };
    };
    return InjectAnnotationBuilder;
}(core_1.RefRegistration));
exports.InjectAnnotationBuilder = InjectAnnotationBuilder;
/**
 * Annotation class builder token.
 */
exports.AnnotationBuilderToken = new InjectAnnotationBuilder('');
/**
 * Default Annotation class builder token.
 */
exports.DefaultAnnotationBuilderToken = new InjectAnnotationBuilder('default');


});

unwrapExports(IAnnotationBuilder);
var IAnnotationBuilder_1 = IAnnotationBuilder.InjectAnnotationBuilder;
var IAnnotationBuilder_2 = IAnnotationBuilder.AnnotationBuilderToken;
var IAnnotationBuilder_3 = IAnnotationBuilder.DefaultAnnotationBuilderToken;

var Annotation = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });


/**
 * create type builder decorator
 *
 * @export
 * @template T
 * @param {string} name
 * @param {string} [decorType]
 * @param {(Token<IAnnotationBuilder<any>> | IAnnotationBuilder<any>)} [defaultBuilder]
 * @param {defaultBoot?: Token<any> | ((metadata: T) => Token<any>)} [defaultBoot]
 * @param {MetadataAdapter} [adapter]
 * @param {MetadataExtends<T>} [metadataExtends]
 * @returns {IAnnotationDecorator<T>}
 */
function createAnnotationDecorator(name, defaultBuilder, defaultBoot, adapter, metadataExtends) {
    return core_1.createClassDecorator(name, function (args) {
        if (adapter) {
            adapter(args);
        }
    }, function (metadata) {
        if (metadataExtends) {
            metadataExtends(metadata);
        }
        if (defaultBuilder && !metadata.defaultAnnoBuilder) {
            metadata.defaultAnnoBuilder = defaultBuilder;
        }
        if (!metadata.bootstrap && defaultBoot) {
            var defboot = core_1.isToken(defaultBoot) ? defaultBoot : defaultBoot(metadata);
            if (defboot) {
                metadata.bootstrap = defboot;
            }
        }
        return metadata;
    });
}
exports.createAnnotationDecorator = createAnnotationDecorator;
/**
 * Annotation decorator, use to define class build way via config.
 *
 * @Annotation
 */
exports.Annotation = createAnnotationDecorator('Annotation', IAnnotationBuilder.AnnotationBuilderToken);


});

unwrapExports(Annotation);
var Annotation_1 = Annotation.createAnnotationDecorator;
var Annotation_2 = Annotation.Annotation;

var IModuleBuilder = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });


var moduleBuilderDesc = 'DI_ModuleBuilder';
/**
 * inject module builder token.
 *
 * @export
 * @class InjectModuleBuilder
 * @extends {Registration<T>}
 * @template T
 */
var InjectModuleBuilderToken = /** @class */ (function (_super) {
    tslib_1.__extends(InjectModuleBuilderToken, _super);
    function InjectModuleBuilderToken(type) {
        return _super.call(this, type, moduleBuilderDesc) || this;
    }
    InjectModuleBuilderToken.getClassAnnations = function () {
        return { "name": "InjectModuleBuilderToken", "params": { "constructor": ["type"] } };
    };
    return InjectModuleBuilderToken;
}(core_1.RefRegistration));
exports.InjectModuleBuilderToken = InjectModuleBuilderToken;
/**
 * default module builder token.
 */
exports.ModuleBuilderToken = new InjectModuleBuilderToken(Object);


});

unwrapExports(IModuleBuilder);
var IModuleBuilder_1 = IModuleBuilder.InjectModuleBuilderToken;
var IModuleBuilder_2 = IModuleBuilder.ModuleBuilderToken;

var DIModule = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });



/**
 * create bootstrap decorator.
 *
 * @export
 * @template T
 * @param {string} name decorator name.
 * @param {Token<IModuleBuilder>} [defaultBuilder]
 * @param {Token<IAnnotationBuilder<any>>} [defaultAnnoBuilder]
 * @param {defaultBoot?: Token<any> | ((metadata: T) => Token<any>)} [defaultBoot]
 * @param {MetadataAdapter} [adapter]
 * @param {MetadataExtends<T>} [metadataExtends]
 * @returns {IDIModuleDecorator<T>}
 */
function createDIModuleDecorator(name, defaultBuilder, defaultAnnoBuilder, defaultBoot, adapter, metadataExtends) {
    return core_1.createClassDecorator(name, function (args) {
        if (adapter) {
            adapter(args);
        }
    }, function (metadata) {
        if (metadataExtends) {
            metadataExtends(metadata);
        }
        if (!metadata.name && core_1.isClass(metadata.token)) {
            var isuglify = /^[a-z]$/.test(metadata.token.name);
            var classAnnations = core_1.lang.getClassAnnations(metadata.token);
            if (isuglify && classAnnations) {
                metadata.name = classAnnations.name;
            }
            else {
                metadata.name = metadata.token.name;
            }
        }
        metadata.decorType = name;
        if (defaultBuilder && !metadata.defaultBuilder) {
            metadata.defaultBuilder = defaultBuilder;
        }
        if (defaultAnnoBuilder && !metadata.defaultAnnoBuilder) {
            metadata.defaultAnnoBuilder = defaultAnnoBuilder;
        }
        if (!metadata.bootstrap && defaultBoot) {
            var defboot = core_1.isToken(defaultBoot) ? defaultBoot : defaultBoot(metadata);
            if (defboot) {
                metadata.bootstrap = defboot;
            }
        }
    });
}
exports.createDIModuleDecorator = createDIModuleDecorator;
/**
 * DIModule Decorator, definde class as DI module.
 *
 * @DIModule
 */
exports.DIModule = createDIModuleDecorator('DIModule', IModuleBuilder.ModuleBuilderToken, IAnnotationBuilder.AnnotationBuilderToken);


});

unwrapExports(DIModule);
var DIModule_1 = DIModule.createDIModuleDecorator;
var DIModule_2 = DIModule.DIModule;

var Runnable = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });


/**
 * runnable options token.
 */
exports.RunnableOptionsToken = new core_1.InjectToken('boot_runnable_options');
/**
 * boot.
 *
 * @export
 * @class Boot
 * @implements {IBoot<T>}
 * @template T
 */
var RunnableBase = /** @class */ (function () {
    function RunnableBase(options) {
        this._options = options;
    }
    Object.defineProperty(RunnableBase.prototype, "options", {
        get: function () {
            return this._options;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RunnableBase.prototype, "bootOptions", {
        get: function () {
            return this._bootOptions;
        },
        enumerable: true,
        configurable: true
    });
    RunnableBase.prototype.onInit = function (options, bootOptions) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                if (options) {
                    this._options = options;
                }
                this._bootOptions = bootOptions;
                return [2 /*return*/];
            });
        });
    };
    RunnableBase.prototype.getTarget = function () {
        return this.options.instance;
    };
    RunnableBase.prototype.getModuleToken = function () {
        return this.options.mdToken;
    };
    RunnableBase.prototype.getTargetType = function () {
        return this.options.type || core_1.lang.getClass(this.options.instance);
    };
    RunnableBase.getClassAnnations = function () {
        return { "name": "RunnableBase", "params": { "constructor": ["options"], "onInit": ["options", "bootOptions"], "getTarget": [], "getModuleToken": [], "getTargetType": [] } };
    };
    tslib_1.__decorate([
        core_1.Inject(core_1.ContainerToken),
        tslib_1.__metadata("design:type", Object)
    ], RunnableBase.prototype, "container", void 0);
    return RunnableBase;
}());
exports.RunnableBase = RunnableBase;
/**
 * target is Runnable or not.
 *
 * @export
 * @param {*} target
 * @returns {target is RunnableBase<any>}
 */
function isRunnable(target) {
    if (target instanceof RunnableBase) {
        return true;
    }
    return false;
}
exports.isRunnable = isRunnable;


});

unwrapExports(Runnable);
var Runnable_1 = Runnable.RunnableOptionsToken;
var Runnable_2 = Runnable.RunnableBase;
var Runnable_3 = Runnable.isRunnable;

var Runner_1 = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });



/**
 * boot element.
 *
 * @export
 * @abstract
 * @class Boot
 * @implements {IBoot}
 */
var Runner = /** @class */ (function (_super) {
    tslib_1.__extends(Runner, _super);
    function Runner(options) {
        return _super.call(this, options) || this;
    }
    Runner.getClassAnnations = function () {
        return { "name": "Runner", "params": { "constructor": ["options"], "run": ["data"] } };
    };
    return Runner;
}(Runnable.RunnableBase));
exports.Runner = Runner;
/**
 * target is runner or not.
 *
 * @export
 * @param {*} target
 * @returns {target is IRunner<any>}
 */
function isRunner(target) {
    if (target instanceof Runner) {
        return true;
    }
    if (target && core_1.isFunction(target.run)) {
        return true;
    }
    return false;
}
exports.isRunner = isRunner;


});

unwrapExports(Runner_1);
var Runner_2 = Runner_1.Runner;
var Runner_3 = Runner_1.isRunner;

var Service_1 = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });



/**
 * base service.
 *
 * @export
 * @abstract
 * @class Service
 * @implements {IService}
 */
var Service = /** @class */ (function (_super) {
    tslib_1.__extends(Service, _super);
    function Service(options) {
        return _super.call(this, options) || this;
    }
    Service.getClassAnnations = function () {
        return { "name": "Service", "params": { "constructor": ["options"], "start": ["data"], "stop": [] } };
    };
    return Service;
}(Runnable.RunnableBase));
exports.Service = Service;
/**
 * target is service or not.
 *
 * @export
 * @param {*} target
 * @returns {target is IService<any>}
 */
function isService(target) {
    if (target instanceof Service) {
        return true;
    }
    if (target && core_1.isFunction(target.start)) {
        return true;
    }
    return false;
}
exports.isService = isService;


});

unwrapExports(Service_1);
var Service_2 = Service_1.Service;
var Service_3 = Service_1.isService;

var types = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });


/**
 * application runner token.
 *
 * @export
 * @class InjectRunnerToken
 * @extends {Registration<IRunner<T>>}
 * @template T
 */
var InjectRunnableToken = /** @class */ (function (_super) {
    tslib_1.__extends(InjectRunnableToken, _super);
    function InjectRunnableToken(type) {
        return _super.call(this, type, 'Runnable') || this;
    }
    InjectRunnableToken.getClassAnnations = function () {
        return { "name": "InjectRunnableToken", "params": { "constructor": ["type"] } };
    };
    return InjectRunnableToken;
}(core_1.RefRegistration));
exports.InjectRunnableToken = InjectRunnableToken;
/**
 * default service token.
 */
exports.ServiceToken = new InjectRunnableToken('service');
/**
 * default runner token.
 */
exports.RunnerToken = new InjectRunnableToken('runner');


});

unwrapExports(types);
var types_1 = types.InjectRunnableToken;
var types_2 = types.ServiceToken;
var types_3 = types.RunnerToken;

var runnable = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });

tslib_1.__exportStar(Runnable, exports);
tslib_1.__exportStar(Runner_1, exports);
tslib_1.__exportStar(Service_1, exports);
tslib_1.__exportStar(types, exports);


});

unwrapExports(runnable);

var boots = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });


/**
 * boot
 *
 * @export
 * @abstract
 * @class Boot
 * @extends {Service<T>}
 * @implements {IBoot<T>}
 * @template T
 */
var Boot = /** @class */ (function (_super) {
    tslib_1.__extends(Boot, _super);
    function Boot() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    /**
     * on boot init.
     *
     * @param {RunnableOptions<T>} options
     * @param {RunOptions<T>} bootOptions
     * @returns {Promise<void>}
     * @memberof IBoot
     */
    Boot.prototype.onInit = function (options, bootOptions) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, _super.prototype.onInit.call(this, options, bootOptions)];
                    case 1:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    /**
     * run boot.
     *
     * @abstract
     * @param {*} [data]
     * @returns {Promise<any>}
     * @memberof Boot
     */
    Boot.prototype.run = function (data) {
        return this.start(data);
    };
    Boot.getClassAnnations = function () {
        return { "name": "Boot", "params": { "onInit": ["options", "bootOptions"], "run": ["data"] } };
    };
    return Boot;
}(runnable.Service));
exports.Boot = Boot;


});

unwrapExports(boots);
var boots_1 = boots.Boot;

var DIModuleValidate = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });


/**
 * DIModule Validate Token
 */
exports.DIModuleValidateToken = new core_1.InjectModuleValidateToken('@DIModule');
/**
 * DIModuel Validate
 *
 * @export
 * @class DIModuelValidate
 * @extends {ModuelValidate}
 */
var DIModuelValidate = /** @class */ (function (_super) {
    tslib_1.__extends(DIModuelValidate, _super);
    function DIModuelValidate() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    DIModuelValidate.prototype.getDecorator = function () {
        return '@DIModule';
    };
    DIModuelValidate.getClassAnnations = function () {
        return { "name": "DIModuelValidate", "params": { "getDecorator": [] } };
    };
    DIModuelValidate = tslib_1.__decorate([
        core_1.Singleton(exports.DIModuleValidateToken)
    ], DIModuelValidate);
    return DIModuelValidate;
}(core_1.ModuelValidate));
exports.DIModuelValidate = DIModuelValidate;


});

unwrapExports(DIModuleValidate);
var DIModuleValidate_1 = DIModuleValidate.DIModuleValidateToken;
var DIModuleValidate_2 = DIModuleValidate.DIModuelValidate;

var ContainerPool_1 = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });

var rootContainer = '__ioc_root_container';
/**
 * container pool
 *
 * @export
 * @class ContainerPool
 */
var ContainerPool = /** @class */ (function () {
    function ContainerPool(containerBuilder) {
        this.containerBuilder = containerBuilder;
        this.pools = new Map();
    }
    ContainerPool.prototype.createContainer = function () {
        return this.containerBuilder.create();
    };
    ContainerPool.prototype.getTokenKey = function (token) {
        if (token instanceof core_1.Registration) {
            return token.toString();
        }
        return token;
    };
    ContainerPool.prototype.isDefault = function (container) {
        return container === this._default;
    };
    ContainerPool.prototype.hasDefault = function () {
        return !!this._default;
    };
    ContainerPool.prototype.getDefault = function () {
        if (!this._default) {
            this._default = this.createContainer();
            this.pools.set(rootContainer, [this._default]);
        }
        return this._default;
    };
    ContainerPool.prototype.set = function (token, container) {
        var key = this.getTokenKey(token);
        if (this.pools.has(key)) {
            this.pools.get(key).push(container);
        }
        else {
            this.pools.set(key, [container]);
        }
    };
    ContainerPool.prototype.get = function (token) {
        var key = this.getTokenKey(token);
        if (!this.has(key)) {
            return null;
        }
        return this.pools.get(token);
    };
    ContainerPool.prototype.has = function (token) {
        return this.pools.has(this.getTokenKey(token));
    };
    ContainerPool.prototype.create = function (token, parent) {
        parent = parent || this.getDefault();
        var container = parent.getBuilder().create();
        this.setParent(container, parent);
        this.set(token, container);
        return container;
    };
    ContainerPool.prototype.setParent = function (container, parent) {
        if (this.isDefault(container)) {
            return;
        }
        parent = parent || this.getDefault();
        if (parent && parent !== container) {
            if (container.parent) {
                container.parent.children.splice(container.parent.children.indexOf(container), 1);
            }
            parent.children.push(container);
            container.parent = parent;
        }
    };
    ContainerPool.prototype.keys = function () {
        return Array.from(this.pools.keys());
    };
    ContainerPool.prototype.values = function () {
        return Array.from(this.pools.values()).reduce(function (p, c) { return p.concat(c); }, []);
    };
    ContainerPool.prototype.iterator = function (express, root) {
        root = root || this.getDefault();
        express(root);
        this.iteratorChildren(express, root.children);
    };
    ContainerPool.prototype.iteratorChildren = function (express, children) {
        var _this = this;
        if (children && children.length) {
            children.forEach(function (c) {
                express(c);
                _this.iteratorChildren(express, c.children);
            });
        }
    };
    ContainerPool.getClassAnnations = function () {
        return { "name": "ContainerPool", "params": { "constructor": ["containerBuilder"], "createContainer": [], "getTokenKey": ["token"], "isDefault": ["container"], "hasDefault": [], "getDefault": [], "set": ["token", "container"], "get": ["token"], "has": ["token"], "create": ["token", "parent"], "setParent": ["container", "parent"], "keys": [], "values": [], "iterator": ["express", "root"], "iteratorChildren": ["express", "children"] } };
    };
    return ContainerPool;
}());
exports.ContainerPool = ContainerPool;
/**
 *  container pool token.
 */
exports.ContainerPoolToken = new core_1.InjectToken('DI_ContainerPool');


});

unwrapExports(ContainerPool_1);
var ContainerPool_2 = ContainerPool_1.ContainerPool;
var ContainerPool_3 = ContainerPool_1.ContainerPoolToken;

var Events_1 = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });

/**
 * custom events.
 *
 * @export
 * @class Events
 */
var Events = /** @class */ (function () {
    function Events() {
        this.eventsMap = {};
    }
    Events.prototype.on = function (name, event) {
        this.eventsMap[name] = this.eventsMap[name] || [];
        if (this.eventsMap[name].indexOf(event) < 0) {
            this.eventsMap[name].push(event);
        }
        return this;
    };
    Events.prototype.once = function (name, event) {
        var _this = this;
        var onceEvent = function () {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }
            event.apply(void 0, args);
            _this.off(name, onceEvent);
        };
        this.on(name, onceEvent);
    };
    Events.prototype.off = function (name, event) {
        if (this.eventsMap[name]) {
            if (event) {
                this.eventsMap[name].splice(this.eventsMap[name].indexOf(event), 1);
            }
            else {
                delete this.eventsMap[name];
            }
        }
        return this;
    };
    Events.prototype.emit = function (name) {
        var args = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            args[_i - 1] = arguments[_i];
        }
        var events = this.eventsMap[name];
        if (core_1.isArray(events)) {
            events.forEach(function (ev) {
                ev.apply(void 0, args);
            });
        }
    };
    Events.getClassAnnations = function () {
        return { "name": "Events", "params": { "constructor": [], "on": ["name", "event"], "once": ["name", "event"], "off": ["name", "event"], "emit": ["name", "args"] } };
    };
    return Events;
}());
exports.Events = Events;


});

unwrapExports(Events_1);
var Events_2 = Events_1.Events;

var types$2 = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });


/**
 * is target di module class or not.
 *
 * @export
 * @param {*} target
 * @returns
 */
function isDIModuleClass(target) {
    return core_1.isClass(target) && core_1.hasOwnClassMetadata(DIModule.DIModule, target);
}
exports.isDIModuleClass = isDIModuleClass;


});

unwrapExports(types$2);
var types_1$1 = types$2.isDIModuleClass;

var utils = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });

tslib_1.__exportStar(ContainerPool_1, exports);
tslib_1.__exportStar(Events_1, exports);
tslib_1.__exportStar(types$2, exports);


});

unwrapExports(utils);

var InjectedModule_1 = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });


/**
 * injected module.
 *
 * @export
 * @class InjectedModule
 * @template T
 */
var InjectedModule = /** @class */ (function () {
    function InjectedModule(token, config, container, type, exports) {
        this.token = token;
        this.config = config;
        this.container = container;
        this.type = type;
        this.exports = exports;
    }
    InjectedModule.prototype.getProviderMap = function () {
        if (!this.exports) {
            this.exports = this.container;
        }
        return this.exports;
    };
    Object.defineProperty(InjectedModule.prototype, "size", {
        get: function () {
            return this.getProviderMap().size;
        },
        enumerable: true,
        configurable: true
    });
    InjectedModule.prototype.resolve = function (token, resway) {
        var providers = [];
        for (var _i = 2; _i < arguments.length; _i++) {
            providers[_i - 2] = arguments[_i];
        }
        var _a;
        var way;
        if (core_1.isNumber(resway)) {
            way = resway;
        }
        else {
            way = core_1.ResoveWay.all;
            if (resway) {
                providers.unshift(resway);
            }
        }
        var pdr = this.getProviderMap();
        if (pdr && (way & core_1.ResoveWay.current) && pdr.has(token)) {
            return pdr.resolve.apply(pdr, [token, core_1.ResoveWay.current].concat(providers));
        }
        if (pdr !== this.container && (way & core_1.ResoveWay.traverse)) {
            return (_a = this.container).resolve.apply(_a, [token, core_1.ResoveWay.traverse].concat(providers));
        }
        return null;
    };
    InjectedModule.prototype.has = function (token, aliasOrway) {
        var key = this.container.getTokenKey(token, core_1.isString(aliasOrway) ? aliasOrway : null);
        var pdr = this.getProviderMap();
        var resway = core_1.isNumber(aliasOrway) ? aliasOrway : core_1.ResoveWay.all;
        if (pdr && (resway & core_1.ResoveWay.current) && pdr.has(key)) {
            return true;
        }
        if (pdr !== this.container && (resway & core_1.ResoveWay.traverse)) {
            return this.container.has(key, core_1.ResoveWay.traverse);
        }
        return false;
    };
    InjectedModule.prototype.getTokenImpl = function (token, resway) {
        return this.container.getTokenImpl(token, resway || core_1.ResoveWay.nodes);
    };
    InjectedModule.prototype.unregister = function (token, resway) {
        var key = this.container.getTokenKey(token);
        var pdr = this.getProviderMap();
        resway = core_1.isNumber(resway) ? resway : core_1.ResoveWay.nodes;
        if (pdr && (resway & core_1.ResoveWay.current)) {
            pdr.unregister(key, core_1.ResoveWay.current);
        }
        if (resway & core_1.ResoveWay.traverse) {
            this.container.unregister(key, core_1.ResoveWay.nodes);
        }
        return this;
    };
    InjectedModule.prototype.forEach = function (callbackfn) {
        var _this = this;
        var pdr = this.getProviderMap();
        if (pdr) {
            if (pdr.forEach(function (fac, tk) {
                if (core_1.isToken(tk)) {
                    return callbackfn(tk, fac, pdr);
                }
            }) === false) {
                return false;
            }
        }
        return this.container.forEach(function (tk, fac) {
            return callbackfn(tk, fac, _this.container);
        });
    };
    InjectedModule.getClassAnnations = function () {
        return { "name": "InjectedModule", "params": { "constructor": ["token", "config", "container", "type", "exports"], "getProviderMap": [], "resolve": ["token", "resway", "providers"], "has": ["token", "aliasOrway"], "getTokenImpl": ["token", "resway"], "unregister": ["token", "resway"], "forEach": ["callbackfn"] } };
    };
    return InjectedModule;
}());
exports.InjectedModule = InjectedModule;
/**
 * Injected Module Token.
 *
 * @export
 * @class InjectModuleMetaConfigToken
 * @extends {Registration<Type<T>>}
 * @template T
 */
var InjectedModuleToken = /** @class */ (function (_super) {
    tslib_1.__extends(InjectedModuleToken, _super);
    function InjectedModuleToken(type) {
        return _super.call(this, type, 'InjectedModule') || this;
    }
    InjectedModuleToken.getClassAnnations = function () {
        return { "name": "InjectedModuleToken", "params": { "constructor": ["type"] } };
    };
    return InjectedModuleToken;
}(core_1.Registration));
exports.InjectedModuleToken = InjectedModuleToken;


});

unwrapExports(InjectedModule_1);
var InjectedModule_2 = InjectedModule_1.InjectedModule;
var InjectedModule_3 = InjectedModule_1.InjectedModuleToken;

var ConfigureRegister_1 = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });


/**
 * configure register.
 *
 * @export
 * @abstract
 * @class ConfigureRegister
 * @implements {IConfigureRegister<T>}
 * @template T
 */
var ConfigureRegister = /** @class */ (function () {
    function ConfigureRegister() {
    }
    ConfigureRegister.getClassAnnations = function () {
        return { "name": "ConfigureRegister", "params": { "constructor": [], "register": ["config", "runBuilder"] } };
    };
    tslib_1.__decorate([
        core_1.Inject(core_1.ContainerToken),
        tslib_1.__metadata("design:type", Object)
    ], ConfigureRegister.prototype, "container", void 0);
    ConfigureRegister = tslib_1.__decorate([
        core_1.Abstract(),
        tslib_1.__metadata("design:paramtypes", [])
    ], ConfigureRegister);
    return ConfigureRegister;
}());
exports.ConfigureRegister = ConfigureRegister;


});

unwrapExports(ConfigureRegister_1);
var ConfigureRegister_2 = ConfigureRegister_1.ConfigureRegister;

var IConfigureManager = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });

/**
 * configure manager token.
 */
exports.ConfigureMgrToken = new core_1.InjectToken('config-mgr');
/**
 * default configuration token.
 */
exports.DefaultConfigureToken = new core_1.InjectToken('DI_Default_Configuration');
/**
 * configure loader token.
 */
exports.ConfigureLoaderToken = new core_1.InjectToken('DI_Configure_Loader');


});

unwrapExports(IConfigureManager);
var IConfigureManager_1 = IConfigureManager.ConfigureMgrToken;
var IConfigureManager_2 = IConfigureManager.DefaultConfigureToken;
var IConfigureManager_3 = IConfigureManager.ConfigureLoaderToken;

var IRunnableBuilder = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });

/**
 *  process run root.
 */
exports.ProcessRunRootToken = new core_1.InjectToken('ioc_processRunRoot');
/**
 * runable builder token.
 */
exports.RunnableBuilderToken = new core_1.InjectToken('DI_RunnableBuilder');
exports.CurrentRunnableBuilderToken = new core_1.InjectToken('Current_DI_RunnableBuilder');


});

unwrapExports(IRunnableBuilder);
var IRunnableBuilder_1 = IRunnableBuilder.ProcessRunRootToken;
var IRunnableBuilder_2 = IRunnableBuilder.RunnableBuilderToken;
var IRunnableBuilder_3 = IRunnableBuilder.CurrentRunnableBuilderToken;

var DIModuleInjector_1 = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });








/**
 * DIModule injector token.
 */
exports.DIModuleInjectorToken = new core_1.InjectModuleInjectorToken('@DIModule');
/**
 * DIModule injector.
 *
 * @export
 * @class DIModuleInjector
 * @extends {ModuleInjector}
 */
var DIModuleInjector = /** @class */ (function (_super) {
    tslib_1.__extends(DIModuleInjector, _super);
    function DIModuleInjector(validate) {
        return _super.call(this, validate) || this;
    }
    DIModuleInjector.prototype.setup = function (container, type) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.importModule(container, type)];
                    case 1:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    DIModuleInjector.prototype.syncSetup = function (container, type) {
        throw new Error('DIModule can not sync setup.');
    };
    DIModuleInjector.prototype.valid = function (container, type) {
        if (!this.validate) {
            return true;
        }
        return this.validate.valid(type);
    };
    DIModuleInjector.prototype.getMetaAccessor = function (container, token) {
        return container.getService(core_1.MetaAccessorToken, token);
    };
    DIModuleInjector.prototype.importModule = function (container, type) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var pools, decorator, accor, metaConfig, newContainer, exps, injMd;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        pools = container.get(utils.ContainerPoolToken);
                        decorator = this.validate.getDecorator();
                        accor = this.getMetaAccessor(container, decorator);
                        metaConfig = accor.getMetadata(type, container, undefined, decorator ? function (dec) { return dec === decorator; } : undefined);
                        newContainer = metaConfig.asRoot === true ? pools.getDefault() : pools.create(type, container);
                        newContainer.register(type);
                        return [4 /*yield*/, this.registerConfgureDepds(newContainer, metaConfig, type)];
                    case 1:
                        _a.sent();
                        return [4 /*yield*/, this.getConfigExports(newContainer, metaConfig)];
                    case 2:
                        exps = _a.sent();
                        injMd = new InjectedModule_1.InjectedModule(metaConfig.token || type, metaConfig, newContainer, type, exps);
                        container.bindProvider(new core_1.InjectReference(core_1.ProviderMap, type), exps);
                        container.bindProvider(new InjectedModule_1.InjectedModuleToken(type), injMd);
                        if (metaConfig.asRoot) {
                            return [2 /*return*/, injMd];
                        }
                        return [4 /*yield*/, this.registerConfigExports(container, newContainer, injMd)];
                    case 3:
                        _a.sent();
                        if (!!pools.isDefault(newContainer)) return [3 /*break*/, 5];
                        return [4 /*yield*/, this.registerConfigrue(newContainer, injMd)];
                    case 4:
                        _a.sent();
                        _a.label = 5;
                    case 5: return [2 /*return*/, injMd];
                }
            });
        });
    };
    DIModuleInjector.prototype.registerConfgureDepds = function (container, config, type) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!(core_1.isArray(config.imports) && config.imports.length)) return [3 /*break*/, 2];
                        return [4 /*yield*/, container.loadModule.apply(container, config.imports)];
                    case 1:
                        _a.sent();
                        _a.label = 2;
                    case 2: return [2 /*return*/];
                }
            });
        });
    };
    DIModuleInjector.prototype.getConfigExports = function (container, config) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var parser, map, builder, exptypes;
            return tslib_1.__generator(this, function (_a) {
                parser = container.getProviderParser();
                map = parser.parse.apply(parser, config.providers || []);
                // bind module providers
                container.bindProviders(map);
                builder = container.getBuilder();
                exptypes = [].concat.apply([], builder.loader.getTypes(config.exports || []));
                exptypes.forEach(function (ty) {
                    var classPd = container.resolveValue(new core_1.InjectClassProvidesToken(ty));
                    map.add(ty, function () {
                        var pds = [];
                        for (var _i = 0; _i < arguments.length; _i++) {
                            pds[_i] = arguments[_i];
                        }
                        return container.resolve.apply(container, [ty, core_1.ResoveWay.nodes].concat(pds));
                    });
                    if (classPd && core_1.isArray(classPd.provides) && classPd.provides.length) {
                        classPd.provides.forEach(function (p) {
                            if (!map.has(p)) {
                                map.add(p, function () {
                                    var pds = [];
                                    for (var _i = 0; _i < arguments.length; _i++) {
                                        pds[_i] = arguments[_i];
                                    }
                                    return container.resolve.apply(container, [p, core_1.ResoveWay.nodes].concat(pds));
                                });
                            }
                        });
                    }
                });
                return [2 /*return*/, map];
            });
        });
    };
    DIModuleInjector.prototype.registerConfigExports = function (container, newContainer, injMd) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var chain;
            return tslib_1.__generator(this, function (_a) {
                if (container === newContainer) {
                    return [2 /*return*/, container];
                }
                if (injMd) {
                    chain = container.getResolvers();
                    chain.next(injMd);
                }
                return [2 /*return*/, container];
            });
        });
    };
    DIModuleInjector.prototype.registerConfigrue = function (newContainer, injMd) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var registers, mgr, appConfig_1;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        registers = [];
                        newContainer.iteratorServices(function (serType, fac, resolver) {
                            registers.push({
                                resolver: resolver,
                                serType: serType
                            });
                        }, ConfigureRegister_1.ConfigureRegister, injMd.type, true, core_1.ResoveWay.current);
                        if (!registers.length) return [3 /*break*/, 3];
                        mgr = newContainer.get(IConfigureManager.ConfigureMgrToken);
                        return [4 /*yield*/, mgr.getConfig()];
                    case 1:
                        appConfig_1 = _a.sent();
                        return [4 /*yield*/, Promise.all(registers.map(function (ser) { return ser.resolver.resolve(ser.serType).register(appConfig_1, ser.resolver.resolve(IRunnableBuilder.CurrentRunnableBuilderToken)); }))];
                    case 2:
                        _a.sent();
                        _a.label = 3;
                    case 3: return [2 /*return*/];
                }
            });
        });
    };
    DIModuleInjector.getClassAnnations = function () {
        return { "name": "DIModuleInjector", "params": { "constructor": ["validate"], "setup": ["container", "type"], "syncSetup": ["container", "type"], "valid": ["container", "type"], "getMetaAccessor": ["container", "token"], "importModule": ["container", "type"], "registerConfgureDepds": ["container", "config", "type"], "getConfigExports": ["container", "config"], "registerConfigExports": ["container", "newContainer", "injMd"], "registerConfigrue": ["newContainer", "injMd"] } };
    };
    DIModuleInjector = tslib_1.__decorate([
        core_1.Singleton(exports.DIModuleInjectorToken),
        tslib_1.__param(0, core_1.Inject(DIModuleValidate.DIModuleValidateToken)),
        tslib_1.__metadata("design:paramtypes", [Object])
    ], DIModuleInjector);
    return DIModuleInjector;
}(core_1.ModuleInjector));
exports.DIModuleInjector = DIModuleInjector;


});

unwrapExports(DIModuleInjector_1);
var DIModuleInjector_2 = DIModuleInjector_1.DIModuleInjectorToken;
var DIModuleInjector_3 = DIModuleInjector_1.DIModuleInjector;

var AnnoBuildStrategy_1 = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });


/**
 * annotation build strategy token inject.
 *
 * @export
 * @class InjectAnnoBuildStrategyToken
 * @extends {RefRegistration<IAnnoBuildStrategy<T>>}
 * @template T
 */
var InjectAnnoBuildStrategyToken = /** @class */ (function (_super) {
    tslib_1.__extends(InjectAnnoBuildStrategyToken, _super);
    function InjectAnnoBuildStrategyToken(type) {
        return _super.call(this, type, 'AnnoBuildStrategy') || this;
    }
    InjectAnnoBuildStrategyToken.getClassAnnations = function () {
        return { "name": "InjectAnnoBuildStrategyToken", "params": { "constructor": ["type"] } };
    };
    return InjectAnnoBuildStrategyToken;
}(core_1.RefRegistration));
exports.InjectAnnoBuildStrategyToken = InjectAnnoBuildStrategyToken;
/**
 *  base AnnoBuildStrategy Token
 */
exports.AnnoBuildStrategyToken = new InjectAnnoBuildStrategyToken(Object);
/**
 * annotation instance build strategy.
 *
 * @export
 * @class AnnoBuildStrategy
 * @implements {IAnnoBuildStrategy<T>}
 * @template T
 */
var AnnoBuildStrategy = /** @class */ (function () {
    function AnnoBuildStrategy() {
    }
    AnnoBuildStrategy.prototype.build = function (instance, config, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!instance) {
                            return [2 /*return*/];
                        }
                        return [4 /*yield*/, this.beforeInit(instance, config)];
                    case 1:
                        _a.sent();
                        return [4 /*yield*/, this.process(instance, config, options)];
                    case 2:
                        _a.sent();
                        return [4 /*yield*/, this.afterInit(instance, config)];
                    case 3:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    /**
     * after annotation instance init.
     *
     * @protected
     * @param {BootHooks<T>} instance
     * @param {AnnotationConfigure<T>} config
     * @returns {Promise<void>}
     * @memberof AnnoBuildStrategy
     */
    AnnoBuildStrategy.prototype.beforeInit = function (instance, config) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!core_1.isFunction(instance.anBeforeInit)) return [3 /*break*/, 2];
                        return [4 /*yield*/, instance.anBeforeInit(config)];
                    case 1:
                        _a.sent();
                        _a.label = 2;
                    case 2: return [2 /*return*/];
                }
            });
        });
    };
    /**
     * build annotation instance process.
     *
     * @protected
     * @param {T} instance
     * @param {AnnotationConfigure<T>} config
     * @param {BuildOptions<T>} [options]
     * @returns {Promise<void>}
     * @memberof AnnoBuildStrategy
     */
    AnnoBuildStrategy.prototype.process = function (instance, config, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                return [2 /*return*/];
            });
        });
    };
    /**
     * after annotation instance init.
     *
     * @protected
     * @param {BootHooks<T>} instance
     * @param {AnnotationConfigure<T>} config
     * @returns {Promise<void>}
     * @memberof AnnoBuildStrategy
     */
    AnnoBuildStrategy.prototype.afterInit = function (instance, config) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!core_1.isFunction(instance.anAfterInit)) return [3 /*break*/, 2];
                        return [4 /*yield*/, instance.anAfterInit(config)];
                    case 1:
                        _a.sent();
                        _a.label = 2;
                    case 2: return [2 /*return*/];
                }
            });
        });
    };
    AnnoBuildStrategy.getClassAnnations = function () {
        return { "name": "AnnoBuildStrategy", "params": { "build": ["instance", "config", "options"], "beforeInit": ["instance", "config"], "process": ["instance", "config", "options"], "afterInit": ["instance", "config"] } };
    };
    AnnoBuildStrategy = tslib_1.__decorate([
        core_1.Singleton(exports.AnnoBuildStrategyToken)
    ], AnnoBuildStrategy);
    return AnnoBuildStrategy;
}());
exports.AnnoBuildStrategy = AnnoBuildStrategy;


});

unwrapExports(AnnoBuildStrategy_1);
var AnnoBuildStrategy_2 = AnnoBuildStrategy_1.InjectAnnoBuildStrategyToken;
var AnnoBuildStrategy_3 = AnnoBuildStrategy_1.AnnoBuildStrategyToken;
var AnnoBuildStrategy_4 = AnnoBuildStrategy_1.AnnoBuildStrategy;

var AnnotationBuilder_1 = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });





/**
 * Annotation class builder. build class with metadata and config.
 *
 * @export
 * @class AnnotationBuilder
 * @implements {implements IAnnotationBuilder<T>}
 * @template T
 */
var AnnotationBuilder = /** @class */ (function () {
    function AnnotationBuilder() {
    }
    AnnotationBuilder_1 = AnnotationBuilder;
    /**
     * get metadata accessor.
     *
     * @param {(Token<any>|| AnnotationConfigure<T>)} token
     * @param {AnnotationConfigure<T>} [config]
     * @returns {IMetaAccessor<any>}
     * @memberof AnnotationBuilder
     */
    AnnotationBuilder.prototype.getMetaAccessor = function (token, config) {
        var mtk;
        if (core_1.isToken(token)) {
            mtk = token;
        }
        else {
            config = token;
        }
        return this.container.getService(core_1.MetaAccessorToken, mtk ? [mtk, core_1.lang.getClass(this)] : core_1.lang.getClass(this), function (tk) { return new core_1.InjectMetaAccessorToken(tk); }, config ? (config.defaultMetaAccessor || core_1.MetaAccessorToken) : core_1.MetaAccessorToken);
    };
    /**
     * build token type with config.
     *
     * @param {(Token<T> | AnnotationConfigure<T>)} token
     * @param {(AnnotationConfigure<T> | BuildOptions<T>)} [config]
     * @param {BuildOptions<T>} [options]
     * @returns {Promise<T>}
     * @memberof AnnotationBuilder
     */
    AnnotationBuilder.prototype.build = function (token, config, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var tk, cfg, accessor, builder, instance, strategy;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (core_1.isToken(token)) {
                            tk = token;
                            if (core_1.isUndefined(options)) {
                                options = config;
                                cfg = null;
                            }
                            else {
                                cfg = config;
                            }
                            cfg = this.getMetaAccessor(tk, cfg).getMetadata(tk, this.container, cfg);
                        }
                        else {
                            options = config;
                            cfg = token;
                            accessor = this.getMetaAccessor(cfg);
                            tk = accessor.getToken(cfg, this.container);
                            if (core_1.isToken(tk)) {
                                cfg = accessor.getMetadata(tk, this.container, cfg);
                            }
                        }
                        if (core_1.isClass(tk) && !this.container.hasRegister(tk)) {
                            this.container.register(tk);
                        }
                        if (options && core_1.isFunction(options.vaild) && !options.vaild(tk)) {
                            return [2 /*return*/, null];
                        }
                        builder = (options && options.builder) ? options.builder : this.getBuilder(tk, cfg);
                        if (!!this.isEqual(builder)) return [3 /*break*/, 2];
                        return [4 /*yield*/, builder.build(tk, cfg, core_1.lang.assign(options || {}, { builder: builder }))];
                    case 1: return [2 /*return*/, _a.sent()];
                    case 2: return [4 /*yield*/, this.registerExts(cfg)];
                    case 3:
                        _a.sent();
                        return [4 /*yield*/, this.createInstance(tk, cfg, options)];
                    case 4:
                        instance = _a.sent();
                        if (!instance) {
                            return [2 /*return*/, null];
                        }
                        strategy = this.container.getService(AnnoBuildStrategy_1.AnnoBuildStrategyToken, [
                            tk || core_1.lang.getClass(instance)
                        ].concat(((options && options.target) ? [{ target: core_1.lang.getClass(options.target), level: core_1.RefTagLevel.self }] : []), [
                            { target: core_1.lang.getClass(this), level: core_1.RefTagLevel.self }
                        ]), function (tk) { return new AnnoBuildStrategy_1.InjectAnnoBuildStrategyToken(tk); });
                        if (!strategy) return [3 /*break*/, 6];
                        return [4 /*yield*/, strategy.build(instance, cfg, options)];
                    case 5:
                        _a.sent();
                        _a.label = 6;
                    case 6:
                        if (options && core_1.isFunction(options.onCompleted)) {
                            options.onCompleted(tk, cfg, instance, this);
                        }
                        return [2 /*return*/, instance];
                }
            });
        });
    };
    /**
     * run runable.
     *
     * @param {(Token<T> | AnnotationConfigure<T>)} runable
     * @param {*} [data] bootstrap data, build data, Runnable data.
     * @returns {Promise<Runnable<T>>}
     * @memberof IGModuleBuilder
     */
    AnnotationBuilder.prototype.boot = function (runable, config, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var instance, builder, tk, cfg, onCompleted, data, runOptions, runner;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        onCompleted = function (token, config, hook, builder) {
                            tk = token;
                            cfg = config;
                            instance = hook;
                            builder = builder;
                        };
                        if (!core_1.isToken(runable)) return [3 /*break*/, 2];
                        if (core_1.isUndefined(options)) {
                            options = config;
                            cfg = null;
                        }
                        else {
                            cfg = config;
                        }
                        return [4 /*yield*/, this.build(runable, cfg, core_1.lang.assign({
                                onCompleted: onCompleted
                            }, options))];
                    case 1:
                        _a.sent();
                        return [3 /*break*/, 4];
                    case 2: return [4 /*yield*/, this.build(runable, core_1.lang.assign({
                            onCompleted: onCompleted
                        }, config))];
                    case 3:
                        _a.sent();
                        _a.label = 4;
                    case 4:
                        builder = builder || this;
                        data = options ? options.data : undefined;
                        runOptions = { instance: instance, type: core_1.lang.getClass(instance), mdToken: tk, config: cfg, data: data };
                        runner = builder.resolveRunable(instance, runOptions, options);
                        if (!(runner && core_1.isFunction(runner.onInit))) return [3 /*break*/, 6];
                        return [4 /*yield*/, runner.onInit(runOptions, options)];
                    case 5:
                        _a.sent();
                        _a.label = 6;
                    case 6:
                        if (!runnable.isRunner(runner)) return [3 /*break*/, 8];
                        return [4 /*yield*/, runner.run(data)];
                    case 7:
                        _a.sent();
                        return [3 /*break*/, 10];
                    case 8:
                        if (!runnable.isService(runner)) return [3 /*break*/, 10];
                        return [4 /*yield*/, runner.start(data)];
                    case 9:
                        _a.sent();
                        _a.label = 10;
                    case 10: return [2 /*return*/, runner];
                }
            });
        });
    };
    /**
     * run annotation instance.
     *
     * @param {T} instance
     * @param {AnnotationConfigure<T>} [config]
     * @param {Token<T>} [token]
     * @returns {Promise<Runnable<T>>}
     * @memberof AnnotationBuilder
     */
    AnnotationBuilder.prototype.resolveRunable = function (instance, runableOptions, options) {
        if (!instance) {
            return null;
        }
        var tk = runableOptions.mdToken || runableOptions.type;
        if (!runableOptions.config) {
            runableOptions.config = this.getMetaAccessor(tk).getMetadata(tk, this.container);
        }
        if (instance instanceof runnable.Runner || instance instanceof runnable.Service) {
            return instance;
        }
        else {
            var provider = { provide: runnable.RunnableOptionsToken, useValue: runableOptions.config };
            return this.container.getService([runnable.RunnerToken, runnable.ServiceToken], [
                tk
            ].concat(((options && options.target) ? [{ target: core_1.lang.getClass(options.target), level: core_1.RefTagLevel.self }] : []), [
                { target: core_1.lang.getClass(this), level: core_1.RefTagLevel.self }
            ]), function (tk) { return new runnable.InjectRunnableToken(tk); }, runableOptions.config.defaultRunnable || true, provider);
        }
    };
    AnnotationBuilder.prototype.createInstance = function (token, config, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var instance;
            return tslib_1.__generator(this, function (_a) {
                if (!core_1.isToken(token)) {
                    return [2 /*return*/, null];
                }
                instance = this.resolveToken.apply(this, [token, options ? options.target : undefined].concat((options.providers || [])));
                if (core_1.isNullOrUndefined(instance)) {
                    console.log("can not find token " + (token ? token.toString() : null) + " in container.");
                    return [2 /*return*/, null];
                }
                return [2 /*return*/, instance];
            });
        });
    };
    AnnotationBuilder.prototype.getBuilder = function (token, config, options) {
        var builder;
        if (config && config.annoBuilder) {
            if (core_1.isClass(config.annoBuilder)) {
                if (!this.container.has(config.annoBuilder)) {
                    this.container.register(config.annoBuilder);
                }
            }
            if (core_1.isToken(config.annoBuilder)) {
                builder = this.container.resolve(config.annoBuilder);
            }
            else if (config.annoBuilder instanceof AnnotationBuilder_1) {
                builder = config.annoBuilder;
            }
        }
        if (options && options.builder) {
            builder = options.builder;
        }
        if (!builder && token) {
            builder = this.container.getService(IAnnotationBuilder.AnnotationBuilderToken, [
                token
            ].concat((options && options.target) ? [{ target: core_1.lang.getClass(options.target), level: core_1.RefTagLevel.self }] : []), function (tk) { return new IAnnotationBuilder.InjectAnnotationBuilder(tk); }, config.defaultAnnoBuilder || IAnnotationBuilder.AnnotationBuilderToken);
        }
        if (builder && !builder.container) {
            builder.container = this.container;
        }
        return builder || this;
    };
    /**
     * register extension before create instance.
     *
     * @protected
     * @param {AnnotationConfigure<T>} [config]
     * @memberof AnnotationBuilder
     */
    AnnotationBuilder.prototype.registerExts = function (config) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                return [2 /*return*/];
            });
        });
    };
    AnnotationBuilder.prototype.isEqual = function (build) {
        if (!build) {
            return false;
        }
        if (build === this) {
            return true;
        }
        if (core_1.lang.getClass(build) === core_1.lang.getClass(this)) {
            return true;
        }
        return false;
    };
    AnnotationBuilder.prototype.resolveToken = function (token, target) {
        var providers = [];
        for (var _i = 2; _i < arguments.length; _i++) {
            providers[_i - 2] = arguments[_i];
        }
        var _a, _b;
        if (target) {
            return (_a = this.container).getService.apply(_a, [token, target].concat(providers));
        }
        return (_b = this.container).resolve.apply(_b, [token].concat(providers));
    };
    AnnotationBuilder.getClassAnnations = function () {
        return { "name": "AnnotationBuilder", "params": { "constructor": [], "getMetaAccessor": ["token", "config"], "build": ["token", "config", "options"], "boot": ["runable", "config", "options"], "resolveRunable": ["instance", "runableOptions", "options"], "createInstance": ["token", "config", "options"], "getBuilder": ["token", "config", "options"], "registerExts": ["config"], "isEqual": ["build"], "resolveToken": ["token", "target", "providers"] } };
    };
    var AnnotationBuilder_1;
    tslib_1.__decorate([
        core_1.Inject(core_1.ContainerToken),
        tslib_1.__metadata("design:type", Object)
    ], AnnotationBuilder.prototype, "container", void 0);
    AnnotationBuilder = AnnotationBuilder_1 = tslib_1.__decorate([
        core_1.Singleton(IAnnotationBuilder.AnnotationBuilderToken),
        tslib_1.__metadata("design:paramtypes", [])
    ], AnnotationBuilder);
    return AnnotationBuilder;
}());
exports.AnnotationBuilder = AnnotationBuilder;


});

unwrapExports(AnnotationBuilder_1);
var AnnotationBuilder_2 = AnnotationBuilder_1.AnnotationBuilder;

var annotations = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });

tslib_1.__exportStar(IAnnotationBuilder, exports);
tslib_1.__exportStar(AnnoBuildStrategy_1, exports);
tslib_1.__exportStar(AnnotationBuilder_1, exports);


});

unwrapExports(annotations);

var ModuleBuilder_1 = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });







/**
 * inject module load token.
 *
 * @export
 * @class InjectModuleLoadToken
 * @extends {Registration<T>}
 * @template T
 */
var InjectModuleLoadToken = /** @class */ (function (_super) {
    tslib_1.__extends(InjectModuleLoadToken, _super);
    function InjectModuleLoadToken(token) {
        return _super.call(this, token, 'module_loader') || this;
    }
    InjectModuleLoadToken.getClassAnnations = function () {
        return { "name": "InjectModuleLoadToken", "params": { "constructor": ["token"] } };
    };
    return InjectModuleLoadToken;
}(core_1.Registration));
exports.InjectModuleLoadToken = InjectModuleLoadToken;
/**
 * module builder
 *
 * @export
 * @class ModuleBuilder
 * @implements {IModuleBuilder}
 * @template T
 */
var ModuleBuilder = /** @class */ (function () {
    function ModuleBuilder() {
    }
    ModuleBuilder.prototype.getPools = function () {
        return this.pools;
    };
    ModuleBuilder.prototype.getInjectedModule = function (type) {
        return this.getPools().getDefault().get(new InjectedModule_1.InjectedModuleToken(type));
    };
    /**
     * get metadata accessor.
     *
     * @param {IContainer} container
     * @param {Token<any>} token
     * @param {ModuleConfigure} [config]
     * @returns {IMetaAccessor<any>}
     * @memberof ModuleBuilder
     */
    ModuleBuilder.prototype.getMetaAccessor = function (container, token, config) {
        var mtk;
        if (core_1.isToken(token)) {
            mtk = token;
        }
        else {
            config = token;
        }
        return container.getService(core_1.MetaAccessorToken, mtk ? [mtk, core_1.lang.getClass(this)] : core_1.lang.getClass(this), function (tk) { return new core_1.InjectMetaAccessorToken(tk); }, config ? (config.defaultMetaAccessor || core_1.MetaAccessorToken) : core_1.MetaAccessorToken);
    };
    /**
     * load module.
     *
     * @param {(Token<T> | ModuleConfigure)} token
     * @param {(ModuleConfig<T> | BootOptions<T>)} [config]
     * @param {BootOptions<T>} [options]
     * @returns {Promise<InjectedModule<T>>}
     * @memberof ModuleBuilder
     */
    ModuleBuilder.prototype.load = function (token, config, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var params, env, parent, injmd;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        params = this.vaildParams(token, config, options);
                        env = params.options ? params.options.env : null;
                        if (env instanceof InjectedModule_1.InjectedModule) {
                            return [2 /*return*/, env];
                        }
                        return [4 /*yield*/, this.getParentContainer(env)];
                    case 1:
                        parent = _a.sent();
                        if (!params.token) return [3 /*break*/, 5];
                        return [4 /*yield*/, this.loadViaToken(params.token, parent)];
                    case 2:
                        injmd = _a.sent();
                        if (!params.config) return [3 /*break*/, 4];
                        return [4 /*yield*/, this.loadViaConfig(params.config, injmd.container || parent)];
                    case 3:
                        _a.sent();
                        _a.label = 4;
                    case 4: return [2 /*return*/, injmd];
                    case 5: return [4 /*yield*/, this.loadViaConfig(params.config, parent)];
                    case 6: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    /**
     * bootstrap with module and config.
     *
     * @param {(Token<T> | ModuleConfig<T>)} token
     * @param {(ModuleConfig<T> | BootOptions<T>)} [config]
     * @param {BootOptions<T>} [options]
     * @returns {Promise<Runnable<T>>}
     * @memberof ModuleBuilder
     */
    ModuleBuilder.prototype.bootstrap = function (token, config, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var params, injmdl, cfg, tk, container, accessor, bootToken, anBuilder, mdBuilder;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        params = this.vaildParams(token, config, options);
                        options = params.options || {};
                        if (!params.token) return [3 /*break*/, 2];
                        return [4 /*yield*/, this.load(params.token, params.config, options)];
                    case 1:
                        injmdl = _a.sent();
                        return [3 /*break*/, 4];
                    case 2: return [4 /*yield*/, this.load(params.config, options)];
                    case 3:
                        injmdl = _a.sent();
                        _a.label = 4;
                    case 4:
                        options.env = injmdl;
                        cfg = injmdl.config;
                        tk = injmdl.token || injmdl.type;
                        container = injmdl.container;
                        accessor = this.getMetaAccessor(container, tk);
                        bootToken = accessor.getBootToken(cfg, container);
                        if (!bootToken) return [3 /*break*/, 6];
                        anBuilder = this.getAnnoBuilder(container, bootToken, cfg);
                        return [4 /*yield*/, anBuilder.boot(bootToken, cfg, options)];
                    case 5: return [2 /*return*/, _a.sent()];
                    case 6:
                        mdBuilder = this.getAnnoBuilder(container, tk, cfg);
                        return [4 /*yield*/, mdBuilder.boot(tk, cfg, options)];
                    case 7: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    ModuleBuilder.prototype.vaildParams = function (token, config, options) {
        var params = {};
        if (core_1.isToken(token)) {
            params.token = token;
            if (core_1.isUndefined(options)) {
                params.options = config;
            }
            else {
                params.config = config;
                params.options = options;
            }
        }
        else {
            params.config = token;
            params.options = config;
        }
        return params;
    };
    ModuleBuilder.prototype.loadViaToken = function (token, parent) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var injmdl, cfg;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.import(token, parent)];
                    case 1:
                        injmdl = _a.sent();
                        if (!injmdl) {
                            cfg = this.getMetaAccessor(parent, token).getMetadata(token, parent);
                            injmdl = new InjectedModule_1.InjectedModule(token, cfg, parent);
                        }
                        return [2 /*return*/, injmdl];
                }
            });
        });
    };
    ModuleBuilder.prototype.loadViaConfig = function (config, parent) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var injmd, token, cfg, parser, pdrmap_1;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        injmd = null;
                        token = this.getMetaAccessor(parent, config).getToken(config, parent);
                        if (!token) return [3 /*break*/, 2];
                        return [4 /*yield*/, this.import(token, parent)];
                    case 1:
                        injmd = _a.sent();
                        if (!injmd) {
                            cfg = this.getMetaAccessor(parent, token).getMetadata(token, parent, config);
                            injmd = new InjectedModule_1.InjectedModule(token, cfg, parent);
                        }
                        return [3 /*break*/, 3];
                    case 2:
                        token = null;
                        _a.label = 3;
                    case 3:
                        if (!(core_1.isArray(config.imports) && config.imports.length)) return [3 /*break*/, 5];
                        return [4 /*yield*/, parent.loadModule.apply(parent, config.imports)];
                    case 4:
                        _a.sent();
                        _a.label = 5;
                    case 5:
                        if (core_1.isArray(config.providers) && config.providers.length) {
                            parser = parent.getProviderParser();
                            pdrmap_1 = parser.parse.apply(parser, config.providers);
                            pdrmap_1.keys().forEach(function (key) {
                                core_1.isToken(key) && parent.bindProvider(key, function () {
                                    var providers = [];
                                    for (var _i = 0; _i < arguments.length; _i++) {
                                        providers[_i] = arguments[_i];
                                    }
                                    return pdrmap_1.resolve.apply(pdrmap_1, [key].concat(providers));
                                });
                            });
                        }
                        if (injmd) {
                            injmd.config = core_1.lang.assign({}, injmd.config, config);
                        }
                        else {
                            injmd = new InjectedModule_1.InjectedModule(token, config, parent);
                        }
                        return [2 /*return*/, injmd];
                }
            });
        });
    };
    ModuleBuilder.prototype.import = function (token, parent) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var type, key;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        type = core_1.isClass(token) ? token : parent.getTokenImpl(token);
                        if (!core_1.isClass(type)) return [3 /*break*/, 3];
                        key = new InjectedModule_1.InjectedModuleToken(type);
                        if (!parent.has(key)) return [3 /*break*/, 1];
                        return [2 /*return*/, parent.get(key)];
                    case 1: return [4 /*yield*/, parent.loadModule(type)];
                    case 2:
                        _a.sent();
                        if (parent.has(key)) {
                            return [2 /*return*/, parent.get(key)];
                        }
                        _a.label = 3;
                    case 3: return [2 /*return*/, null];
                }
            });
        });
    };
    ModuleBuilder.prototype.getParentContainer = function (env) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var parent;
            return tslib_1.__generator(this, function (_a) {
                if (env) {
                    if (env instanceof core_1.Container) {
                        parent = env;
                    }
                    else if (env instanceof InjectedModule_1.InjectedModule) {
                        parent = env.container.parent;
                    }
                }
                if (!parent) {
                    parent = this.getPools().getDefault();
                }
                return [2 /*return*/, parent];
            });
        });
    };
    ModuleBuilder.prototype.getAnnoBuilder = function (container, token, config) {
        var builder;
        if (core_1.isClass(config.annoBuilder)) {
            if (!container.has(config.annoBuilder)) {
                container.register(config.annoBuilder);
            }
        }
        if (core_1.isToken(config.annoBuilder)) {
            builder = container.resolve(config.annoBuilder);
        }
        else if (config.annoBuilder instanceof annotations.AnnotationBuilder) {
            builder = config.annoBuilder;
        }
        if (!builder && token) {
            builder = container.getService(annotations.AnnotationBuilderToken, token, function (tk) { return new annotations.InjectAnnotationBuilder(tk); }, config.defaultAnnoBuilder || annotations.AnnotationBuilderToken);
        }
        if (builder) {
            builder.container = container;
        }
        return builder;
    };
    ModuleBuilder.getClassAnnations = function () {
        return { "name": "ModuleBuilder", "params": { "constructor": [], "getPools": [], "getInjectedModule": ["type"], "getMetaAccessor": ["container", "token", "config"], "load": ["token", "config", "options"], "bootstrap": ["token", "config", "options"], "vaildParams": ["token", "config", "options"], "loadViaToken": ["token", "parent"], "loadViaConfig": ["config", "parent"], "import": ["token", "parent"], "getParentContainer": ["env"], "getAnnoBuilder": ["container", "token", "config"] } };
    };
    tslib_1.__decorate([
        core_1.Inject(utils.ContainerPoolToken),
        tslib_1.__metadata("design:type", utils.ContainerPool)
    ], ModuleBuilder.prototype, "pools", void 0);
    ModuleBuilder = tslib_1.__decorate([
        core_1.Singleton(IModuleBuilder.ModuleBuilderToken),
        tslib_1.__metadata("design:paramtypes", [])
    ], ModuleBuilder);
    return ModuleBuilder;
}());
exports.ModuleBuilder = ModuleBuilder;


});

unwrapExports(ModuleBuilder_1);
var ModuleBuilder_2 = ModuleBuilder_1.InjectModuleLoadToken;
var ModuleBuilder_3 = ModuleBuilder_1.ModuleBuilder;

var modules = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });

tslib_1.__exportStar(DIModuleInjector_1, exports);
tslib_1.__exportStar(DIModuleValidate, exports);
tslib_1.__exportStar(InjectedModule_1, exports);
tslib_1.__exportStar(IModuleBuilder, exports);
tslib_1.__exportStar(ModuleBuilder_1, exports);


});

unwrapExports(modules);

var BootModuleInjector = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });



/**
 * Bootstrap module Validate Token
 */
exports.BootstrapValidateToken = new core_1.InjectModuleValidateToken('@Bootstrap');
/**
 * Bootstrap module Validate
 *
 * @export
 * @class DIModuelValidate
 * @extends {ModuelValidate}
 */
var BootstrapValidate = /** @class */ (function (_super) {
    tslib_1.__extends(BootstrapValidate, _super);
    function BootstrapValidate() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    BootstrapValidate.prototype.getDecorator = function () {
        return '@Bootstrap';
    };
    BootstrapValidate.getClassAnnations = function () {
        return { "name": "BootstrapValidate", "params": { "getDecorator": [] } };
    };
    BootstrapValidate = tslib_1.__decorate([
        core_1.Singleton(exports.BootstrapValidateToken)
    ], BootstrapValidate);
    return BootstrapValidate;
}(core_1.ModuelValidate));
exports.BootstrapValidate = BootstrapValidate;
exports.BootstrapInjectorToken = new core_1.InjectModuleInjectorToken('@Bootstrap');
var BootstrapInjector = /** @class */ (function (_super) {
    tslib_1.__extends(BootstrapInjector, _super);
    function BootstrapInjector(validate) {
        return _super.call(this, validate) || this;
    }
    BootstrapInjector.getClassAnnations = function () {
        return { "name": "BootstrapInjector", "params": { "constructor": ["validate"] } };
    };
    BootstrapInjector = tslib_1.__decorate([
        core_1.Singleton(exports.BootstrapInjectorToken),
        tslib_1.__param(0, core_1.Inject(exports.BootstrapValidateToken)),
        tslib_1.__metadata("design:paramtypes", [Object])
    ], BootstrapInjector);
    return BootstrapInjector;
}(modules.DIModuleInjector));
exports.BootstrapInjector = BootstrapInjector;


});

unwrapExports(BootModuleInjector);
var BootModuleInjector_1 = BootModuleInjector.BootstrapValidateToken;
var BootModuleInjector_2 = BootModuleInjector.BootstrapValidate;
var BootModuleInjector_3 = BootModuleInjector.BootstrapInjectorToken;
var BootModuleInjector_4 = BootModuleInjector.BootstrapInjector;

var ConfigureManager_1 = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });



/**
 * configure manager.
 *
 * @export
 * @class ConfigureManager
 */
var ConfigureManager = /** @class */ (function () {
    /**
     * Creates an instance of ConfigureManager.
     * @param {string} [baseURL]
     * @memberof ConfigureManager
     */
    function ConfigureManager(baseURL) {
        this.baseURL = baseURL;
        this.configs = [];
    }
    /**
     * use configuration.
     *
     * @param {(string | AppConfigure)} [config]
     * @returns {this} this configure manager.
     * @memberof Bootstrap
     */
    ConfigureManager.prototype.useConfiguration = function (config) {
        if (core_1.isUndefined(config)) {
            config = '';
        }
        // clean cached config.
        this.config = null;
        var idx = this.configs.indexOf(config);
        if (idx >= 0) {
            this.configs.splice(idx, 1);
        }
        this.configs.push(config);
        return this;
    };
    /**
     * get config.
     *
     * @returns {Promise<T>}
     * @memberof ConfigureManager
     */
    ConfigureManager.prototype.getConfig = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var _a;
            return tslib_1.__generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        if (!!this.config) return [3 /*break*/, 2];
                        _a = this;
                        return [4 /*yield*/, this.initConfig()];
                    case 1:
                        _a.config = _b.sent();
                        _b.label = 2;
                    case 2: return [2 /*return*/, this.config];
                }
            });
        });
    };
    /**
     * init config.
     *
     * @protected
     * @returns
     * @memberof ConfigureManager
     */
    ConfigureManager.prototype.initConfig = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var config, exts;
            var _this = this;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.getDefaultConfig()];
                    case 1:
                        config = _a.sent();
                        if (this.configs.length < 1) {
                            this.configs.push(''); // load default loader config.
                        }
                        return [4 /*yield*/, Promise.all(this.configs.map(function (cfg) {
                                if (core_1.isString(cfg)) {
                                    return _this.loadConfig(cfg);
                                }
                                else {
                                    return cfg;
                                }
                            }))];
                    case 2:
                        exts = _a.sent();
                        exts.forEach(function (exCfg) {
                            if (exCfg) {
                                core_1.lang.assign(config, exCfg);
                            }
                        });
                        return [2 /*return*/, config];
                }
            });
        });
    };
    /**
     * load config.
     *
     * @protected
     * @param {string} src
     * @returns {Promise<T>}
     * @memberof ConfigureManager
     */
    ConfigureManager.prototype.loadConfig = function (src) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var loader, builder, cfg;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!this.container.has(IConfigureManager.ConfigureLoaderToken)) return [3 /*break*/, 2];
                        loader = this.container.resolve(IConfigureManager.ConfigureLoaderToken, { baseURL: this.baseURL, container: this.container });
                        return [4 /*yield*/, loader.load(src)];
                    case 1: return [2 /*return*/, _a.sent()];
                    case 2:
                        if (!src) return [3 /*break*/, 4];
                        builder = this.container.getBuilder();
                        return [4 /*yield*/, builder.loader.load([src])];
                    case 3:
                        cfg = _a.sent();
                        return [2 /*return*/, cfg.length ? cfg[0] : null];
                    case 4: return [2 /*return*/, null];
                }
            });
        });
    };
    /**
     * get default config.
     *
     * @protected
     * @returns {Promise<T>}
     * @memberof ConfigureManager
     */
    ConfigureManager.prototype.getDefaultConfig = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                if (this.container.has(IConfigureManager.DefaultConfigureToken)) {
                    return [2 /*return*/, this.container.resolve(IConfigureManager.DefaultConfigureToken)];
                }
                else {
                    return [2 /*return*/, {}];
                }
                return [2 /*return*/];
            });
        });
    };
    ConfigureManager.getClassAnnations = function () {
        return { "name": "ConfigureManager", "params": { "constructor": ["baseURL"], "useConfiguration": ["config"], "getConfig": [], "initConfig": [], "loadConfig": ["src"], "getDefaultConfig": [] } };
    };
    tslib_1.__decorate([
        core_1.Inject(core_1.ContainerToken),
        tslib_1.__metadata("design:type", Object)
    ], ConfigureManager.prototype, "container", void 0);
    ConfigureManager = tslib_1.__decorate([
        core_1.Singleton(IConfigureManager.ConfigureMgrToken),
        tslib_1.__metadata("design:paramtypes", [String])
    ], ConfigureManager);
    return ConfigureManager;
}());
exports.ConfigureManager = ConfigureManager;


});

unwrapExports(ConfigureManager_1);
var ConfigureManager_2 = ConfigureManager_1.ConfigureManager;

var BootModule_1 = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });








/**
 * Bootstrap ext for ioc. auto run setup after registered.
 * with @IocExt('setup') decorator.
 * @export
 * @class BootModule
 */
var BootModule = /** @class */ (function () {
    function BootModule(container) {
        this.container = container;
    }
    /**
     * register aop for container.
     *
     * @memberof AopModule
     */
    BootModule.prototype.setup = function () {
        var container = this.container;
        var lifeScope = container.get(core_1.LifeScopeToken);
        lifeScope.registerDecorator(Annotation.Annotation, core_1.CoreActions.bindProvider, core_1.CoreActions.cache, core_1.CoreActions.componentBeforeInit, core_1.CoreActions.componentInit, core_1.CoreActions.componentAfterInit);
        lifeScope.registerDecorator(DIModule.DIModule, core_1.CoreActions.bindProvider, core_1.CoreActions.cache, core_1.CoreActions.componentBeforeInit, core_1.CoreActions.componentInit, core_1.CoreActions.componentAfterInit);
        lifeScope.registerDecorator(Bootstrap.Bootstrap, core_1.CoreActions.bindProvider, core_1.CoreActions.cache, core_1.CoreActions.componentBeforeInit, core_1.CoreActions.componentInit, core_1.CoreActions.componentAfterInit);
        container.use(annotations, modules, boot);
    };
    BootModule.getClassAnnations = function () {
        return { "name": "BootModule", "params": { "constructor": ["container"], "setup": [] } };
    };
    BootModule = tslib_1.__decorate([
        core_1.IocExt('setup'),
        tslib_1.__param(0, core_1.Inject(core_1.ContainerToken)),
        tslib_1.__metadata("design:paramtypes", [Object])
    ], BootModule);
    return BootModule;
}());
exports.BootModule = BootModule;


});

unwrapExports(BootModule_1);
var BootModule_2 = BootModule_1.BootModule;

var RunnableBuilder_1 = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });









/**
 * runnable events
 *
 * @export
 * @enum {number}
 */
var RunnableEvents;
(function (RunnableEvents) {
    /**
     * on root container created.
     */
    RunnableEvents["onRootContainerCreated"] = "onRootContainerCreated";
    /**
     * on root container inited.
     */
    RunnableEvents["onRootContainerInited"] = "onRootContainerInited";
    /**
     * on module created.
     */
    RunnableEvents["onModuleCreated"] = "onModuleCreated";
    /**
     * on registered runner use module.
     */
    RunnableEvents["registeredExt"] = "registeredExt";
})(RunnableEvents = exports.RunnableEvents || (exports.RunnableEvents = {}));
/**
 * runnable builder.
 *
 * @export
 * @class RunnableBuilder
 * @extends {ModuleBuilder}
 * @template T
 */
var RunnableBuilder = /** @class */ (function (_super) {
    tslib_1.__extends(RunnableBuilder, _super);
    function RunnableBuilder(baseURL) {
        var _this = _super.call(this) || this;
        _this.inited = false;
        _this._baseURL = baseURL;
        _this.customRegs = [];
        _this.depModules = [];
        _this.beforeInitPds = new Map();
        _this.afterInitPds = new Map();
        _this.events = new utils.Events();
        _this.initEvents();
        return _this;
    }
    RunnableBuilder.prototype.initEvents = function () {
        var _this = this;
        this.on(RunnableEvents.onRootContainerInited, function (container) {
            _this.afterInitPds.forEach(function (val, key) {
                container.bindProvider(key, val);
            });
        });
    };
    RunnableBuilder.prototype.on = function (name, event) {
        this.events.on(name, event);
        return this;
    };
    RunnableBuilder.prototype.off = function (name, event) {
        this.events.off(name, event);
        return this;
    };
    RunnableBuilder.prototype.emit = function (name) {
        var args = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            args[_i - 1] = arguments[_i];
        }
        var _a;
        (_a = this.events).emit.apply(_a, [name].concat(args));
    };
    RunnableBuilder.prototype.getPools = function () {
        if (!this.pools) {
            this.pools = new utils.ContainerPool(this.createContainerBuilder());
            this.createDefaultContainer();
        }
        return this.pools;
    };
    RunnableBuilder.prototype.getRunRoot = function (resolver) {
        return this._baseURL || (resolver || this.getPools().getDefault()).resolve(IRunnableBuilder.ProcessRunRootToken) || '';
    };
    /**
     * has register in pools.
     * use must after `initContainerPools`.
     *
     * @template T
     * @param {Token<T>} key
     * @returns {boolean}
     * @memberof RunnableBuilder
     */
    RunnableBuilder.prototype.has = function (key, aliasOrway) {
        return this.getPools().values().some(function (c) { return c.has(key, aliasOrway); });
    };
    /**
     * resove token in pools.
     * use must after `initContainerPools`.
     * @template T
     * @param {Token<T>} token
     * @param {...ParamProviders[]} providers
     * @returns {T}
     * @memberof RunnableBuilder
     */
    RunnableBuilder.prototype.resolve = function (token, resway) {
        var providers = [];
        for (var _i = 2; _i < arguments.length; _i++) {
            providers[_i - 2] = arguments[_i];
        }
        var resolved;
        this.getPools().values().some(function (c) {
            resolved = c.resolve.apply(c, [token, resway].concat(providers));
            return !core_1.isNullOrUndefined(resolved);
        });
        return resolved;
    };
    RunnableBuilder.prototype.createContainerBuilder = function () {
        return new core_1.ContainerBuilder();
    };
    /**
     * use module as global Depdences module.
     *
     * @param {...LoadType[]} modules
     * @returns {this}
     * @memberof RunnableBuilder
     */
    RunnableBuilder.prototype.use = function () {
        var modules$$2 = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            modules$$2[_i] = arguments[_i];
        }
        this.depModules = this.depModules.concat(modules$$2);
        this.inited = false;
        return this;
    };
    /**
     * bind provider
     *
     * @template T
     * @param {Token<T>} provide
     * @param {Token<T> | Factory<T>} provider
     * @param {boolean} [beforeInit]
     * @returns {this}
     * @memberof RunnableBuilder
     */
    RunnableBuilder.prototype.provider = function (provide, provider, beforeInit) {
        if (beforeInit) {
            this.beforeInitPds.set(provide, provider);
        }
        else {
            this.afterInitPds.set(provide, provider);
        }
        return this;
    };
    RunnableBuilder.prototype.getProvider = function (provide, beforeInit) {
        if (beforeInit) {
            return this.beforeInitPds.has(provide) ? this.beforeInitPds.get(provide) : null;
        }
        else {
            return this.afterInitPds.has(provide) ? this.afterInitPds.get(provide) : null;
        }
    };
    RunnableBuilder.prototype.initContainerPools = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var container;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (this.inited) {
                            return [2 /*return*/];
                        }
                        container = this.getPools().getDefault();
                        return [4 /*yield*/, this.registerExts(container)];
                    case 1:
                        _a.sent();
                        return [4 /*yield*/, this.registerByConfigure(container)];
                    case 2:
                        _a.sent();
                        this.inited = true;
                        this.events.emit(RunnableEvents.onRootContainerInited, container);
                        return [2 /*return*/];
                }
            });
        });
    };
    RunnableBuilder.prototype.load = function (token, config, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.initContainerPools()];
                    case 1:
                        _a.sent();
                        return [4 /*yield*/, _super.prototype.load.call(this, token, config, options)];
                    case 2: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    RunnableBuilder.prototype.bootstrap = function (token, config, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var params, injmdl, _a, builder, _b, _c;
            return tslib_1.__generator(this, function (_d) {
                switch (_d.label) {
                    case 0:
                        params = this.vaildParams(token, config, options);
                        options = params.options || {};
                        if (!params.token) return [3 /*break*/, 2];
                        return [4 /*yield*/, this.load(params.token, params.config, options)];
                    case 1:
                        _a = _d.sent();
                        return [3 /*break*/, 4];
                    case 2: return [4 /*yield*/, this.load(params.config, options)];
                    case 3:
                        _a = _d.sent();
                        _d.label = 4;
                    case 4:
                        injmdl = _a;
                        options.env = injmdl;
                        options.bootBuilder = this;
                        options.configManager = this.getConfigManager();
                        builder = this.getBuilder(injmdl);
                        if (!!this.isSame(builder, this)) return [3 /*break*/, 9];
                        if (!params.token) return [3 /*break*/, 6];
                        return [4 /*yield*/, builder.bootstrap(params.token, params.config, options)];
                    case 5:
                        _b = _d.sent();
                        return [3 /*break*/, 8];
                    case 6: return [4 /*yield*/, builder.bootstrap(params.config, options)];
                    case 7:
                        _b = _d.sent();
                        _d.label = 8;
                    case 8: return [2 /*return*/, _b];
                    case 9:
                        if (!params.token) return [3 /*break*/, 11];
                        return [4 /*yield*/, _super.prototype.bootstrap.call(this, params.token, params.config, options)];
                    case 10:
                        _c = _d.sent();
                        return [3 /*break*/, 13];
                    case 11: return [4 /*yield*/, _super.prototype.bootstrap.call(this, params.config, options)];
                    case 12:
                        _c = _d.sent();
                        _d.label = 13;
                    case 13: return [2 /*return*/, _c];
                }
            });
        });
    };
    RunnableBuilder.prototype.isSame = function (mdb1, mdb2) {
        if (!mdb1 || !mdb2) {
            return false;
        }
        if (mdb1 === mdb2) {
            return true;
        }
        if (core_1.lang.getClass(mdb1) === core_1.lang.getClass(mdb2)) {
            return true;
        }
        return false;
    };
    /**
     * get module builder
     *
     * @param {(Token<T> |RunnableConfigure)} token
     * @param {ModuleEnv} [env]
     * @returns {IModuleBuilder<T>}
     * @memberof RunnableBuilder
     */
    RunnableBuilder.prototype.getBuilderByConfig = function (token, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var injmdl;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.load(token, options)];
                    case 1:
                        injmdl = _a.sent();
                        return [2 /*return*/, this.getBuilder(injmdl)];
                }
            });
        });
    };
    RunnableBuilder.prototype.getBuilder = function (injmdl) {
        var cfg = injmdl.config;
        var container = injmdl.container;
        var builder;
        if (cfg) {
            if (core_1.isToken(cfg.builder)) {
                if (core_1.isClass(cfg.builder)) {
                    if (!container.has(cfg.builder)) {
                        container.register(cfg.builder);
                    }
                    if (cfg.builder === core_1.lang.getClass(this)) {
                        builder = this;
                    }
                }
                if (!builder) {
                    builder = container.resolve(cfg.builder);
                }
            }
            else if (cfg.builder instanceof modules.ModuleBuilder) {
                builder = cfg.builder;
            }
        }
        var tko = injmdl.token;
        if (!builder && tko) {
            builder = container.getService(modules.ModuleBuilderToken, tko, function (tk) { return new modules.InjectModuleBuilderToken(tk); }, cfg.defaultBuilder || modules.ModuleBuilderToken);
        }
        return builder || this;
    };
    RunnableBuilder.prototype.getConfigManager = function () {
        if (!this.configMgr) {
            this.configMgr = this.createConfigureMgr();
        }
        return this.configMgr;
    };
    RunnableBuilder.prototype.createConfigureMgr = function () {
        var container = this.getPools().getDefault();
        return container.getService(IConfigureManager.ConfigureMgrToken, core_1.lang.getClass(this), { baseURL: this.getRunRoot(container) });
    };
    RunnableBuilder.prototype.createDefaultContainer = function () {
        var _this = this;
        var container = this.pools.getDefault();
        container.register(BootModule_1.BootModule);
        var chain = container.getBuilder().getInjectorChain(container);
        chain.first(container.resolve(modules.DIModuleInjectorToken));
        chain.first(container.resolve(BootModuleInjector.BootstrapInjectorToken));
        container.bindProvider(utils.ContainerPoolToken, function () { return _this.getPools(); });
        container.bindProvider(IRunnableBuilder.CurrentRunnableBuilderToken, function () { return _this; });
        this.beforeInitPds.forEach(function (val, key) {
            container.bindProvider(key, val);
        });
        this.events.emit(RunnableEvents.onRootContainerCreated, container);
        return container;
    };
    /**
     * register ioc exts
     *
     * @protected
     * @param {IContainer} container
     * @param {AppConfigure} config
     * @memberof RunnableBuilder
     */
    RunnableBuilder.prototype.registerExts = function (container) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var types;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!this.depModules.length) return [3 /*break*/, 2];
                        return [4 /*yield*/, container.loadModule.apply(container, this.depModules)];
                    case 1:
                        types = _a.sent();
                        this.emit(RunnableEvents.registeredExt, types, container);
                        _a.label = 2;
                    case 2: return [2 /*return*/];
                }
            });
        });
    };
    /**
     * register by configure.
     *
     * @protected
     * @param {IContainer} container
     * @param {ModuleConfig<T>} config
     * @returns {Promise<void>}
     * @memberof RunnableBuilder
     */
    RunnableBuilder.prototype.registerByConfigure = function (container) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var configManager, config, curClass, registers;
            var _this = this;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        configManager = this.getConfigManager();
                        return [4 /*yield*/, configManager.getConfig()];
                    case 1:
                        config = _a.sent();
                        if (!config.baseURL) {
                            config.baseURL = this.getRunRoot(container);
                        }
                        return [4 /*yield*/, core_1.PromiseUtil.step(this.customRegs.map(function (cs) { return function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
                                var tokens;
                                return tslib_1.__generator(this, function (_a) {
                                    switch (_a.label) {
                                        case 0: return [4 /*yield*/, cs(container, config, this)];
                                        case 1:
                                            tokens = _a.sent();
                                            return [2 /*return*/, tokens];
                                    }
                                });
                            }); }; }))];
                    case 2:
                        _a.sent();
                        curClass = core_1.lang.getClass(this);
                        registers = [];
                        // only run root register.
                        container.iteratorServices(function (serType, fac, resolver) {
                            if (!config.baseURL) {
                                config.baseURL = _this.getRunRoot(resolver);
                            }
                            registers.push({
                                resolver: resolver,
                                serType: serType
                            });
                        }, ConfigureRegister_1.ConfigureRegister, curClass, true, core_1.ResoveWay.nodes);
                        return [4 /*yield*/, Promise.all(registers.map(function (ser) { return ser.resolver.resolve(ser.serType).register(config, _this); }))];
                    case 3:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    RunnableBuilder.getClassAnnations = function () {
        return { "name": "RunnableBuilder", "params": { "constructor": ["baseURL"], "initEvents": [], "on": ["name", "event"], "off": ["name", "event"], "emit": ["name", "args"], "getPools": [], "getRunRoot": ["resolver"], "has": ["key", "aliasOrway"], "resolve": ["token", "resway", "providers"], "createContainerBuilder": [], "use": ["modules"], "provider": ["provide", "provider", "beforeInit"], "getProvider": ["provide", "beforeInit"], "initContainerPools": [], "load": ["token", "config", "options"], "bootstrap": ["token", "config", "options"], "isSame": ["mdb1", "mdb2"], "getBuilderByConfig": ["token", "options"], "getBuilder": ["injmdl"], "getConfigManager": [], "createConfigureMgr": [], "createDefaultContainer": [], "registerExts": ["container"], "registerByConfigure": ["container"] } };
    };
    RunnableBuilder = tslib_1.__decorate([
        core_1.Injectable(IRunnableBuilder.RunnableBuilderToken),
        tslib_1.__metadata("design:paramtypes", [String])
    ], RunnableBuilder);
    return RunnableBuilder;
}(modules.ModuleBuilder));
exports.RunnableBuilder = RunnableBuilder;


});

unwrapExports(RunnableBuilder_1);
var RunnableBuilder_2 = RunnableBuilder_1.RunnableEvents;
var RunnableBuilder_3 = RunnableBuilder_1.RunnableBuilder;

var IApplicationBuilder = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });

/**
 *  application builder token.
 */
exports.ApplicationBuilderToken = new core_1.InjectToken('DI_AppBuilder');


});

unwrapExports(IApplicationBuilder);
var IApplicationBuilder_1 = IApplicationBuilder.ApplicationBuilderToken;

var AppConfigure = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });

/**
 * application configuration token.
 */
exports.AppConfigureToken = new core_1.InjectToken('DI_APP_Configuration');


});

unwrapExports(AppConfigure);
var AppConfigure_1 = AppConfigure.AppConfigureToken;

var ApplicationBuilder_1 = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });




/**
 * application events
 *
 * @export
 * @enum {number}
 */
exports.AppEvents = RunnableBuilder_1.RunnableEvents;
/**
 * application events
 */
exports.ApplicationEvents = RunnableBuilder_1.RunnableEvents;
/**
 * application builder.
 *
 * @export
 * @class Default ApplicationBuilder
 * @extends {ModuleBuilder}
 * @template T
 */
var ApplicationBuilder = /** @class */ (function (_super) {
    tslib_1.__extends(ApplicationBuilder, _super);
    function ApplicationBuilder(baseURL) {
        var _this = _super.call(this, baseURL) || this;
        _this.configs = [];
        return _this;
    }
    /**
     * use configuration.
     *
     * @param {(string | AppConfigure)} [config]
     * @returns {this} global config for this application.
     * @memberof Bootstrap
     */
    ApplicationBuilder.prototype.useConfiguration = function (config) {
        if (this.configMgr) {
            this.configMgr.useConfiguration(config);
        }
        this.configs.push(config);
        return this;
    };
    ApplicationBuilder.prototype.createConfigureMgr = function () {
        var cfgMgr = _super.prototype.createConfigureMgr.call(this);
        if (this.configs && this.configs.length) {
            this.configs.forEach(function (cfg) { return cfgMgr.useConfiguration(cfg); });
        }
        return cfgMgr;
    };
    ApplicationBuilder.getClassAnnations = function () {
        return { "name": "ApplicationBuilder", "params": { "constructor": ["baseURL"], "useConfiguration": ["config"], "createConfigureMgr": [] } };
    };
    ApplicationBuilder = tslib_1.__decorate([
        core_1.Injectable(IApplicationBuilder.ApplicationBuilderToken),
        tslib_1.__metadata("design:paramtypes", [String])
    ], ApplicationBuilder);
    return ApplicationBuilder;
}(RunnableBuilder_1.RunnableBuilder));
exports.ApplicationBuilder = ApplicationBuilder;


});

unwrapExports(ApplicationBuilder_1);
var ApplicationBuilder_2 = ApplicationBuilder_1.AppEvents;
var ApplicationBuilder_3 = ApplicationBuilder_1.ApplicationEvents;
var ApplicationBuilder_4 = ApplicationBuilder_1.ApplicationBuilder;

var boot = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });

tslib_1.__exportStar(boots, exports);
tslib_1.__exportStar(BootModuleInjector, exports);
tslib_1.__exportStar(IConfigureManager, exports);
tslib_1.__exportStar(ConfigureManager_1, exports);
tslib_1.__exportStar(ConfigureRegister_1, exports);
tslib_1.__exportStar(IRunnableBuilder, exports);
tslib_1.__exportStar(RunnableBuilder_1, exports);
tslib_1.__exportStar(IApplicationBuilder, exports);
tslib_1.__exportStar(AppConfigure, exports);
tslib_1.__exportStar(ApplicationBuilder_1, exports);


});

unwrapExports(boot);

var Bootstrap = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });




/**
 * create bootstrap decorator.
 *
 * @export
 * @template T
 * @param {string} name
 * @param {Token<IRunnableBuilder<any>>>} [builder] default builder
 * @param {Token<IAnnotationBuilder<any>>} [defaultAnnoBuilder] default type builder.
 * @param {defaultBoot?: Token<any> | ((metadata: T) => Token<any>)} [defaultBoot]
 * @param {MetadataAdapter} [adapter]
 * @param {MetadataExtends<T>} [metadataExtends]
 * @returns {IBootstrapDecorator<T>}
 */
function createBootstrapDecorator(name, defaultBuilder, defaultAnnoBuilder, defaultBoot, adapter, metadataExtends) {
    return DIModule.createDIModuleDecorator(name, defaultBuilder, defaultAnnoBuilder, defaultBoot, adapter, function (metadata) {
        if (metadataExtends) {
            metadataExtends(metadata);
        }
        // static main.
        if (core_1.isClass(metadata.type) && core_1.isFunction(metadata.type['main'])) {
            setTimeout(function () {
                metadata.type['main'](metadata);
            }, 100);
        }
        else if (metadata.bootstrap) {
            setTimeout(function () {
                var builder;
                if (core_1.isClass(metadata.builder) && core_1.lang.isExtendsClass(metadata.builder, boot.RunnableBuilder)) {
                    builder = new metadata.builder();
                }
                if (!builder) {
                    builder = new boot.ApplicationBuilder();
                }
                if (builder instanceof boot.ApplicationBuilder) {
                    if (metadata.bootConfiguration) {
                        builder.useConfiguration(metadata.bootConfiguration);
                    }
                    builder.useConfiguration(core_1.lang.omit(metadata, 'imports', 'exports', 'providers', 'bootDeps', 'bootConfiguration'));
                }
                builder
                    .use.apply(builder, (metadata.bootDeps || [])).bootstrap(metadata.type);
            }, 100);
        }
        else {
            throw new Error("boot config error. has not found static main and bootstrap in [class: " + metadata.type.name + "]");
        }
        return metadata;
    });
}
exports.createBootstrapDecorator = createBootstrapDecorator;
/**
 * Bootstrap Decorator, definde class as mvc bootstrap module.
 *
 * @Bootstrap
 */
exports.Bootstrap = createBootstrapDecorator('Bootstrap', boot.ApplicationBuilderToken, IAnnotationBuilder.AnnotationBuilderToken);


});

unwrapExports(Bootstrap);
var Bootstrap_1 = Bootstrap.createBootstrapDecorator;
var Bootstrap_2 = Bootstrap.Bootstrap;

var decorators = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });

tslib_1.__exportStar(Annotation, exports);
tslib_1.__exportStar(DIModule, exports);
tslib_1.__exportStar(Bootstrap, exports);


});

unwrapExports(decorators);

var D__workspace_github_tsioc_packages_bootstrap_lib = createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });

tslib_1.__exportStar(decorators, exports);
tslib_1.__exportStar(boot, exports);
tslib_1.__exportStar(annotations, exports);
tslib_1.__exportStar(modules, exports);
tslib_1.__exportStar(runnable, exports);
tslib_1.__exportStar(utils, exports);
tslib_1.__exportStar(BootModule_1, exports);


});

var index$6 = unwrapExports(D__workspace_github_tsioc_packages_bootstrap_lib);

module.exports = index$6;

//# sourceMappingURL=sourcemaps/bootstrap.js.map
