import { AnnotationConfigure } from './AnnotationConfigure';
import { BuildOptions, BootHooks } from './AnnoType';
import { RefRegistration, Token } from '@ts-ioc/core';
/**
 * annotation instance build strategy.
 *
 * @export
 * @interface IAnnoBuildStrategy
 * @template T
 */
export interface IAnnoBuildStrategy<T> {
    /**
     * build strategy
     *
     * @param {T} instance
     * @param {AnnotationConfigure<T>} config
     * @param {BuildOptions<T>} options
     * @returns {Promise<void>}
     * @memberof IAnnoBuildStrategy
     */
    build(instance: T, config: AnnotationConfigure<T>, options: BuildOptions<T>): Promise<void>;
}
/**
 * annotation build strategy token inject.
 *
 * @export
 * @class InjectAnnoBuildStrategyToken
 * @extends {RefRegistration<IAnnoBuildStrategy<T>>}
 * @template T
 */
export declare class InjectAnnoBuildStrategyToken<T> extends RefRegistration<IAnnoBuildStrategy<T>> {
    constructor(type: Token<T>);
    static getClassAnnations(): any;
}
/**
 *  base AnnoBuildStrategy Token
 */
export declare const AnnoBuildStrategyToken: InjectAnnoBuildStrategyToken<Object>;
/**
 * annotation instance build strategy.
 *
 * @export
 * @class AnnoBuildStrategy
 * @implements {IAnnoBuildStrategy<T>}
 * @template T
 */
export declare class AnnoBuildStrategy<T> implements IAnnoBuildStrategy<T> {
    build(instance: T, config: AnnotationConfigure<T>, options: BuildOptions<T>): Promise<void>;
    /**
     * after annotation instance init.
     *
     * @protected
     * @param {BootHooks<T>} instance
     * @param {AnnotationConfigure<T>} config
     * @returns {Promise<void>}
     * @memberof AnnoBuildStrategy
     */
    protected beforeInit(instance: BootHooks<T>, config: AnnotationConfigure<T>): Promise<void>;
    /**
     * build annotation instance process.
     *
     * @protected
     * @param {T} instance
     * @param {AnnotationConfigure<T>} config
     * @param {BuildOptions<T>} [options]
     * @returns {Promise<void>}
     * @memberof AnnoBuildStrategy
     */
    protected process(instance: T, config: AnnotationConfigure<T>, options?: BuildOptions<T>): Promise<void>;
    /**
     * after annotation instance init.
     *
     * @protected
     * @param {BootHooks<T>} instance
     * @param {AnnotationConfigure<T>} config
     * @returns {Promise<void>}
     * @memberof AnnoBuildStrategy
     */
    protected afterInit(instance: BootHooks<T>, config: AnnotationConfigure<T>): Promise<void>;
    static getClassAnnations(): any;
}
