"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var core_1 = require("@ts-ioc/core");
var IAnnotationBuilder_1 = require("./IAnnotationBuilder");
var runnable_1 = require("../runnable");
var AnnoBuildStrategy_1 = require("./AnnoBuildStrategy");
/**
 * Annotation class builder. build class with metadata and config.
 *
 * @export
 * @class AnnotationBuilder
 * @implements {implements IAnnotationBuilder<T>}
 * @template T
 */
var AnnotationBuilder = /** @class */ (function () {
    function AnnotationBuilder() {
    }
    AnnotationBuilder_1 = AnnotationBuilder;
    /**
     * get metadata accessor.
     *
     * @param {(Token<any>|| AnnotationConfigure<T>)} token
     * @param {AnnotationConfigure<T>} [config]
     * @returns {IMetaAccessor<any>}
     * @memberof AnnotationBuilder
     */
    AnnotationBuilder.prototype.getMetaAccessor = function (token, config) {
        var mtk;
        if (core_1.isToken(token)) {
            mtk = token;
        }
        else {
            config = token;
        }
        return this.container.getService(core_1.MetaAccessorToken, mtk ? [mtk, core_1.lang.getClass(this)] : core_1.lang.getClass(this), function (tk) { return new core_1.InjectMetaAccessorToken(tk); }, config ? (config.defaultMetaAccessor || core_1.MetaAccessorToken) : core_1.MetaAccessorToken);
    };
    /**
     * build token type with config.
     *
     * @param {(Token<T> | AnnotationConfigure<T>)} token
     * @param {(AnnotationConfigure<T> | BuildOptions<T>)} [config]
     * @param {BuildOptions<T>} [options]
     * @returns {Promise<T>}
     * @memberof AnnotationBuilder
     */
    AnnotationBuilder.prototype.build = function (token, config, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var tk, cfg, accessor, builder, instance, strategy;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (core_1.isToken(token)) {
                            tk = token;
                            if (core_1.isUndefined(options)) {
                                options = config;
                                cfg = null;
                            }
                            else {
                                cfg = config;
                            }
                            cfg = this.getMetaAccessor(tk, cfg).getMetadata(tk, this.container, cfg);
                        }
                        else {
                            options = config;
                            cfg = token;
                            accessor = this.getMetaAccessor(cfg);
                            tk = accessor.getToken(cfg, this.container);
                            if (core_1.isToken(tk)) {
                                cfg = accessor.getMetadata(tk, this.container, cfg);
                            }
                        }
                        if (core_1.isClass(tk) && !this.container.hasRegister(tk)) {
                            this.container.register(tk);
                        }
                        if (options && core_1.isFunction(options.vaild) && !options.vaild(tk)) {
                            return [2 /*return*/, null];
                        }
                        builder = (options && options.builder) ? options.builder : this.getBuilder(tk, cfg);
                        if (!!this.isEqual(builder)) return [3 /*break*/, 2];
                        return [4 /*yield*/, builder.build(tk, cfg, core_1.lang.assign(options || {}, { builder: builder }))];
                    case 1: return [2 /*return*/, _a.sent()];
                    case 2: return [4 /*yield*/, this.registerExts(cfg)];
                    case 3:
                        _a.sent();
                        return [4 /*yield*/, this.createInstance(tk, cfg, options)];
                    case 4:
                        instance = _a.sent();
                        if (!instance) {
                            return [2 /*return*/, null];
                        }
                        strategy = this.container.getService(AnnoBuildStrategy_1.AnnoBuildStrategyToken, [
                            tk || core_1.lang.getClass(instance)
                        ].concat(((options && options.target) ? [{ target: core_1.lang.getClass(options.target), level: core_1.RefTagLevel.self }] : []), [
                            { target: core_1.lang.getClass(this), level: core_1.RefTagLevel.self }
                        ]), function (tk) { return new AnnoBuildStrategy_1.InjectAnnoBuildStrategyToken(tk); });
                        if (!strategy) return [3 /*break*/, 6];
                        return [4 /*yield*/, strategy.build(instance, cfg, options)];
                    case 5:
                        _a.sent();
                        _a.label = 6;
                    case 6:
                        if (options && core_1.isFunction(options.onCompleted)) {
                            options.onCompleted(tk, cfg, instance, this);
                        }
                        return [2 /*return*/, instance];
                }
            });
        });
    };
    /**
     * run runable.
     *
     * @param {(Token<T> | AnnotationConfigure<T>)} runable
     * @param {*} [data] bootstrap data, build data, Runnable data.
     * @returns {Promise<Runnable<T>>}
     * @memberof IGModuleBuilder
     */
    AnnotationBuilder.prototype.boot = function (runable, config, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var instance, builder, tk, cfg, onCompleted, data, runOptions, runner;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        onCompleted = function (token, config, hook, builder) {
                            tk = token;
                            cfg = config;
                            instance = hook;
                            builder = builder;
                        };
                        if (!core_1.isToken(runable)) return [3 /*break*/, 2];
                        if (core_1.isUndefined(options)) {
                            options = config;
                            cfg = null;
                        }
                        else {
                            cfg = config;
                        }
                        return [4 /*yield*/, this.build(runable, cfg, core_1.lang.assign({
                                onCompleted: onCompleted
                            }, options))];
                    case 1:
                        _a.sent();
                        return [3 /*break*/, 4];
                    case 2: return [4 /*yield*/, this.build(runable, core_1.lang.assign({
                            onCompleted: onCompleted
                        }, config))];
                    case 3:
                        _a.sent();
                        _a.label = 4;
                    case 4:
                        builder = builder || this;
                        data = options ? options.data : undefined;
                        runOptions = { instance: instance, type: core_1.lang.getClass(instance), mdToken: tk, config: cfg, data: data };
                        runner = builder.resolveRunable(instance, runOptions, options);
                        if (!(runner && core_1.isFunction(runner.onInit))) return [3 /*break*/, 6];
                        return [4 /*yield*/, runner.onInit(runOptions, options)];
                    case 5:
                        _a.sent();
                        _a.label = 6;
                    case 6:
                        if (!runnable_1.isRunner(runner)) return [3 /*break*/, 8];
                        return [4 /*yield*/, runner.run(data)];
                    case 7:
                        _a.sent();
                        return [3 /*break*/, 10];
                    case 8:
                        if (!runnable_1.isService(runner)) return [3 /*break*/, 10];
                        return [4 /*yield*/, runner.start(data)];
                    case 9:
                        _a.sent();
                        _a.label = 10;
                    case 10: return [2 /*return*/, runner];
                }
            });
        });
    };
    /**
     * run annotation instance.
     *
     * @param {T} instance
     * @param {AnnotationConfigure<T>} [config]
     * @param {Token<T>} [token]
     * @returns {Promise<Runnable<T>>}
     * @memberof AnnotationBuilder
     */
    AnnotationBuilder.prototype.resolveRunable = function (instance, runableOptions, options) {
        if (!instance) {
            return null;
        }
        var tk = runableOptions.mdToken || runableOptions.type;
        if (!runableOptions.config) {
            runableOptions.config = this.getMetaAccessor(tk).getMetadata(tk, this.container);
        }
        if (instance instanceof runnable_1.Runner || instance instanceof runnable_1.Service) {
            return instance;
        }
        else {
            var provider = { provide: runnable_1.RunnableOptionsToken, useValue: runableOptions.config };
            return this.container.getService([runnable_1.RunnerToken, runnable_1.ServiceToken], [
                tk
            ].concat(((options && options.target) ? [{ target: core_1.lang.getClass(options.target), level: core_1.RefTagLevel.self }] : []), [
                { target: core_1.lang.getClass(this), level: core_1.RefTagLevel.self }
            ]), function (tk) { return new runnable_1.InjectRunnableToken(tk); }, runableOptions.config.defaultRunnable || true, provider);
        }
    };
    AnnotationBuilder.prototype.createInstance = function (token, config, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var instance;
            return tslib_1.__generator(this, function (_a) {
                if (!core_1.isToken(token)) {
                    return [2 /*return*/, null];
                }
                instance = this.resolveToken.apply(this, [token, options ? options.target : undefined].concat((options.providers || [])));
                if (core_1.isNullOrUndefined(instance)) {
                    console.log("can not find token " + (token ? token.toString() : null) + " in container.");
                    return [2 /*return*/, null];
                }
                return [2 /*return*/, instance];
            });
        });
    };
    AnnotationBuilder.prototype.getBuilder = function (token, config, options) {
        var builder;
        if (config && config.annoBuilder) {
            if (core_1.isClass(config.annoBuilder)) {
                if (!this.container.has(config.annoBuilder)) {
                    this.container.register(config.annoBuilder);
                }
            }
            if (core_1.isToken(config.annoBuilder)) {
                builder = this.container.resolve(config.annoBuilder);
            }
            else if (config.annoBuilder instanceof AnnotationBuilder_1) {
                builder = config.annoBuilder;
            }
        }
        if (options && options.builder) {
            builder = options.builder;
        }
        if (!builder && token) {
            builder = this.container.getService(IAnnotationBuilder_1.AnnotationBuilderToken, [
                token
            ].concat((options && options.target) ? [{ target: core_1.lang.getClass(options.target), level: core_1.RefTagLevel.self }] : []), function (tk) { return new IAnnotationBuilder_1.InjectAnnotationBuilder(tk); }, config.defaultAnnoBuilder || IAnnotationBuilder_1.AnnotationBuilderToken);
        }
        if (builder && !builder.container) {
            builder.container = this.container;
        }
        return builder || this;
    };
    /**
     * register extension before create instance.
     *
     * @protected
     * @param {AnnotationConfigure<T>} [config]
     * @memberof AnnotationBuilder
     */
    AnnotationBuilder.prototype.registerExts = function (config) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                return [2 /*return*/];
            });
        });
    };
    AnnotationBuilder.prototype.isEqual = function (build) {
        if (!build) {
            return false;
        }
        if (build === this) {
            return true;
        }
        if (core_1.lang.getClass(build) === core_1.lang.getClass(this)) {
            return true;
        }
        return false;
    };
    AnnotationBuilder.prototype.resolveToken = function (token, target) {
        var providers = [];
        for (var _i = 2; _i < arguments.length; _i++) {
            providers[_i - 2] = arguments[_i];
        }
        var _a, _b;
        if (target) {
            return (_a = this.container).getService.apply(_a, [token, target].concat(providers));
        }
        return (_b = this.container).resolve.apply(_b, [token].concat(providers));
    };
    AnnotationBuilder.getClassAnnations = function () {
        return { "name": "AnnotationBuilder", "params": { "constructor": [], "getMetaAccessor": ["token", "config"], "build": ["token", "config", "options"], "boot": ["runable", "config", "options"], "resolveRunable": ["instance", "runableOptions", "options"], "createInstance": ["token", "config", "options"], "getBuilder": ["token", "config", "options"], "registerExts": ["config"], "isEqual": ["build"], "resolveToken": ["token", "target", "providers"] } };
    };
    var AnnotationBuilder_1;
    tslib_1.__decorate([
        core_1.Inject(core_1.ContainerToken),
        tslib_1.__metadata("design:type", Object)
    ], AnnotationBuilder.prototype, "container", void 0);
    AnnotationBuilder = AnnotationBuilder_1 = tslib_1.__decorate([
        core_1.Singleton(IAnnotationBuilder_1.AnnotationBuilderToken),
        tslib_1.__metadata("design:paramtypes", [])
    ], AnnotationBuilder);
    return AnnotationBuilder;
}());
exports.AnnotationBuilder = AnnotationBuilder;

//# sourceMappingURL=../sourcemaps/annotations/AnnotationBuilder.js.map
