"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var core_1 = require("@ts-ioc/core");
var IApplicationBuilder_1 = require("./IApplicationBuilder");
var RunnableBuilder_1 = require("./RunnableBuilder");
/**
 * application events
 *
 * @export
 * @enum {number}
 */
exports.AppEvents = RunnableBuilder_1.RunnableEvents;
/**
 * application events
 */
exports.ApplicationEvents = RunnableBuilder_1.RunnableEvents;
/**
 * application builder.
 *
 * @export
 * @class Default ApplicationBuilder
 * @extends {ModuleBuilder}
 * @template T
 */
var ApplicationBuilder = /** @class */ (function (_super) {
    tslib_1.__extends(ApplicationBuilder, _super);
    function ApplicationBuilder(baseURL) {
        var _this = _super.call(this, baseURL) || this;
        _this.configs = [];
        return _this;
    }
    /**
     * use configuration.
     *
     * @param {(string | AppConfigure)} [config]
     * @returns {this} global config for this application.
     * @memberof Bootstrap
     */
    ApplicationBuilder.prototype.useConfiguration = function (config) {
        if (this.configMgr) {
            this.configMgr.useConfiguration(config);
        }
        this.configs.push(config);
        return this;
    };
    ApplicationBuilder.prototype.createConfigureMgr = function () {
        var cfgMgr = _super.prototype.createConfigureMgr.call(this);
        if (this.configs && this.configs.length) {
            this.configs.forEach(function (cfg) { return cfgMgr.useConfiguration(cfg); });
        }
        return cfgMgr;
    };
    ApplicationBuilder.getClassAnnations = function () {
        return { "name": "ApplicationBuilder", "params": { "constructor": ["baseURL"], "useConfiguration": ["config"], "createConfigureMgr": [] } };
    };
    ApplicationBuilder = tslib_1.__decorate([
        core_1.Injectable(IApplicationBuilder_1.ApplicationBuilderToken),
        tslib_1.__metadata("design:paramtypes", [String])
    ], ApplicationBuilder);
    return ApplicationBuilder;
}(RunnableBuilder_1.RunnableBuilder));
exports.ApplicationBuilder = ApplicationBuilder;

//# sourceMappingURL=../sourcemaps/boot/ApplicationBuilder.js.map
