import { Token, MetadataAdapter, MetadataExtends, ITypeDecorator, LoadType } from '@ts-ioc/core';
import { IAnnotationBuilder } from '../annotations/IAnnotationBuilder';
import { IRunnableBuilder, AppConfigure } from '../boot';
/**
 * bootstrap metadata.
 *
 * @export
 * @interface BootstrapMetadata
 * @extends {AppConfigure}
 */
export interface BootstrapMetadata extends AppConfigure {
    /**
     * module bootstrap token.
     *
     * @type {Token<T>}
     * @memberof AnnotationConfigure
     */
    bootstrap?: Token<any>;
    /**
     * boot dependencies
     *
     * @type {LoadType[]}
     * @memberof BootstrapMetadata
     */
    bootDeps?: LoadType[];
    /**
     * configuration.
     *
     * @type {AppConfigure}
     * @memberof BootstrapMetadata
     */
    bootConfiguration?: AppConfigure;
}
/**
 * Bootstrap decorator, use to define class is a task element.
 *
 * @export
 * @interface IBootstrapDecorator
 * @extends {ITypeDecorator<T>}
 * @template T
 */
export interface IBootstrapDecorator<T extends BootstrapMetadata> extends ITypeDecorator<T> {
    /**
     * Bootstrap decorator, use to define class as Application Bootstrap element.
     *
     * @Bootstrap
     *
     * @param {T} metadata bootstrap metadate config.
     */
    (metadata: T): ClassDecorator;
}
/**
 * create bootstrap decorator.
 *
 * @export
 * @template T
 * @param {string} name
 * @param {Token<IRunnableBuilder<any>>>} [builder] default builder
 * @param {Token<IAnnotationBuilder<any>>} [defaultAnnoBuilder] default type builder.
 * @param {defaultBoot?: Token<any> | ((metadata: T) => Token<any>)} [defaultBoot]
 * @param {MetadataAdapter} [adapter]
 * @param {MetadataExtends<T>} [metadataExtends]
 * @returns {IBootstrapDecorator<T>}
 */
export declare function createBootstrapDecorator<T extends BootstrapMetadata>(name: string, defaultBuilder?: Token<IRunnableBuilder<any>>, defaultAnnoBuilder?: Token<IAnnotationBuilder<any>>, defaultBoot?: Token<any> | ((metadata: T) => Token<any>), adapter?: MetadataAdapter, metadataExtends?: MetadataExtends<T>): IBootstrapDecorator<T>;
/**
 * Bootstrap Decorator, definde class as mvc bootstrap module.
 *
 * @Bootstrap
 */
export declare const Bootstrap: IBootstrapDecorator<BootstrapMetadata>;
