"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var core_1 = require("@ts-ioc/core");
var IAnnotationBuilder_1 = require("../annotations/IAnnotationBuilder");
var DIModule_1 = require("./DIModule");
var boot_1 = require("../boot");
/**
 * create bootstrap decorator.
 *
 * @export
 * @template T
 * @param {string} name
 * @param {Token<IRunnableBuilder<any>>>} [builder] default builder
 * @param {Token<IAnnotationBuilder<any>>} [defaultAnnoBuilder] default type builder.
 * @param {defaultBoot?: Token<any> | ((metadata: T) => Token<any>)} [defaultBoot]
 * @param {MetadataAdapter} [adapter]
 * @param {MetadataExtends<T>} [metadataExtends]
 * @returns {IBootstrapDecorator<T>}
 */
function createBootstrapDecorator(name, defaultBuilder, defaultAnnoBuilder, defaultBoot, adapter, metadataExtends) {
    return DIModule_1.createDIModuleDecorator(name, defaultBuilder, defaultAnnoBuilder, defaultBoot, adapter, function (metadata) {
        if (metadataExtends) {
            metadataExtends(metadata);
        }
        // static main.
        if (core_1.isClass(metadata.type) && core_1.isFunction(metadata.type['main'])) {
            setTimeout(function () {
                metadata.type['main'](metadata);
            }, 100);
        }
        else if (metadata.bootstrap) {
            setTimeout(function () {
                var builder;
                if (core_1.isClass(metadata.builder) && core_1.lang.isExtendsClass(metadata.builder, boot_1.RunnableBuilder)) {
                    builder = new metadata.builder();
                }
                if (!builder) {
                    builder = new boot_1.ApplicationBuilder();
                }
                if (builder instanceof boot_1.ApplicationBuilder) {
                    if (metadata.bootConfiguration) {
                        builder.useConfiguration(metadata.bootConfiguration);
                    }
                    builder.useConfiguration(core_1.lang.omit(metadata, 'imports', 'exports', 'providers', 'bootDeps', 'bootConfiguration'));
                }
                builder
                    .use.apply(builder, (metadata.bootDeps || [])).bootstrap(metadata.type);
            }, 100);
        }
        else {
            throw new Error("boot config error. has not found static main and bootstrap in [class: " + metadata.type.name + "]");
        }
        return metadata;
    });
}
exports.createBootstrapDecorator = createBootstrapDecorator;
/**
 * Bootstrap Decorator, definde class as mvc bootstrap module.
 *
 * @Bootstrap
 */
exports.Bootstrap = createBootstrapDecorator('Bootstrap', boot_1.ApplicationBuilderToken, IAnnotationBuilder_1.AnnotationBuilderToken);

//# sourceMappingURL=../sourcemaps/decorators/Bootstrap.js.map
