"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var core_1 = require("@ts-ioc/core");
var IModuleBuilder_1 = require("../modules/IModuleBuilder");
var IAnnotationBuilder_1 = require("../annotations/IAnnotationBuilder");
/**
 * create bootstrap decorator.
 *
 * @export
 * @template T
 * @param {string} name decorator name.
 * @param {Token<IModuleBuilder>} [defaultBuilder]
 * @param {Token<IAnnotationBuilder<any>>} [defaultAnnoBuilder]
 * @param {defaultBoot?: Token<any> | ((metadata: T) => Token<any>)} [defaultBoot]
 * @param {MetadataAdapter} [adapter]
 * @param {MetadataExtends<T>} [metadataExtends]
 * @returns {IDIModuleDecorator<T>}
 */
function createDIModuleDecorator(name, defaultBuilder, defaultAnnoBuilder, defaultBoot, adapter, metadataExtends) {
    return core_1.createClassDecorator(name, function (args) {
        if (adapter) {
            adapter(args);
        }
    }, function (metadata) {
        if (metadataExtends) {
            metadataExtends(metadata);
        }
        if (!metadata.name && core_1.isClass(metadata.token)) {
            var isuglify = /^[a-z]$/.test(metadata.token.name);
            var classAnnations = core_1.lang.getClassAnnations(metadata.token);
            if (isuglify && classAnnations) {
                metadata.name = classAnnations.name;
            }
            else {
                metadata.name = metadata.token.name;
            }
        }
        metadata.decorType = name;
        if (defaultBuilder && !metadata.defaultBuilder) {
            metadata.defaultBuilder = defaultBuilder;
        }
        if (defaultAnnoBuilder && !metadata.defaultAnnoBuilder) {
            metadata.defaultAnnoBuilder = defaultAnnoBuilder;
        }
        if (!metadata.bootstrap && defaultBoot) {
            var defboot = core_1.isToken(defaultBoot) ? defaultBoot : defaultBoot(metadata);
            if (defboot) {
                metadata.bootstrap = defboot;
            }
        }
    });
}
exports.createDIModuleDecorator = createDIModuleDecorator;
/**
 * DIModule Decorator, definde class as DI module.
 *
 * @DIModule
 */
exports.DIModule = createDIModuleDecorator('DIModule', IModuleBuilder_1.ModuleBuilderToken, IAnnotationBuilder_1.AnnotationBuilderToken);

//# sourceMappingURL=../sourcemaps/decorators/DIModule.js.map
