"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var core_1 = require("@ts-ioc/core");
var DIModuleValidate_1 = require("./DIModuleValidate");
var utils_1 = require("../utils");
var InjectedModule_1 = require("./InjectedModule");
var ConfigureRegister_1 = require("../boot/ConfigureRegister");
var IConfigureManager_1 = require("../boot/IConfigureManager");
var IRunnableBuilder_1 = require("../boot/IRunnableBuilder");
/**
 * DIModule injector token.
 */
exports.DIModuleInjectorToken = new core_1.InjectModuleInjectorToken('@DIModule');
/**
 * DIModule injector.
 *
 * @export
 * @class DIModuleInjector
 * @extends {ModuleInjector}
 */
var DIModuleInjector = /** @class */ (function (_super) {
    tslib_1.__extends(DIModuleInjector, _super);
    function DIModuleInjector(validate) {
        return _super.call(this, validate) || this;
    }
    DIModuleInjector.prototype.setup = function (container, type) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.importModule(container, type)];
                    case 1:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    DIModuleInjector.prototype.syncSetup = function (container, type) {
        throw new Error('DIModule can not sync setup.');
    };
    DIModuleInjector.prototype.valid = function (container, type) {
        if (!this.validate) {
            return true;
        }
        return this.validate.valid(type);
    };
    DIModuleInjector.prototype.getMetaAccessor = function (container, token) {
        return container.getService(core_1.MetaAccessorToken, token);
    };
    DIModuleInjector.prototype.importModule = function (container, type) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var pools, decorator, accor, metaConfig, newContainer, exps, injMd;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        pools = container.get(utils_1.ContainerPoolToken);
                        decorator = this.validate.getDecorator();
                        accor = this.getMetaAccessor(container, decorator);
                        metaConfig = accor.getMetadata(type, container, undefined, decorator ? function (dec) { return dec === decorator; } : undefined);
                        newContainer = metaConfig.asRoot === true ? pools.getDefault() : pools.create(type, container);
                        newContainer.register(type);
                        return [4 /*yield*/, this.registerConfgureDepds(newContainer, metaConfig, type)];
                    case 1:
                        _a.sent();
                        return [4 /*yield*/, this.getConfigExports(newContainer, metaConfig)];
                    case 2:
                        exps = _a.sent();
                        injMd = new InjectedModule_1.InjectedModule(metaConfig.token || type, metaConfig, newContainer, type, exps);
                        container.bindProvider(new core_1.InjectReference(core_1.ProviderMap, type), exps);
                        container.bindProvider(new InjectedModule_1.InjectedModuleToken(type), injMd);
                        if (metaConfig.asRoot) {
                            return [2 /*return*/, injMd];
                        }
                        return [4 /*yield*/, this.registerConfigExports(container, newContainer, injMd)];
                    case 3:
                        _a.sent();
                        if (!!pools.isDefault(newContainer)) return [3 /*break*/, 5];
                        return [4 /*yield*/, this.registerConfigrue(newContainer, injMd)];
                    case 4:
                        _a.sent();
                        _a.label = 5;
                    case 5: return [2 /*return*/, injMd];
                }
            });
        });
    };
    DIModuleInjector.prototype.registerConfgureDepds = function (container, config, type) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!(core_1.isArray(config.imports) && config.imports.length)) return [3 /*break*/, 2];
                        return [4 /*yield*/, container.loadModule.apply(container, config.imports)];
                    case 1:
                        _a.sent();
                        _a.label = 2;
                    case 2: return [2 /*return*/];
                }
            });
        });
    };
    DIModuleInjector.prototype.getConfigExports = function (container, config) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var parser, map, builder, exptypes;
            return tslib_1.__generator(this, function (_a) {
                parser = container.getProviderParser();
                map = parser.parse.apply(parser, config.providers || []);
                // bind module providers
                container.bindProviders(map);
                builder = container.getBuilder();
                exptypes = [].concat.apply([], builder.loader.getTypes(config.exports || []));
                exptypes.forEach(function (ty) {
                    var classPd = container.resolveValue(new core_1.InjectClassProvidesToken(ty));
                    map.add(ty, function () {
                        var pds = [];
                        for (var _i = 0; _i < arguments.length; _i++) {
                            pds[_i] = arguments[_i];
                        }
                        return container.resolve.apply(container, [ty, core_1.ResoveWay.nodes].concat(pds));
                    });
                    if (classPd && core_1.isArray(classPd.provides) && classPd.provides.length) {
                        classPd.provides.forEach(function (p) {
                            if (!map.has(p)) {
                                map.add(p, function () {
                                    var pds = [];
                                    for (var _i = 0; _i < arguments.length; _i++) {
                                        pds[_i] = arguments[_i];
                                    }
                                    return container.resolve.apply(container, [p, core_1.ResoveWay.nodes].concat(pds));
                                });
                            }
                        });
                    }
                });
                return [2 /*return*/, map];
            });
        });
    };
    DIModuleInjector.prototype.registerConfigExports = function (container, newContainer, injMd) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var chain;
            return tslib_1.__generator(this, function (_a) {
                if (container === newContainer) {
                    return [2 /*return*/, container];
                }
                if (injMd) {
                    chain = container.getResolvers();
                    chain.next(injMd);
                }
                return [2 /*return*/, container];
            });
        });
    };
    DIModuleInjector.prototype.registerConfigrue = function (newContainer, injMd) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var registers, mgr, appConfig_1;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        registers = [];
                        newContainer.iteratorServices(function (serType, fac, resolver) {
                            registers.push({
                                resolver: resolver,
                                serType: serType
                            });
                        }, ConfigureRegister_1.ConfigureRegister, injMd.type, true, core_1.ResoveWay.current);
                        if (!registers.length) return [3 /*break*/, 3];
                        mgr = newContainer.get(IConfigureManager_1.ConfigureMgrToken);
                        return [4 /*yield*/, mgr.getConfig()];
                    case 1:
                        appConfig_1 = _a.sent();
                        return [4 /*yield*/, Promise.all(registers.map(function (ser) { return ser.resolver.resolve(ser.serType).register(appConfig_1, ser.resolver.resolve(IRunnableBuilder_1.CurrentRunnableBuilderToken)); }))];
                    case 2:
                        _a.sent();
                        _a.label = 3;
                    case 3: return [2 /*return*/];
                }
            });
        });
    };
    DIModuleInjector.getClassAnnations = function () {
        return { "name": "DIModuleInjector", "params": { "constructor": ["validate"], "setup": ["container", "type"], "syncSetup": ["container", "type"], "valid": ["container", "type"], "getMetaAccessor": ["container", "token"], "importModule": ["container", "type"], "registerConfgureDepds": ["container", "config", "type"], "getConfigExports": ["container", "config"], "registerConfigExports": ["container", "newContainer", "injMd"], "registerConfigrue": ["newContainer", "injMd"] } };
    };
    DIModuleInjector = tslib_1.__decorate([
        core_1.Singleton(exports.DIModuleInjectorToken),
        tslib_1.__param(0, core_1.Inject(DIModuleValidate_1.DIModuleValidateToken)),
        tslib_1.__metadata("design:paramtypes", [Object])
    ], DIModuleInjector);
    return DIModuleInjector;
}(core_1.ModuleInjector));
exports.DIModuleInjector = DIModuleInjector;

//# sourceMappingURL=../sourcemaps/modules/DIModuleInjector.js.map
