"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var core_1 = require("@ts-ioc/core");
/**
 * injected module.
 *
 * @export
 * @class InjectedModule
 * @template T
 */
var InjectedModule = /** @class */ (function () {
    function InjectedModule(token, config, container, type, exports) {
        this.token = token;
        this.config = config;
        this.container = container;
        this.type = type;
        this.exports = exports;
    }
    InjectedModule.prototype.getProviderMap = function () {
        if (!this.exports) {
            this.exports = this.container;
        }
        return this.exports;
    };
    Object.defineProperty(InjectedModule.prototype, "size", {
        get: function () {
            return this.getProviderMap().size;
        },
        enumerable: true,
        configurable: true
    });
    InjectedModule.prototype.resolve = function (token, resway) {
        var providers = [];
        for (var _i = 2; _i < arguments.length; _i++) {
            providers[_i - 2] = arguments[_i];
        }
        var _a;
        var way;
        if (core_1.isNumber(resway)) {
            way = resway;
        }
        else {
            way = core_1.ResoveWay.all;
            if (resway) {
                providers.unshift(resway);
            }
        }
        var pdr = this.getProviderMap();
        if (pdr && (way & core_1.ResoveWay.current) && pdr.has(token)) {
            return pdr.resolve.apply(pdr, [token, core_1.ResoveWay.current].concat(providers));
        }
        if (pdr !== this.container && (way & core_1.ResoveWay.traverse)) {
            return (_a = this.container).resolve.apply(_a, [token, core_1.ResoveWay.traverse].concat(providers));
        }
        return null;
    };
    InjectedModule.prototype.has = function (token, aliasOrway) {
        var key = this.container.getTokenKey(token, core_1.isString(aliasOrway) ? aliasOrway : null);
        var pdr = this.getProviderMap();
        var resway = core_1.isNumber(aliasOrway) ? aliasOrway : core_1.ResoveWay.all;
        if (pdr && (resway & core_1.ResoveWay.current) && pdr.has(key)) {
            return true;
        }
        if (pdr !== this.container && (resway & core_1.ResoveWay.traverse)) {
            return this.container.has(key, core_1.ResoveWay.traverse);
        }
        return false;
    };
    InjectedModule.prototype.getTokenImpl = function (token, resway) {
        return this.container.getTokenImpl(token, resway || core_1.ResoveWay.nodes);
    };
    InjectedModule.prototype.unregister = function (token, resway) {
        var key = this.container.getTokenKey(token);
        var pdr = this.getProviderMap();
        resway = core_1.isNumber(resway) ? resway : core_1.ResoveWay.nodes;
        if (pdr && (resway & core_1.ResoveWay.current)) {
            pdr.unregister(key, core_1.ResoveWay.current);
        }
        if (resway & core_1.ResoveWay.traverse) {
            this.container.unregister(key, core_1.ResoveWay.nodes);
        }
        return this;
    };
    InjectedModule.prototype.forEach = function (callbackfn) {
        var _this = this;
        var pdr = this.getProviderMap();
        if (pdr) {
            if (pdr.forEach(function (fac, tk) {
                if (core_1.isToken(tk)) {
                    return callbackfn(tk, fac, pdr);
                }
            }) === false) {
                return false;
            }
        }
        return this.container.forEach(function (tk, fac) {
            return callbackfn(tk, fac, _this.container);
        });
    };
    InjectedModule.getClassAnnations = function () {
        return { "name": "InjectedModule", "params": { "constructor": ["token", "config", "container", "type", "exports"], "getProviderMap": [], "resolve": ["token", "resway", "providers"], "has": ["token", "aliasOrway"], "getTokenImpl": ["token", "resway"], "unregister": ["token", "resway"], "forEach": ["callbackfn"] } };
    };
    return InjectedModule;
}());
exports.InjectedModule = InjectedModule;
/**
 * Injected Module Token.
 *
 * @export
 * @class InjectModuleMetaConfigToken
 * @extends {Registration<Type<T>>}
 * @template T
 */
var InjectedModuleToken = /** @class */ (function (_super) {
    tslib_1.__extends(InjectedModuleToken, _super);
    function InjectedModuleToken(type) {
        return _super.call(this, type, 'InjectedModule') || this;
    }
    InjectedModuleToken.getClassAnnations = function () {
        return { "name": "InjectedModuleToken", "params": { "constructor": ["type"] } };
    };
    return InjectedModuleToken;
}(core_1.Registration));
exports.InjectedModuleToken = InjectedModuleToken;

//# sourceMappingURL=../sourcemaps/modules/InjectedModule.js.map
