import 'reflect-metadata';
import { IContainer, Token, Registration, IMetaAccessor, Type } from '@ts-ioc/core';
import { IModuleBuilder, ModuleEnv, BootOptions } from './IModuleBuilder';
import { ModuleConfigure, ModuleConfig } from './ModuleConfigure';
import { ContainerPool } from '../utils';
import { Runnable } from '../runnable';
import { IAnnotationBuilder } from '../annotations';
import { InjectedModule } from './InjectedModule';
/**
 * inject module load token.
 *
 * @export
 * @class InjectModuleLoadToken
 * @extends {Registration<T>}
 * @template T
 */
export declare class InjectModuleLoadToken<T> extends Registration<T> {
    constructor(token: Token<T>);
    static getClassAnnations(): any;
}
/**
 * module builder
 *
 * @export
 * @class ModuleBuilder
 * @implements {IModuleBuilder}
 * @template T
 */
export declare class ModuleBuilder<T> implements IModuleBuilder<T> {
    protected pools: ContainerPool;
    constructor();
    getPools(): ContainerPool;
    getInjectedModule<T>(type: Type<T>): InjectedModule<T>;
    /**
     * get metadata accessor.
     *
     * @param {IContainer} container
     * @param {Token<any>} token
     * @param {ModuleConfigure} [config]
     * @returns {IMetaAccessor<any>}
     * @memberof ModuleBuilder
     */
    getMetaAccessor(container: IContainer, token: Token<any> | ModuleConfigure, config?: ModuleConfigure): IMetaAccessor<any>;
    /**
     * load module.
     *
     * @param {(Token<T> | ModuleConfigure)} token
     * @param {(ModuleConfig<T> | BootOptions<T>)} [config]
     * @param {BootOptions<T>} [options]
     * @returns {Promise<InjectedModule<T>>}
     * @memberof ModuleBuilder
     */
    load(token: Token<T> | ModuleConfigure, config?: ModuleConfig<T> | BootOptions<T>, options?: BootOptions<T>): Promise<InjectedModule<T>>;
    /**
     * bootstrap with module and config.
     *
     * @param {(Token<T> | ModuleConfig<T>)} token
     * @param {(ModuleConfig<T> | BootOptions<T>)} [config]
     * @param {BootOptions<T>} [options]
     * @returns {Promise<Runnable<T>>}
     * @memberof ModuleBuilder
     */
    bootstrap(token: Token<T> | ModuleConfig<T>, config?: ModuleConfig<T> | BootOptions<T>, options?: BootOptions<T>): Promise<Runnable<T>>;
    protected vaildParams(token: Token<T> | ModuleConfig<T>, config?: ModuleConfig<T> | BootOptions<T>, options?: BootOptions<T>): {
        token?: Token<T>;
        config?: ModuleConfig<T>;
        options?: BootOptions<T>;
    };
    protected loadViaToken(token: Token<T>, parent: IContainer): Promise<InjectedModule<T>>;
    protected loadViaConfig(config: ModuleConfigure, parent: IContainer): Promise<InjectedModule<T>>;
    protected import(token: Token<T>, parent: IContainer): Promise<InjectedModule<T>>;
    protected getParentContainer(env?: ModuleEnv): Promise<IContainer>;
    protected getAnnoBuilder(container: IContainer, token: Token<any>, config: ModuleConfigure): IAnnotationBuilder<any>;
    static getClassAnnations(): any;
}
