"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
require("reflect-metadata");
var core_1 = require("@ts-ioc/core");
var IModuleBuilder_1 = require("./IModuleBuilder");
var utils_1 = require("../utils");
var annotations_1 = require("../annotations");
var InjectedModule_1 = require("./InjectedModule");
/**
 * inject module load token.
 *
 * @export
 * @class InjectModuleLoadToken
 * @extends {Registration<T>}
 * @template T
 */
var InjectModuleLoadToken = /** @class */ (function (_super) {
    tslib_1.__extends(InjectModuleLoadToken, _super);
    function InjectModuleLoadToken(token) {
        return _super.call(this, token, 'module_loader') || this;
    }
    InjectModuleLoadToken.getClassAnnations = function () {
        return { "name": "InjectModuleLoadToken", "params": { "constructor": ["token"] } };
    };
    return InjectModuleLoadToken;
}(core_1.Registration));
exports.InjectModuleLoadToken = InjectModuleLoadToken;
/**
 * module builder
 *
 * @export
 * @class ModuleBuilder
 * @implements {IModuleBuilder}
 * @template T
 */
var ModuleBuilder = /** @class */ (function () {
    function ModuleBuilder() {
    }
    ModuleBuilder.prototype.getPools = function () {
        return this.pools;
    };
    ModuleBuilder.prototype.getInjectedModule = function (type) {
        return this.getPools().getDefault().get(new InjectedModule_1.InjectedModuleToken(type));
    };
    /**
     * get metadata accessor.
     *
     * @param {IContainer} container
     * @param {Token<any>} token
     * @param {ModuleConfigure} [config]
     * @returns {IMetaAccessor<any>}
     * @memberof ModuleBuilder
     */
    ModuleBuilder.prototype.getMetaAccessor = function (container, token, config) {
        var mtk;
        if (core_1.isToken(token)) {
            mtk = token;
        }
        else {
            config = token;
        }
        return container.getService(core_1.MetaAccessorToken, mtk ? [mtk, core_1.lang.getClass(this)] : core_1.lang.getClass(this), function (tk) { return new core_1.InjectMetaAccessorToken(tk); }, config ? (config.defaultMetaAccessor || core_1.MetaAccessorToken) : core_1.MetaAccessorToken);
    };
    /**
     * load module.
     *
     * @param {(Token<T> | ModuleConfigure)} token
     * @param {(ModuleConfig<T> | BootOptions<T>)} [config]
     * @param {BootOptions<T>} [options]
     * @returns {Promise<InjectedModule<T>>}
     * @memberof ModuleBuilder
     */
    ModuleBuilder.prototype.load = function (token, config, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var params, env, parent, injmd;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        params = this.vaildParams(token, config, options);
                        env = params.options ? params.options.env : null;
                        if (env instanceof InjectedModule_1.InjectedModule) {
                            return [2 /*return*/, env];
                        }
                        return [4 /*yield*/, this.getParentContainer(env)];
                    case 1:
                        parent = _a.sent();
                        if (!params.token) return [3 /*break*/, 5];
                        return [4 /*yield*/, this.loadViaToken(params.token, parent)];
                    case 2:
                        injmd = _a.sent();
                        if (!params.config) return [3 /*break*/, 4];
                        return [4 /*yield*/, this.loadViaConfig(params.config, injmd.container || parent)];
                    case 3:
                        _a.sent();
                        _a.label = 4;
                    case 4: return [2 /*return*/, injmd];
                    case 5: return [4 /*yield*/, this.loadViaConfig(params.config, parent)];
                    case 6: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    /**
     * bootstrap with module and config.
     *
     * @param {(Token<T> | ModuleConfig<T>)} token
     * @param {(ModuleConfig<T> | BootOptions<T>)} [config]
     * @param {BootOptions<T>} [options]
     * @returns {Promise<Runnable<T>>}
     * @memberof ModuleBuilder
     */
    ModuleBuilder.prototype.bootstrap = function (token, config, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var params, injmdl, cfg, tk, container, accessor, bootToken, anBuilder, mdBuilder;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        params = this.vaildParams(token, config, options);
                        options = params.options || {};
                        if (!params.token) return [3 /*break*/, 2];
                        return [4 /*yield*/, this.load(params.token, params.config, options)];
                    case 1:
                        injmdl = _a.sent();
                        return [3 /*break*/, 4];
                    case 2: return [4 /*yield*/, this.load(params.config, options)];
                    case 3:
                        injmdl = _a.sent();
                        _a.label = 4;
                    case 4:
                        options.env = injmdl;
                        cfg = injmdl.config;
                        tk = injmdl.token || injmdl.type;
                        container = injmdl.container;
                        accessor = this.getMetaAccessor(container, tk);
                        bootToken = accessor.getBootToken(cfg, container);
                        if (!bootToken) return [3 /*break*/, 6];
                        anBuilder = this.getAnnoBuilder(container, bootToken, cfg);
                        return [4 /*yield*/, anBuilder.boot(bootToken, cfg, options)];
                    case 5: return [2 /*return*/, _a.sent()];
                    case 6:
                        mdBuilder = this.getAnnoBuilder(container, tk, cfg);
                        return [4 /*yield*/, mdBuilder.boot(tk, cfg, options)];
                    case 7: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    ModuleBuilder.prototype.vaildParams = function (token, config, options) {
        var params = {};
        if (core_1.isToken(token)) {
            params.token = token;
            if (core_1.isUndefined(options)) {
                params.options = config;
            }
            else {
                params.config = config;
                params.options = options;
            }
        }
        else {
            params.config = token;
            params.options = config;
        }
        return params;
    };
    ModuleBuilder.prototype.loadViaToken = function (token, parent) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var injmdl, cfg;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.import(token, parent)];
                    case 1:
                        injmdl = _a.sent();
                        if (!injmdl) {
                            cfg = this.getMetaAccessor(parent, token).getMetadata(token, parent);
                            injmdl = new InjectedModule_1.InjectedModule(token, cfg, parent);
                        }
                        return [2 /*return*/, injmdl];
                }
            });
        });
    };
    ModuleBuilder.prototype.loadViaConfig = function (config, parent) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var injmd, token, cfg, parser, pdrmap_1;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        injmd = null;
                        token = this.getMetaAccessor(parent, config).getToken(config, parent);
                        if (!token) return [3 /*break*/, 2];
                        return [4 /*yield*/, this.import(token, parent)];
                    case 1:
                        injmd = _a.sent();
                        if (!injmd) {
                            cfg = this.getMetaAccessor(parent, token).getMetadata(token, parent, config);
                            injmd = new InjectedModule_1.InjectedModule(token, cfg, parent);
                        }
                        return [3 /*break*/, 3];
                    case 2:
                        token = null;
                        _a.label = 3;
                    case 3:
                        if (!(core_1.isArray(config.imports) && config.imports.length)) return [3 /*break*/, 5];
                        return [4 /*yield*/, parent.loadModule.apply(parent, config.imports)];
                    case 4:
                        _a.sent();
                        _a.label = 5;
                    case 5:
                        if (core_1.isArray(config.providers) && config.providers.length) {
                            parser = parent.getProviderParser();
                            pdrmap_1 = parser.parse.apply(parser, config.providers);
                            pdrmap_1.keys().forEach(function (key) {
                                core_1.isToken(key) && parent.bindProvider(key, function () {
                                    var providers = [];
                                    for (var _i = 0; _i < arguments.length; _i++) {
                                        providers[_i] = arguments[_i];
                                    }
                                    return pdrmap_1.resolve.apply(pdrmap_1, [key].concat(providers));
                                });
                            });
                        }
                        if (injmd) {
                            injmd.config = core_1.lang.assign({}, injmd.config, config);
                        }
                        else {
                            injmd = new InjectedModule_1.InjectedModule(token, config, parent);
                        }
                        return [2 /*return*/, injmd];
                }
            });
        });
    };
    ModuleBuilder.prototype.import = function (token, parent) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var type, key;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        type = core_1.isClass(token) ? token : parent.getTokenImpl(token);
                        if (!core_1.isClass(type)) return [3 /*break*/, 3];
                        key = new InjectedModule_1.InjectedModuleToken(type);
                        if (!parent.has(key)) return [3 /*break*/, 1];
                        return [2 /*return*/, parent.get(key)];
                    case 1: return [4 /*yield*/, parent.loadModule(type)];
                    case 2:
                        _a.sent();
                        if (parent.has(key)) {
                            return [2 /*return*/, parent.get(key)];
                        }
                        _a.label = 3;
                    case 3: return [2 /*return*/, null];
                }
            });
        });
    };
    ModuleBuilder.prototype.getParentContainer = function (env) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var parent;
            return tslib_1.__generator(this, function (_a) {
                if (env) {
                    if (env instanceof core_1.Container) {
                        parent = env;
                    }
                    else if (env instanceof InjectedModule_1.InjectedModule) {
                        parent = env.container.parent;
                    }
                }
                if (!parent) {
                    parent = this.getPools().getDefault();
                }
                return [2 /*return*/, parent];
            });
        });
    };
    ModuleBuilder.prototype.getAnnoBuilder = function (container, token, config) {
        var builder;
        if (core_1.isClass(config.annoBuilder)) {
            if (!container.has(config.annoBuilder)) {
                container.register(config.annoBuilder);
            }
        }
        if (core_1.isToken(config.annoBuilder)) {
            builder = container.resolve(config.annoBuilder);
        }
        else if (config.annoBuilder instanceof annotations_1.AnnotationBuilder) {
            builder = config.annoBuilder;
        }
        if (!builder && token) {
            builder = container.getService(annotations_1.AnnotationBuilderToken, token, function (tk) { return new annotations_1.InjectAnnotationBuilder(tk); }, config.defaultAnnoBuilder || annotations_1.AnnotationBuilderToken);
        }
        if (builder) {
            builder.container = container;
        }
        return builder;
    };
    ModuleBuilder.getClassAnnations = function () {
        return { "name": "ModuleBuilder", "params": { "constructor": [], "getPools": [], "getInjectedModule": ["type"], "getMetaAccessor": ["container", "token", "config"], "load": ["token", "config", "options"], "bootstrap": ["token", "config", "options"], "vaildParams": ["token", "config", "options"], "loadViaToken": ["token", "parent"], "loadViaConfig": ["config", "parent"], "import": ["token", "parent"], "getParentContainer": ["env"], "getAnnoBuilder": ["container", "token", "config"] } };
    };
    tslib_1.__decorate([
        core_1.Inject(utils_1.ContainerPoolToken),
        tslib_1.__metadata("design:type", utils_1.ContainerPool)
    ], ModuleBuilder.prototype, "pools", void 0);
    ModuleBuilder = tslib_1.__decorate([
        core_1.Singleton(IModuleBuilder_1.ModuleBuilderToken),
        tslib_1.__metadata("design:paramtypes", [])
    ], ModuleBuilder);
    return ModuleBuilder;
}());
exports.ModuleBuilder = ModuleBuilder;

//# sourceMappingURL=../sourcemaps/modules/ModuleBuilder.js.map
