"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var core_1 = require("@ts-ioc/core");
var rootContainer = '__ioc_root_container';
/**
 * container pool
 *
 * @export
 * @class ContainerPool
 */
var ContainerPool = /** @class */ (function () {
    function ContainerPool(containerBuilder) {
        this.containerBuilder = containerBuilder;
        this.pools = new Map();
    }
    ContainerPool.prototype.createContainer = function () {
        return this.containerBuilder.create();
    };
    ContainerPool.prototype.getTokenKey = function (token) {
        if (token instanceof core_1.Registration) {
            return token.toString();
        }
        return token;
    };
    ContainerPool.prototype.isDefault = function (container) {
        return container === this._default;
    };
    ContainerPool.prototype.hasDefault = function () {
        return !!this._default;
    };
    ContainerPool.prototype.getDefault = function () {
        if (!this._default) {
            this._default = this.createContainer();
            this.pools.set(rootContainer, [this._default]);
        }
        return this._default;
    };
    ContainerPool.prototype.set = function (token, container) {
        var key = this.getTokenKey(token);
        if (this.pools.has(key)) {
            this.pools.get(key).push(container);
        }
        else {
            this.pools.set(key, [container]);
        }
    };
    ContainerPool.prototype.get = function (token) {
        var key = this.getTokenKey(token);
        if (!this.has(key)) {
            return null;
        }
        return this.pools.get(token);
    };
    ContainerPool.prototype.has = function (token) {
        return this.pools.has(this.getTokenKey(token));
    };
    ContainerPool.prototype.create = function (token, parent) {
        parent = parent || this.getDefault();
        var container = parent.getBuilder().create();
        this.setParent(container, parent);
        this.set(token, container);
        return container;
    };
    ContainerPool.prototype.setParent = function (container, parent) {
        if (this.isDefault(container)) {
            return;
        }
        parent = parent || this.getDefault();
        if (parent && parent !== container) {
            if (container.parent) {
                container.parent.children.splice(container.parent.children.indexOf(container), 1);
            }
            parent.children.push(container);
            container.parent = parent;
        }
    };
    ContainerPool.prototype.keys = function () {
        return Array.from(this.pools.keys());
    };
    ContainerPool.prototype.values = function () {
        return Array.from(this.pools.values()).reduce(function (p, c) { return p.concat(c); }, []);
    };
    ContainerPool.prototype.iterator = function (express, root) {
        root = root || this.getDefault();
        express(root);
        this.iteratorChildren(express, root.children);
    };
    ContainerPool.prototype.iteratorChildren = function (express, children) {
        var _this = this;
        if (children && children.length) {
            children.forEach(function (c) {
                express(c);
                _this.iteratorChildren(express, c.children);
            });
        }
    };
    ContainerPool.getClassAnnations = function () {
        return { "name": "ContainerPool", "params": { "constructor": ["containerBuilder"], "createContainer": [], "getTokenKey": ["token"], "isDefault": ["container"], "hasDefault": [], "getDefault": [], "set": ["token", "container"], "get": ["token"], "has": ["token"], "create": ["token", "parent"], "setParent": ["container", "parent"], "keys": [], "values": [], "iterator": ["express", "root"], "iteratorChildren": ["express", "children"] } };
    };
    return ContainerPool;
}());
exports.ContainerPool = ContainerPool;
/**
 *  container pool token.
 */
exports.ContainerPoolToken = new core_1.InjectToken('DI_ContainerPool');

//# sourceMappingURL=../sourcemaps/utils/ContainerPool.js.map
