"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var core_1 = require("@ts-ioc/core");
/**
 * custom events.
 *
 * @export
 * @class Events
 */
var Events = /** @class */ (function () {
    function Events() {
        this.eventsMap = {};
    }
    Events.prototype.on = function (name, event) {
        this.eventsMap[name] = this.eventsMap[name] || [];
        if (this.eventsMap[name].indexOf(event) < 0) {
            this.eventsMap[name].push(event);
        }
        return this;
    };
    Events.prototype.once = function (name, event) {
        var _this = this;
        var onceEvent = function () {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }
            event.apply(void 0, args);
            _this.off(name, onceEvent);
        };
        this.on(name, onceEvent);
    };
    Events.prototype.off = function (name, event) {
        if (this.eventsMap[name]) {
            if (event) {
                this.eventsMap[name].splice(this.eventsMap[name].indexOf(event), 1);
            }
            else {
                delete this.eventsMap[name];
            }
        }
        return this;
    };
    Events.prototype.emit = function (name) {
        var args = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            args[_i - 1] = arguments[_i];
        }
        var events = this.eventsMap[name];
        if (core_1.isArray(events)) {
            events.forEach(function (ev) {
                ev.apply(void 0, args);
            });
        }
    };
    Events.getClassAnnations = function () {
        return { "name": "Events", "params": { "constructor": [], "on": ["name", "event"], "once": ["name", "event"], "off": ["name", "event"], "emit": ["name", "args"] } };
    };
    return Events;
}());
exports.Events = Events;

//# sourceMappingURL=../sourcemaps/utils/Events.js.map
